#include <cvirte.h>    /* Needed if linking in external compiler; harmless otherwise */
#include <userint.h>
#include <formatio.h>
#include <utility.h>
#include <ansi_c.h>
#include "atedemo.h"

void InitializeTests (void);
void InitializeGUI (void);
void OutputResults(int TestNum);
int RunTest (int TestNum);

int datafile;
double high[10], low[10], value[10];
char *result[10], logfilename[100];
int handle, change, loop, LogData = 1, CurrentTest;
char Batch[10], PartNumber[10], SerialNumber[10], Revision[10];
int shutdownRequested = 0;

int main (int argc, char *argv[])
{
    if (InitCVIRTE (0, argv, 0) == 0)    /* Needed if linking in external compiler; harmless otherwise */
        return -1;    /* out of memory */
    
    handle = LoadPanel (0, "atedemo.uir", PANEL);
    change = LoadPanel (0, "atedemo.uir", INPUT);
    loop = LoadPanel (0, "atedemo.uir", LOOP);
    InitializeTests ();
    InitializeGUI();
    DisplayPanel (handle);
    
    RunUserInterface ();
    CloseFile (datafile);

    return 0;
}


int CVICALLBACK ChangeUUT(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    if (event == EVENT_COMMIT) {
        DisplayPanel (change);
        }
    return(0);
}


int CVICALLBACK UpdateUUT(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    char setupstr[200];

    if (event == EVENT_COMMIT) {
        GetCtrlVal (change, INPUT_PART, PartNumber);
        GetCtrlVal (change, INPUT_SERIAL, SerialNumber);
        GetCtrlVal (change, INPUT_REVISION, Revision);
        GetCtrlVal (change, INPUT_BATCH, Batch);

        SetCtrlVal (handle, PANEL_PART, PartNumber);
        SetCtrlVal (handle, PANEL_SERIAL, SerialNumber);
        SetCtrlVal (handle, PANEL_REVISION, Revision);
        SetCtrlVal (handle, PANEL_BATCH, Batch);

        sprintf (setupstr, "\n------------\n Part Number:  %s\n Serial Number:  %s\n Revision:  %s\n Batch:  %s\n------------\n\n",
                    PartNumber, SerialNumber, Revision, Batch);
        WriteFile (datafile, setupstr, StringLength(setupstr));

        HidePanel (change);
        }
    return(0);
}


int CVICALLBACK LogResults(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    if (event == EVENT_COMMIT) {
        GetCtrlVal (handle, PANEL_LOG, &LogData);
        }
    return(0);
}


int CVICALLBACK Loop(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    if (event == EVENT_COMMIT)
        DisplayPanel (loop);
    return(0);
}


int CVICALLBACK RunLoop(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    int TestNum, iterations, i;

    if (event == EVENT_COMMIT) {
        HidePanel (loop);
        ProcessSystemEvents();
        GetCtrlVal (loop, LOOP_TESTNUMBER, &TestNum);
        GetCtrlVal (loop, LOOP_LOOPNUMBER, &iterations);
        for (i=0;i<iterations;i++){
            if (RunTest(TestNum) == 1) {
                SetCtrlAttribute (handle, PANEL_FAIL, ATTR_VISIBLE, 0);
                SetCtrlAttribute (handle, PANEL_PASS, ATTR_VISIBLE, 1);
                Delay (0.5);
                }
            else {
                SetCtrlAttribute (handle, PANEL_FAIL, ATTR_VISIBLE, 1);
                Delay (0.5);
                }
            ProcessSystemEvents();
            OutputResults(TestNum);
            SetCtrlAttribute (handle, PANEL_PASS, ATTR_VISIBLE, 0);
            SetCtrlAttribute (handle, PANEL_FAIL, ATTR_VISIBLE, 0);
            Delay (0.5);
            ProcessSystemEvents();
            if (shutdownRequested) {
                break;
                }
            }
        }
    return(0);
}


int CVICALLBACK RunSequence(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    int i;

    if (event == EVENT_COMMIT) {
        SetCtrlAttribute (handle, PANEL_LOOP, ATTR_DIMMED, 1);
        for (i=0;i<10;i++){
            if (RunTest(i) == 1) {
                SetCtrlAttribute (handle, PANEL_FAIL, ATTR_VISIBLE, 0);
                SetCtrlAttribute (handle, PANEL_PASS, ATTR_VISIBLE, 1);
                Delay (0.5);
                }
            else {
                SetCtrlAttribute (handle, PANEL_FAIL, ATTR_VISIBLE, 1);
                Delay (0.5);
                }
            ProcessSystemEvents();
            OutputResults(i);
            SetCtrlAttribute (handle, PANEL_PASS, ATTR_VISIBLE, 0);
            SetCtrlAttribute (handle, PANEL_FAIL, ATTR_VISIBLE, 0);
            Delay (0.5);
            ProcessSystemEvents();
            if (shutdownRequested) {
                break;
                }
            }
        }
    SetCtrlAttribute (handle, PANEL_LOOP, ATTR_DIMMED, 0);
    return(0);
}


int RunTest (int TestNum)
{
    value[TestNum] = (rand()/32767.0) * 7.0;

    if ((value[TestNum] <= high[TestNum]) && (value[TestNum] >= low[TestNum])) {
        result[TestNum] = "PASS";
        return (1);
        }
    else {
        result[TestNum] = "FAIL";
        return (0);
        }
}


void OutputResults(int TestNum)
{
    int log;
    char resultstr[500];

    GetCtrlVal (handle, PANEL_LOG, &log);
    sprintf (resultstr, "  Test #%d\n  High Limit:  %f\n  Low Limit:  %f\n  Value:  %f\n  Result:             %s\n\n",
                TestNum, high[TestNum], low[TestNum], value[TestNum], result[TestNum]);
    InsertTextBoxLine (handle, PANEL_RESULTS, 0, resultstr);
    if (LogData) {
        WriteFile (datafile, resultstr, StringLength(resultstr));
        }
}


int CVICALLBACK ViewDataFile(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    char tempstr[100];

    if (event == EVENT_COMMIT) {
        CloseFile (datafile);
        sprintf (tempstr, "Notepad.exe %s", logfilename);
        LaunchExecutable (tempstr);
        datafile = OpenFile (logfilename, VAL_WRITE_ONLY, VAL_APPEND,
                             VAL_ASCII);
        }
    return(0);
}


void InitializeGUI ()
{
    char setupstr[200], dirname[MAX_PATHNAME_LEN];

    SetCtrlAttribute (handle, PANEL_PASS, ATTR_VISIBLE, 0);
    SetCtrlAttribute (handle, PANEL_FAIL, ATTR_VISIBLE, 0);
    GetProjectDir (dirname);
    sprintf (logfilename, "%s\\results.log", dirname);
    datafile = OpenFile (logfilename, VAL_WRITE_ONLY, VAL_TRUNCATE,
                         VAL_ASCII);
    sprintf(setupstr, "LabWindows/CVI Test Sequencer Demo \n-------------------------------\n Date : %s    Time %s \n\n", DateStr(), TimeStr());
    WriteFile (datafile, setupstr, StringLength (setupstr));
}


int CVICALLBACK Shutdown (int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    if (event == EVENT_COMMIT) {
        shutdownRequested = 1;
        QuitUserInterface (0);
        }
    return(0);
}


void InitializeTests ()
{
    high[0] = 5.0;
    high[1] = 4.5;
    high[2] = 4.0;
    high[3] = 5.0;
    high[4] = 4.7;
    high[5] = 4.33;
    high[6] = 2.5;
    high[7] = 5.0;
    high[8] = 3.5;
    high[9] = 5.0;

    low[0] = 2.5;
    low[1] = 2.5;
    low[2] = 2.5;
    low[3] = 2.0;
    low[4] = 2.5;
    low[5] = 2.5;
    low[6] = 0.5;
    low[7] = 2.5;
    low[8] = 1.5;
    low[9] = 2.0;
}
