#include <cvirte.h>    /* Needed if linking in external compiler; harmless otherwise */
#include <userint.h>
#include <analysis.h>
#include <ansi_c.h>
#include "freqresp.h"

void GetParameters(void);

int handle, steps;
double inc, startfreq, stopfreq, amp= 10.0;
double input_array[1024], output_array[1024], freq_array[100], volt_array[100], db_array[100];


int main (int argc, char *argv[])
{
    if (InitCVIRTE (0, argv, 0) == 0)    /* Needed if linking in external compiler; harmless otherwise */
        return -1;    /* out of memory */
    
    handle = LoadPanel (0, "freqresp.uir", PANEL);
    DisplayPanel (handle);
    RunUserInterface ();

    return 0;
}

int CVICALLBACK CurveFit(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    double fit_array[100], coef[100], err;

    if (event == EVENT_COMMIT) {
        PolyFit (freq_array, db_array, steps, 5, fit_array, coef, &err);
        PlotXY (handle, PANEL_GRAPH, freq_array, fit_array, steps, VAL_DOUBLE,
            VAL_DOUBLE, VAL_THIN_LINE, VAL_EMPTY_SQUARE, VAL_SOLID, 1,
            VAL_WHITE);
        }

    return(0);
}

int CVICALLBACK RunTest(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    int volts=0, i;

    if (event == EVENT_COMMIT) {
        GetParameters();
        DeleteGraphPlot (handle, PANEL_GRAPH, -1, 1);
        for (i=0;i<steps;i++) {
            SinePattern (1024, amp, 0.0, freq_array[i], input_array);
            Bw_BPF (input_array, 1024, 100000, 200, 600, 5, output_array);

            RMS (output_array, 1024, &volt_array[i]);
            volt_array[i] = (volt_array[i] * sqrt(2.0)) / amp;

            db_array[i] = 20.0 * log10(volt_array[i]);
            PlotPoint (handle, PANEL_GRAPH, freq_array[i] * (100000.0/1024.0), db_array[i],
                   VAL_EMPTY_SQUARE, VAL_RED);
            SetCtrlVal (handle, PANEL_CURRENT, freq_array[i] * (100000.0/1024.0));
            }
        LinEv1D (freq_array, steps, (100000.0/1024.0), 0.0, freq_array);

        PlotXY (handle, PANEL_GRAPH, freq_array, db_array, steps, VAL_DOUBLE,
                    VAL_DOUBLE, VAL_THIN_LINE, VAL_EMPTY_SQUARE, VAL_SOLID, 1,
                    VAL_YELLOW);

        SetCtrlAttribute(handle, PANEL_CURVEFIT, ATTR_DIMMED, 0);
        }
        return(0);
}


void GetParameters()
{
    int i;

    GetCtrlVal (handle, PANEL_STARTFREQ, &startfreq);
    GetCtrlVal (handle, PANEL_STOPFREQ, &stopfreq);
    GetCtrlVal (handle, PANEL_STEPS, &steps);
    inc = (stopfreq - startfreq)/(steps - 1);
    for (i=0;i<steps;i++)
        freq_array[i] = (startfreq + (i * inc)) * (1024.0/ 100000.0);

}


int CVICALLBACK Shutdown(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    if (event == EVENT_COMMIT)
        QuitUserInterface(0);
    return(0);
}


