#include <utility.h>
#include <ansi_c.h>
#include <userint.h>
#include "iconedit.h"
#include "iconui.h"
#include "iconutil.h"
#include "iconcolr.h"



    
ColorSelect gColorSelector = {0, VAL_WHITE, 0, 0, NULL, 0}; 
    
static int ColorSelect_BtnToIndex(ColorSelect *colorSelector, int control);
static int ColorSelect_IndexToBtn(ColorSelect *colorSelector, int index);


/*********************************************************************************/

void ColorSelect_InstallIcon(ColorSelect *colorSelector, Icon *icon)
{
    int index;
    
    colorSelector->icon = icon; 
    ColorSelect_SetDepth(colorSelector, icon->descr.colorDepth);    

    ColorSelect_SetIndex(colorSelector, colorSelector->colorIndex) ;    /* refresh display of selected color */

        /* set all the colors to themselves to make the pallete buttons update their colors */
    for (index = 0; index < colorSelector->numColors; index++)
        ColorSelect_SetIndexColor(colorSelector, index, ColorSelect_GetIndexColor(colorSelector, index));

    ColorSelect_SetIndexColor(colorSelector, kTransparentColorIndex, ColorSelect_GetIndexColor(colorSelector, kTransparentColorIndex));
}

/*********************************************************************************/

void ColorSelect_UnInstallIcon(ColorSelect *colorSelector, Icon *icon)
{
    if (colorSelector->icon == icon)
        colorSelector->icon = NULL;
}

/*********************************************************************************/

void ColorSelect_SetDepth(ColorSelect *colorSelector, int colorDepth)
{
    int         panelToLoad;
    static int  color2Panel = 0;
    static int  color16Panel = 0;
    static int  color256Panel = 0;
    int         *panelPtr;
    
    if (colorSelector->depth != colorDepth)
        {
        colorSelector->depth = colorDepth;
        SetWaitCursor(TRUE);      
        
        switch (colorDepth)
            {
            case 1:
                colorSelector->numColors = 2;
                panelToLoad = COLOR2;
                panelPtr = &color2Panel;
                break;
            case 4:
                colorSelector->numColors = 16;
                panelToLoad = COLOR16;
                panelPtr = &color16Panel;
                break;
            case 8:
                colorSelector->numColors = 256;
                panelToLoad = COLOR256;
                panelPtr = &color256Panel;
                break;
            }

        if (colorSelector->colorPalettePanel > 0)
            HidePanel(colorSelector->colorPalettePanel);
            
        if (*panelPtr == 0)
            *panelPtr = LoadPanel(mainPanel, "iconui.uir", panelToLoad);
        if (*panelPtr < 0)  
            FatalError("Error loading panel from .uir file", *panelPtr);        
            
        colorSelector->colorPalettePanel = *panelPtr;

            /* make sure the selected color index is in the range of selectable color indices */
        colorSelector->colorIndex = Pin(colorSelector->colorIndex, 0, colorSelector->numColors - 1);        
        
        SetPanelPos(colorSelector->colorPalettePanel, 380, 20);
        SetPanelAttribute (colorSelector->colorPalettePanel, ATTR_ACTIVATE_WHEN_CLICKED_ON, FALSE);
        SetPanelAttribute(colorSelector->colorPalettePanel, ATTR_MOUSE_CURSOR, VAL_POINTING_FINGER_CURSOR);
        SetPanelAttribute(colorSelector->colorPalettePanel, ATTR_VISIBLE, TRUE);
        SetWaitCursor(FALSE);      
        }    
}

/*********************************************************************************/

int ColorSelect_Depth(ColorSelect *colorSelector)
{
    return colorSelector->depth;
}

/*********************************************************************************/

static int ColorSelect_BtnToIndex(ColorSelect *colorSelector, int control)
{
    int index;
    
    if (ColorSelect_IndexToBtn(colorSelector, kTransparentColorIndex) == control) 
        return kTransparentColorIndex;  

    for (index = 0; index < colorSelector->numColors; index++)
        if (ColorSelect_IndexToBtn(colorSelector, index) == control)
            return index;

    AssertMsg(FALSE, "Button not found in ColorSelect_BtnToIndex"); 
    return 0;
}

/*********************************************************************************/

static int ColorSelect_IndexToBtn(ColorSelect *colorSelector, int index)
{
    static int colorIndexButtonList_16[16] =    /* table used to convert between the color selection buttons and their color indexes */
        {
        COLOR16_COLORBTN0,      /* control ID of color selection button for color index 0*/
        COLOR16_COLORBTN1,      /* etc... */
        COLOR16_COLORBTN2,
        COLOR16_COLORBTN3,
        COLOR16_COLORBTN4,
        COLOR16_COLORBTN5,
        COLOR16_COLORBTN6,
        COLOR16_COLORBTN7,
        COLOR16_COLORBTN8,
        COLOR16_COLORBTN9,
        COLOR16_COLORBTN10,
        COLOR16_COLORBTN11,
        COLOR16_COLORBTN12,
        COLOR16_COLORBTN13,
        COLOR16_COLORBTN14,
        COLOR16_COLORBTN15,
        };

    static int colorIndexButtonList_2[2] =  /* table used to convert between the color selection buttons and their color indexes */
        {
        COLOR2_COLORBTN0,       /* control ID of color selection button for color index 0*/
        COLOR2_COLORBTN1,       /* etc... */
        };


    switch (colorSelector->depth)
        {
        case 1:
            if (index == kTransparentColorIndex)
                return COLOR2_ERASE;
            else
                return colorIndexButtonList_2[index];
            break;
        case 4:
            if (index == kTransparentColorIndex)
                return COLOR16_ERASE;
            else
                return colorIndexButtonList_16[index];
            break;
        case 8:
            if (index == kTransparentColorIndex)
                return COLOR256_ERASE;
            else
                return COLOR256_COLORBTN_1 - index; /* buttons are in reverse order on the panel in the .uir file */
            break;
        }
        
    AssertMsg(FALSE, "color selector must have valid depth");       
    return 0;       
}

/*********************************************************************************/

void ColorSelect_SetIndexColor(ColorSelect *colorSelector, int index, int color)
{
    int buttonId;
    
    if (index == kTransparentColorIndex)
        colorSelector->transparentColor = color;
    else
    if (colorSelector->icon)
        colorSelector->icon->colorTable[index] = color; /* set the RGB value of the given color index */

        /* Color the button the same as the color pushing it selects */
    buttonId = ColorSelect_IndexToBtn(colorSelector, index);        /* get the button which selects the given color index */
    SetCtrlAttribute (colorSelector->colorPalettePanel, buttonId, ATTR_ON_COLOR, color); 
    SetCtrlAttribute (colorSelector->colorPalettePanel, buttonId, ATTR_OFF_COLOR, color); 
}

/*********************************************************************************/

int  ColorSelect_GetIndexColor(ColorSelect *colorSelector, int index)
{
    if (index == kTransparentColorIndex)
        return colorSelector->transparentColor;
    else
    if (colorSelector->icon)
        return colorSelector->icon->colorTable[index];
    else
        {
        AssertMsg(FALSE, "error can't get color when color selector doesn't have an icon installed");
        return VAL_BLACK;
        }
}

/*********************************************************************************/

void GetDefaultColorTable(int *colorTable, int colorDepth, int numColors)
{
    static int  default2BitColorTable[]=
        {
        VAL_BLACK,
        VAL_WHITE,
        };
    static int  default4BitColorTable[]=
        {
        VAL_BLACK,
        VAL_DK_RED,
        VAL_DK_GREEN,
        VAL_DK_YELLOW,
        VAL_DK_BLUE,
        VAL_DK_MAGENTA,
        VAL_DK_CYAN,
        VAL_DK_GRAY,
        VAL_LT_GRAY,
        VAL_RED,
        VAL_GREEN,
        VAL_YELLOW,
        VAL_BLUE,
        VAL_MAGENTA,
        VAL_CYAN,
        VAL_WHITE,
        };
    
    switch (colorDepth)
        {
        case 1:
            memmove(colorTable, default2BitColorTable, numColors * sizeof(int));
            break;
        case 4:
            memmove(colorTable, default4BitColorTable, numColors * sizeof(int));
            break;
        case 8:
            {
            int     rIndex, gIndex, bIndex, grayIndex;
            int     colorIndex = 0;
            double  inc = 255.0 / 5;
            double  grayInc = 255.0 / 15;
            int     grayIntensity;
    
                /* divide the cube, 6 on a side */
            for (rIndex = 0; rIndex < 6; rIndex++)
                for (gIndex = 0; gIndex < 6; gIndex++)
                    for (bIndex = 0; bIndex < 6; bIndex++)
                        if (colorIndex < numColors)
                            {
                            colorTable[colorIndex] = MakeColor(rIndex * inc, gIndex * inc, bIndex * inc);
                            colorIndex++;
                            }
                        else
                            return;
                
                /* throw in 16 shades of gray */                    
            for (grayIndex = 0; grayIndex < 16; grayIndex++)                
                if (colorIndex < numColors)  
                    {
                    grayIntensity = grayIndex * grayInc;
                    colorTable[colorIndex] = MakeColor(grayIntensity, grayIntensity, grayIntensity);
                    colorIndex++;
                    }
                else
                    return;

                /* fill in the rest of the table with back (enough is enough!) */
            while (colorIndex < numColors)
                {
                colorTable[colorIndex] = VAL_BLACK;
                colorIndex++;
                }
            }
        default:
            break;
        }
}

/*********************************************************************************/

int ColorSelect_GetIndex(ColorSelect *colorSelect)
{
    return colorSelect->colorIndex;
}

/*********************************************************************************/

void ColorSelect_SetIndex(ColorSelect *colorSelector, int colorIndex)
{
    int i;
    int oldColorIndex = colorSelector->colorIndex;
    
        /* make sure only the button for the selected color index is pushed in */
    for (i=0; i < colorSelector->numColors; i++)
        SetCtrlVal(colorSelector->colorPalettePanel, ColorSelect_IndexToBtn(colorSelector, i), i == colorIndex);

    SetCtrlVal(colorSelector->colorPalettePanel, ColorSelect_IndexToBtn(colorSelector, kTransparentColorIndex), kTransparentColorIndex == colorIndex);

    
    colorSelector->colorIndex = colorIndex;
    
    SetCtrlVal(mainPanel, PANEL_EDITCOLOR, ColorSelect_GetIndexColor(colorSelector, colorIndex)); 
    if (colorIndex != oldColorIndex)
        TextColor_HandleColorChange();
}

/*********************************************************************************/

int  CVICALLBACK ColorBtnCallback(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    if (event == EVENT_LEFT_CLICK)
        ColorSelect_SetIndex(&gColorSelector, ColorSelect_BtnToIndex(&gColorSelector, control));
    return 0;
}

