#include    <cvirte.h>    /* Needed if linking in external compiler; harmless otherwise */
#include    <userint.h>
#include    <utility.h>
#include    <stdio.h>
#include    "toolbox.h"

#include    "iconedit.h"
#include    "iconui.h"
#include    "iconio.h"
#include    "iconutil.h"
#include    "iconcolr.h"
#include    "icontool.h"


#define     kInitialIconColorDepth  4   /* 16 colors */

    
static void     DimmerFunction(int menuBar, int panel);
static int      MainCallback(int panelOrMenuBar,  int controlOrMenuItem, int event, void *callbackData,  int eventData1,  int eventData2);
static void     HandleMouseEditingEvent(int event, Point mousePos);

static void     Icon_Init(Icon *icon, IconType iconType);
static void     Icon_Dispose(Icon *icon);
static void     Icon_ClearPixelData(Icon *icon);

static void     IconFile_Init(IconFile *iconFile);
static void     IconFile_AddDefaultIcons(IconFile *iconFile);
static void     IconFile_SetActive(IconFile *iconFile); 
static void     IconFile_UpdateSelectionList(IconFile *iconFile); 
static int      IconFile_IconIndex(IconFile *iconFile, Icon *icon);
static void     IconFile_DiscardIcon(IconFile *iconFile, Icon *icon);

static int      LoadIconsToActiveIconFile(char *fileName);
static void     SetSelectedIcon(Icon *icon);
static void     SizeAndPositionSampleIcon(void);
static void     CancelSelectionDueToOperation(Selection *sel, Icon *icon);


char        gHelpFile[256];
IconFile    *gActiveIconFile;

int         mainPanel;
int         editPanel;
int         sampleIconPanel;
int         toolPalettePanel;
int         menuBar;
int         gQuitOccurred;
int         iconSelectPanel;
int         clipboardPanel;
int         clipboardCanvas;
    
/*************************************************************************************/

int main (int argc, char *argv[])
{
    int     i;
    char    projDir[MAX_PATHNAME_LEN];

    if (InitCVIRTE (0, argv, 0) == 0)    /* Needed if linking in external compiler; harmless otherwise */
        return -1;    /* out of memory */
    
    GetProjectDir (projDir);
    sprintf (gHelpFile, "%s\\iconedit.hlp", projDir);

    mainPanel = LoadPanel(0, "iconui.uir", PANEL);
    if (mainPanel < 0)  FatalError("Error loading panel from .uir file", mainPanel);        
        
    menuBar = GetPanelMenuBar(mainPanel);

    editPanel = LoadPanel(mainPanel, "iconui.uir", EDIT);
    if (editPanel < 0)  FatalError("Error loading panel from .uir file", editPanel);        
    SetPanelPos(editPanel, 40, 20);

    sampleIconPanel = LoadPanel(mainPanel, "iconui.uir", SAMPLE);
    if (sampleIconPanel < 0)  FatalError("Error loading panel from .uir file", sampleIconPanel);        
    SetPanelPos(sampleIconPanel, 40, 340);

    ColorSelect_SetDepth(&gColorSelector, kInitialIconColorDepth);
    
    toolPalettePanel = LoadPanel(mainPanel, "iconui.uir", TOOLS);
    if (toolPalettePanel < 0)  FatalError("Error loading panel from .uir file", toolPalettePanel);        
    SetPanelPos(toolPalettePanel, 380, 383);
    SetPanelAttribute (toolPalettePanel, ATTR_ACTIVATE_WHEN_CLICKED_ON, FALSE);
    
    iconSelectPanel = LoadPanel (mainPanel, "iconui.uir", ICONSELECT);
    if (iconSelectPanel < 0)  FatalError("Error loading panel from .uir file", iconSelectPanel);        
    SetPanelPos (iconSelectPanel, 155, 340);   /* 135, 360 */
    
    SetPanelAttribute(editPanel, ATTR_MOUSE_CURSOR, VAL_POINTING_FINGER_CURSOR);
    SetPanelAttribute(toolPalettePanel, ATTR_MOUSE_CURSOR, VAL_POINTING_FINGER_CURSOR);
    InstallMenuDimmerCallback(GetPanelMenuBar(mainPanel), DimmerFunction);
    InitTools();

    IconFile_SetActive(IconFile_New());     /* create an empty icon file and make it the current icon file */
    
    if (argc <= 1 || LoadIconsToActiveIconFile(argv[1]) != kFileOpSuccess)
        IconFile_AddDefaultIcons(ActiveIconFile());     
        
    InstallMainCallback(MainCallback, NULL, 0);

    SetSelectedIcon(NULL);
    SetPanelAttribute(sampleIconPanel, ATTR_BACKCOLOR, ColorSelect_GetIndexColor(&gColorSelector, kTransparentColorIndex));  /* make the sample panel bg color match the transparent color */
    EnableExtendedMouseEvents (editPanel, EDIT_ICON_CANVAS, 0.05);
    TextTool_SetMetaFont(NULL, VAL_DIALOG_META_FONT);
    
    DisplayPanel(editPanel);
    DisplayPanel(sampleIconPanel);
    DisplayPanel(toolPalettePanel);
    DisplayPanel(iconSelectPanel);
    DisplayPanel(mainPanel);

    RunUserInterface();
    
    return 0;
}

/*************************************************************************************/

IconFile *ActiveIconFile(void)
{
    return gActiveIconFile;
}

/*************************************************************************************/

static void IconFile_SetActive(IconFile *iconFile)
{
    gActiveIconFile = iconFile;
    IconFile_UpdateSelectionList(iconFile);
    IconFile_UpdateWindowTitle(iconFile);
    SetSelectedIcon(NULL);
}

/*************************************************************************************/

void IconFile_UpdateWindowTitle(IconFile *iconFile)
{
    char messageBuf[300];

    if (iconFile && (ActiveIconFile() == iconFile))
        {
        sprintf(messageBuf, "Icon Editor - %s", iconFile->fileName);
        SetPanelAttribute(mainPanel, ATTR_TITLE, messageBuf);   
        }
}

/*************************************************************************************/

    /*  the constructor for an IconFile */
static void IconFile_Init(IconFile *iconFile)
{
    iconFile->canRedo = FALSE;
    iconFile->canUndo = TRUE;
    iconFile->needsSave = FALSE;
    iconFile->iconList = ListCreate(sizeof(Icon *));
    if (!iconFile->iconList)
        FatalError("Out of memory", 0);
        
    iconFile->selectedIcon = NULL;
    strcpy(iconFile->fileName, kUntitledStr); 
}

/*************************************************************************************/

static void IconFile_AddDefaultIcons(IconFile *iconFile)
{
    Icon_New(iconFile, k32By32_16ColorIcon);
    Icon_New(iconFile, k16By16_16ColorIcon);
}

/*************************************************************************************/

IconFile *IconFile_New(void)
{
    IconFile    *iconFile;
    
    iconFile = malloc(sizeof(IconFile));
    if (!iconFile)
        FatalError("Out of memory", 0);     

    IconFile_Init(iconFile);
    
    return iconFile;
}

/*************************************************************************************/

static void IconFile_DiscardIcon(IconFile *iconFile, Icon *icon)
{
    int index;
    
    index = IconFile_IconIndex(iconFile, icon);
    if (index)
        {
        ListRemoveItem(iconFile->iconList, NULL, index);        
        Icon_Dispose(icon);     
        if (iconFile->selectedIcon == icon)
            iconFile->selectedIcon = NULL;  
        IconFile_UpdateSelectionList(iconFile);
        SetSelectedIcon(NULL);  /* select a new icon */
        }
}

/*************************************************************************************/

void IconFile_Dispose(IconFile *iconFile)
{
    int     numIcons;
    int     index;
    Icon    *iconPtr;
    
    if (iconFile)
        {
        if (ActiveIconFile() == iconFile)
            IconFile_SetActive(NULL);
            
        numIcons = ListNumItems(iconFile->iconList);
        
        for (index = 1; index <= numIcons; index++)
            {
            ListGetItem(iconFile->iconList, &iconPtr, index);
            Icon_Dispose(iconPtr);
            }
            
        ListDispose(iconFile->iconList);
        free(iconFile);
        }
}

/*************************************************************************************/

    /*  returns the 1-based index of an icon in an iconFile's icon list.  Returns 0 if the icon is not in the list */
static int IconFile_IconIndex(IconFile *iconFile, Icon *icon)
{
    int     index;
    int     numIcons;
    Icon    *currIcon;
    
    numIcons = ListNumItems(iconFile->iconList);
    
    for (index = 1; index <= numIcons; index++)
        {
        ListGetItem(iconFile->iconList, &currIcon, index);
        if (currIcon == icon)
            return index;
        }
        
    return 0;
}

/*************************************************************************************/

Icon *SelectedIcon(void)
{
    IconFile    *iconFile = ActiveIconFile();
    
    if (!iconFile)
        return NULL;
    else
        return iconFile->selectedIcon;
}
/*************************************************************************************/

    /*  pass NULL for icon to select the currently selected icon in the icon selection list box */
static void SetSelectedIcon(Icon *icon)
{
    IconFile    *iconFile = ActiveIconFile();
    int         iconIndex;
    
    if (!iconFile)
        return;
        
    if (!icon)
        {
        int selectedItem;
        
        GetCtrlIndex (iconSelectPanel, ICONSELECT_LIST, &selectedItem);     
        if (selectedItem >= 0)
            ListGetItem(iconFile->iconList, &icon, selectedItem + 1);           
        }
        
    if (icon != SelectedIcon())
        {
        int oldCursorState = GetWaitCursorState();
        
        SetWaitCursor(1);
        iconIndex = IconFile_IconIndex(iconFile, icon);
        if (iconIndex > 0)  /* must be an icon in the active icon file */
            {
            iconFile->selectedIcon = icon;
            SetCtrlIndex(iconSelectPanel, ICONSELECT_LIST, iconIndex - 1);
            SizeAndPositionSampleIcon();
            
            ColorSelect_InstallIcon(&gColorSelector, icon);

            Icon_InvalDisplay(icon, kSampleAndEditIcons, kNotAColorIndex);
            CancelSelectionDueToOperation(&gEditSelection, icon);
            Grid_Invalidate();      /* update grid display */
            Display_Update(icon, kSampleAndEditIcons);          
            iconFile->canUndo = FALSE;  /* undo info is stored per icon, new icon means no undo info */
            iconFile->canRedo = FALSE;
            }

        SetWaitCursor(oldCursorState);
        }
}

/*************************************************************************************/

static void Icon_ClearPixelData(Icon *icon)
{
    int         x, y;
    SavedState  state;
    
    for (x = 0; x < icon->descr.width; x++)
        for (y = 0; y < icon->descr.height; y++)
            {
            icon->pixels[x][y].colorIndex = kTransparentColorIndex;
            for (state = 0; state < kNumSavedStates; state++)
                icon->pixels[x][y].savedColorTableIndex[state] = kNotAColorIndex;
            }

    Icon_InvalDisplay(icon, kEditingIcon | kSampleIcon, kNotAColorIndex);
}

/*************************************************************************************/

IconType Icon_DescriptionToType(int height, int width, int numberOfColors)
{
    int             type;
    IconDescription iconDescription;
    
    for (type = kFirstIconType; type <= kLastIconType; type++)
        {
        Icon_TypeToDescription(type, &iconDescription);
        if (iconDescription.height == height && iconDescription.width == width && iconDescription.numColors == numberOfColors)
            return type;
        }

    return kUnsupportedIcon;
}

/*************************************************************************************/

    /*  This function encapsulates the info about the supported icon types. 
        It returns TRUE if the type is recognized, FALSE otherwise.
    */
int Icon_TypeToDescription(IconType type, IconDescription *description)
{
    int     knownType = TRUE;
    int     height;
    int     width;
    int     numberOfColors;
    int     colorDepth;
    int     pixelDataSizeInFile;
    int     maskSizeInFile;
    char    name[kMaxIconNameLength];
    int     pixelRowSize;
    int     maskRowSize;
    
    switch (type)
        {
        case k16By16_2ColorIcon:
        case k16By16_16ColorIcon:
        case k16By16_256ColorIcon:
        case k16By32_2ColorIcon:
        case k16By32_16ColorIcon:
        case k16By32_256ColorIcon:
        case k16By48_2ColorIcon:
        case k16By48_16ColorIcon:
        case k16By48_256ColorIcon:
            height = 16;
            break;
        case k32By16_2ColorIcon:
        case k32By16_16ColorIcon:
        case k32By16_256ColorIcon:
        case k32By32_2ColorIcon:
        case k32By32_16ColorIcon:
        case k32By32_256ColorIcon:
        case k32By48_2ColorIcon:
        case k32By48_16ColorIcon:
        case k32By48_256ColorIcon:
            height = 32;
            break;
        case k48By16_2ColorIcon:
        case k48By16_16ColorIcon:
        case k48By16_256ColorIcon:
        case k48By32_2ColorIcon:
        case k48By32_16ColorIcon:
        case k48By32_256ColorIcon:
        case k48By48_2ColorIcon:
        case k48By48_16ColorIcon:
        case k48By48_256ColorIcon:
            height = 48;
            break;
        }   
        
    switch (type)
        {
        case k16By16_2ColorIcon:
        case k16By16_16ColorIcon:
        case k16By16_256ColorIcon:
        case k32By16_2ColorIcon:
        case k32By16_16ColorIcon:
        case k32By16_256ColorIcon:
        case k48By16_2ColorIcon:
        case k48By16_16ColorIcon:
        case k48By16_256ColorIcon:
            width = 16;
            break;
        case k16By32_2ColorIcon:
        case k16By32_16ColorIcon:
        case k16By32_256ColorIcon:
        case k32By32_2ColorIcon:
        case k32By32_16ColorIcon:
        case k32By32_256ColorIcon:
        case k48By32_2ColorIcon:
        case k48By32_16ColorIcon:
        case k48By32_256ColorIcon:
            width = 32;
            break;
        case k16By48_2ColorIcon:
        case k16By48_16ColorIcon:
        case k16By48_256ColorIcon:
        case k32By48_2ColorIcon:
        case k32By48_16ColorIcon:
        case k32By48_256ColorIcon:
        case k48By48_2ColorIcon:
        case k48By48_16ColorIcon:
        case k48By48_256ColorIcon:
            width = 48;
            break;
        }   
    
    maskRowSize = RoundToNextMultiple(width / 8, 4);    /* row size must be a 4 byte multiple */
    maskSizeInFile = height * maskRowSize;
        
    switch (type)
        {
        case k16By16_2ColorIcon:
        case k16By32_2ColorIcon:
        case k16By48_2ColorIcon:
        case k32By16_2ColorIcon:
        case k32By32_2ColorIcon:
        case k32By48_2ColorIcon:
        case k48By16_2ColorIcon:
        case k48By32_2ColorIcon:
        case k48By48_2ColorIcon:
            numberOfColors = 2;
            colorDepth = 1;
            break;      
        case k16By16_16ColorIcon:
        case k16By32_16ColorIcon:
        case k16By48_16ColorIcon:
        case k32By16_16ColorIcon:
        case k32By32_16ColorIcon:
        case k32By48_16ColorIcon:
        case k48By16_16ColorIcon:
        case k48By32_16ColorIcon:
        case k48By48_16ColorIcon:
            numberOfColors = 16;
            colorDepth = 4;
            break;
        case k16By16_256ColorIcon:
        case k16By32_256ColorIcon:
        case k16By48_256ColorIcon:
        case k32By16_256ColorIcon:
        case k32By32_256ColorIcon:
        case k32By48_256ColorIcon:
        case k48By16_256ColorIcon:
        case k48By32_256ColorIcon:
        case k48By48_256ColorIcon:
            numberOfColors = 256;
            colorDepth = 8;
            break;
        }   

    sprintf(name, "%dx%d, %d Colors", height, width, numberOfColors);
    
    switch (type)
        {
        case k32By32_16ColorIcon:
        case k32By32_2ColorIcon:        
        case k32By32_256ColorIcon:      
        case k16By16_16ColorIcon:
        case k16By16_2ColorIcon:
        case k16By16_256ColorIcon:          
        case k16By32_16ColorIcon:
        case k16By32_2ColorIcon:
        case k16By32_256ColorIcon:
        case k32By16_16ColorIcon:
        case k32By16_2ColorIcon:
        case k32By16_256ColorIcon:
        case k48By48_2ColorIcon:
        case k48By48_16ColorIcon:
        case k48By48_256ColorIcon:
            break;
        default:
            knownType = FALSE;
        }   
    
    if  (knownType)
        {
        pixelRowSize = RoundToNextMultiple((width * colorDepth) / 8, 4); /* row must be a 4 byte multiple */
        pixelDataSizeInFile = height * pixelRowSize;
        
        description->height = height;
        description->width = width;
        description->numColors = numberOfColors;
        description->colorDepth = colorDepth;
        strcpy(description->name, name);
        description->pixelDataSizeInFile = pixelDataSizeInFile;
        description->maskSizeInFile = maskSizeInFile;
        description->maskRowSize = maskRowSize;
        description->pixelRowSize = pixelRowSize;
        }
            
    return knownType;
}

/*************************************************************************************/

    /*  The constructer for an icon */
static void Icon_Init(Icon *icon, IconType iconType)
{
    int             canvasWidth;
    int             canvasHeight;
    IconDescription description;

    Icon_TypeToDescription(iconType, &icon->descr);
        
    icon->type = iconType;
            
    icon->colorTable = malloc(sizeof(int) * icon->descr.numColors);
    if (!icon->colorTable)
        FatalError("Out of memory", 0);
        
    GetDefaultColorTable(icon->colorTable, icon->descr.colorDepth, icon->descr.numColors);
        
        /* The width and height of the editCanvas are assumed to be a multiple of kMaxIconDimension pixels */
    GetCtrlAttribute(editPanel, EDIT_ICON_CANVAS, ATTR_WIDTH, &canvasWidth);
    icon->pixelWidth = (canvasWidth+1) / icon->descr.width; /* +1 counts the zero'th pixel */
    GetCtrlAttribute(editPanel, EDIT_ICON_CANVAS, ATTR_HEIGHT, &canvasHeight);
    icon->pixelHeight = (canvasHeight+1) / icon->descr.height;  /* +1 counts the zero'th pixel */
    Icon_ClearPixelData(icon);
}

/*************************************************************************************/

Icon *Icon_New(IconFile *iconFile, int type)
{
    Icon    *newIcon;
    
    newIcon = malloc(sizeof(Icon));
    if (!newIcon)
        FatalError("Out of memory", 0);
        
    Icon_Init(newIcon, type);
    
    if (!ListInsertItem (iconFile->iconList, &newIcon, END_OF_LIST))
        FatalError("Out of memory", 0);
        
    IconFile_UpdateSelectionList(iconFile);
    
    return newIcon;
}

/*************************************************************************************/

static void Icon_Dispose(Icon *icon)
{
    ColorSelect_UnInstallIcon(&gColorSelector, icon);
    free(icon);
}

/*************************************************************************************/

    /*  This function fills in the contents on the icon selection list box with the icons
        that are present in the iconFile
    */
static void IconFile_UpdateSelectionList(IconFile *iconFile)
{
    int     numIcons;
    int     index;
    Icon    *iconPtr;
    int     selectedIndex = 0;
    
    DeleteListItem(iconSelectPanel, ICONSELECT_LIST, 0, -1);

    if (iconFile)
        {
        numIcons = ListNumItems(iconFile->iconList);
    
        for (index = 1; index <= numIcons; index++)
            {
            ListGetItem(iconFile->iconList, &iconPtr, index);
            if (InsertListItem(iconSelectPanel, ICONSELECT_LIST, -1, iconPtr->descr.name, (int)iconPtr) < 0)
                FatalError("Out of memory", 0); 
            
            if (iconPtr == iconFile->selectedIcon)
                selectedIndex = index - 1;
            }
        
        if (numIcons > 0)
            SetCtrlIndex(iconSelectPanel, ICONSELECT_LIST, selectedIndex);
        }
}

/*************************************************************************************/

int  CVICALLBACK EditingAreaCallback(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    Point       mousePos;
    Icon        *icon = SelectedIcon();
    static int  mouseDownInEditArea = FALSE;
    
    if (event == EVENT_LEFT_DOUBLE_CLICK)
        event = EVENT_LEFT_CLICK;   /* treated the same as a left click */
        
    switch (event)
        {
        case EVENT_LEFT_CLICK:
            mouseDownInEditArea = TRUE;
            break;
        case EVENT_LEFT_MOUSE_UP:
            mouseDownInEditArea = FALSE;
            break;
        }       
        
        
    switch (event)
        {
        case EVENT_LEFT_CLICK:
        case EVENT_LEFT_MOUSE_UP:
        case EVENT_MOUSE_MOVE:
            PointSet(&mousePos, eventData2, eventData1);
            ConvertMouseCoordinates(panel, control, TRUE, TRUE, &mousePos.x, &mousePos.y);
            mousePos.x /= icon->pixelWidth;
            mousePos.y /= icon->pixelHeight;
            if (Sel_IsIconPixelSelected(&gEditSelection, mousePos))
                SetPanelAttribute (panel, ATTR_MOUSE_CURSOR, VAL_OPEN_HAND_CURSOR);
            else
                SetCursorForCurrentTool();
            break;
        }
      
    switch (event)
        {
        case EVENT_LEFT_CLICK:
        case EVENT_LEFT_MOUSE_UP:
        case EVENT_MOUSE_MOVE:
            if (mouseDownInEditArea || event != EVENT_MOUSE_MOVE)
                HandleMouseEditingEvent(event, mousePos);
            break;
        }

      return 0;
}
    
/*************************************************************************************/

    /*  handles mouse click, drag, and release */
static void  HandleMouseEditingEvent(int event, Point mousePos)
{
    static Point    start;
    static Point    selStart;
    static Point    textStart;
    int             tool = GetCurrentTool();
    IconFile        *iconFile = ActiveIconFile();
    Icon            *icon = SelectedIcon();
    int             colorIndex = ColorSelect_GetIndex(&gColorSelector);
    static int      draggingSelection = FALSE;
    int             iconDisplayToUpdate = kEditingIcon;

    if (!iconFile || !icon)
        return;
        
    StartIconBatchDraw(kEditingIcon);
    
    switch (tool)
        {
        case kTextTool:
            if (event == EVENT_LEFT_CLICK)
                {
                draggingSelection = Sel_IsIconPixelSelected(&gEditSelection, mousePos);
                
                if (draggingSelection)
                    {
                    IconFile_SaveUndoInfo(iconFile, kSelectionMoveAction);
                    start = mousePos;     
                    textStart = TextTool_GetPosition();
                    Sel_GetPosition(&gEditSelection, &selStart);
                    }
                else
                    {
                    TextTool_CommitText(&gEditSelection, icon);   
                    selStart = start = mousePos;
                    TextTool_SetPosition(mousePos);
                    Display_Update(icon, kEditingIcon);   
                    }
                }
            else
            if (draggingSelection)
                {
                Point newTextPos = MakePoint(textStart.x + mousePos.x - start.x , textStart.y + mousePos.y - start.y);
                Point newSelPos = MakePoint(selStart.x + mousePos.x - start.x , selStart.y + mousePos.y - start.y);
                TextTool_SetPosition(newTextPos);
                Sel_SetPosition(&gEditSelection, newSelPos);
                
                if (event == EVENT_LEFT_MOUSE_UP)
                    Sel_Apply(&gEditSelection, icon);
                }
            break;
        case kSelectSquareTool:
        case kSelectTool:
            if (event == EVENT_LEFT_CLICK)
                {
                draggingSelection = Sel_IsIconPixelSelected(&gEditSelection, mousePos);
                
                if (draggingSelection)
                    {
                    IconFile_SaveUndoInfo(iconFile, kSelectionMoveAction);
                    start = mousePos;                   
                    Sel_GetPosition(&gEditSelection, &selStart);
                    }
                else
                    {
                    Sel_ClearSelection(&gEditSelection, icon);
                    Sel_SelectPixel(&gEditSelection, icon, mousePos);
                    selStart = start = mousePos;
                    }
                }
            else
            if (draggingSelection)
                {
                Sel_SetPosition(&gEditSelection, MakePoint(selStart.x + mousePos.x - start.x , selStart.y + mousePos.y - start.y));
                
                if (event == EVENT_LEFT_MOUSE_UP)
                    Sel_Apply(&gEditSelection, icon);
                }
            else
                {
                if (tool == kSelectSquareTool)
                    {
                    Sel_ClearSelection(&gEditSelection, icon);
                    GenericDrawRect(selStart, mousePos, FALSE, Sel_SetPixelCallback, &gEditSelection, kSelUnknown);
                    }
                else
                if (tool == kSelectTool)
                    {
                    GenericDrawLine(start, mousePos, Sel_SetPixelCallback, &gEditSelection, kSelUnknown);
                    start = mousePos;
                    }
                    
                if (event == EVENT_LEFT_MOUSE_UP)
                    {
                    if (tool == kSelectTool) 
                        GenericDrawLine (mousePos, selStart, Sel_SetPixelCallback, &gEditSelection, kSelUnknown);
                    Display_Update(icon, kEditingIcon);   /* so the user sees the selection lasso being closed */
                    Sel_Compute(&gEditSelection, icon, FALSE);
                    Sel_LiftUp(&gEditSelection, icon);
                    }
                }
            break;
        case kPencilTool:
            if (event == EVENT_MOUSE_MOVE || event == EVENT_LEFT_CLICK)
                {
                if (event == EVENT_LEFT_CLICK)
                    {
                    IconFile_SaveUndoInfo(iconFile, kDrawingAction);

                    start = mousePos;
                    Icon_SetPixel(icon, start, colorIndex, kNoDisplayIcon);
                    }
                else
                    {
                    Icon_DrawLine(icon, start, mousePos, colorIndex, kNoDisplayIcon);
                    start = mousePos;
                    }
                }
            break;
        case kLineTool:
        case kBoxFrameTool:
        case kBoxFillTool:
        case kOvalFrameTool:
        case kOvalFillTool:
            if (event == EVENT_LEFT_CLICK)
                {
                IconFile_SaveUndoInfo(iconFile, kDrawingAction);

                start = mousePos;
                }
            else
                {
                Icon_RestoreState(icon, kUndoSavedState, FALSE);
                    
                switch(tool)
                    {
                    case kLineTool:
                        Icon_DrawLine(icon, start, mousePos, colorIndex, kNoDisplayIcon);
                        break;
                    case kBoxFrameTool:
                        Icon_DrawRect(icon, start, mousePos, colorIndex, kNoDisplayIcon, FALSE);
                        break;
                    case kBoxFillTool:
                        Icon_DrawRect(icon, start, mousePos, colorIndex, kNoDisplayIcon, TRUE);
                        break;
                    case kOvalFrameTool:
                        Icon_DrawOval(icon, start, mousePos, colorIndex, kNoDisplayIcon, FALSE);
                        break;
                    case kOvalFillTool:
                        Icon_DrawOval(icon, start, mousePos, colorIndex, kNoDisplayIcon, TRUE);
                        break;
                    }

                if (event == EVENT_LEFT_MOUSE_UP)
                    switch(tool)
                        {
                        case kLineTool:
                            Icon_DrawLine(icon, start, mousePos, colorIndex, kNoDisplayIcon);
                            break;
                        case kBoxFrameTool:
                            Icon_DrawRect(icon, start, mousePos, colorIndex, kNoDisplayIcon, FALSE);
                            break;
                        case kBoxFillTool:
                            Icon_DrawRect(icon, start, mousePos, colorIndex, kNoDisplayIcon, TRUE);
                            break;
                        case kOvalFrameTool:
                            Icon_DrawOval(icon, start, mousePos, colorIndex, kNoDisplayIcon, FALSE);
                            break;
                        case kOvalFillTool:
                            Icon_DrawOval(icon, start, mousePos, colorIndex, kNoDisplayIcon, TRUE);
                            break;
                        }                    
                }
            break;
        case kFloodTool:
            if (event == EVENT_LEFT_CLICK)
                {
                IconFile_SaveUndoInfo(iconFile, kDrawingAction);

                SetWaitCursor(1);
                Icon_FloodFill(icon, mousePos, colorIndex, icon->pixels[mousePos.x][mousePos.y].colorIndex, kNoDisplayIcon);
                SetWaitCursor(0);
                iconDisplayToUpdate |= kSampleIcon;
                }
            break;
        }

    if (event == EVENT_LEFT_MOUSE_UP)
        iconDisplayToUpdate |= kSampleIcon;
        
    Display_Update(icon, iconDisplayToUpdate);   
    EndIconBatchDraw(kEditingIcon);
}

/*************************************************************************************/

static void CancelSelectionDueToOperation(Selection *sel, Icon *icon)
{
    Sel_ClearSelection(&gEditSelection, icon);
    TextTool_ClearText(icon);
}

/*************************************************************************************/

void IconFile_SaveUndoInfo(IconFile *iconFile, UndoableAction action)
{
    Icon    *icon = SelectedIcon();

    if (icon)
        {
        switch (action)
            {
                /*  if more than one of these action(s) occur in a row, they are considered all to be
                    one action for undo purposes.
                */
            case kSelectionMoveAction:
                if (iconFile->lastUndoableAction != action)
                    Icon_SaveState(icon, kUndoSavedState);
                break;
            default:
                Icon_SaveState(icon, kUndoSavedState);
                break;
            }
        }   
    
    iconFile->canUndo = TRUE;
    iconFile->canRedo = FALSE;
    iconFile->needsSave = TRUE;
    iconFile->lastUndoableAction = action;
}

/*************************************************************************************/

static void CVICALLBACK DimmerFunction(int menuBar, int panel)
{
    IconFile    *iconFile = ActiveIconFile();
    int         clipAvailable;
    int         selectionActive = Sel_IsActive(&gEditSelection);

    ClipboardGetBitmap (0, &clipAvailable);
    
    SetMenuBarAttribute(menuBar, MENUBAR_FILE_SAVE, ATTR_DIMMED, !iconFile || !iconFile->needsSave);
    SetMenuBarAttribute(menuBar, MENUBAR_EDIT_PASTE, ATTR_DIMMED, !clipAvailable);
    SetMenuBarAttribute(menuBar, MENUBAR_EDIT_CUT, ATTR_DIMMED, !selectionActive);
    SetMenuBarAttribute(menuBar, MENUBAR_EDIT_COPY, ATTR_DIMMED, !selectionActive);
    SetMenuBarAttribute(menuBar, MENUBAR_EDIT_UNDO, ATTR_DIMMED, !iconFile || !iconFile->canUndo);
    SetMenuBarAttribute(menuBar, MENUBAR_EDIT_REDO, ATTR_DIMMED, !iconFile || !iconFile->canRedo);
}

/*************************************************************************************/

void CVICALLBACK ExitCallback(int menubar, int menuItem, void *callbackData, int panel)
{
    int         quit = TRUE;
    IconFile    *iconFile = ActiveIconFile();  

    if (iconFile && iconFile->needsSave)
        quit = !IconFile_PromptToSave(iconFile);
        
    if (quit)
        QuitUserInterface(0);

    gQuitOccurred = quit;
}

/*************************************************************************************/

static int CVICALLBACK MainCallback(int panelOrMenuBar,  int controlOrMenuItem, int event, void *callbackData,  int eventData1,  int eventData2)
{
    if (event == EVENT_END_TASK)
        {
        ExitCallback(0, 0, 0, 0);
        if (gQuitOccurred)      /* Set inside ExitCallback */
            return 0;
        else
            return 1;
        }
    return 0;
}

/*************************************************************************************/

int CVICALLBACK PanelCallback(int panel, int event, void *callbackData, int eventData1, int eventData2)
{
    Icon        *icon = SelectedIcon(); 
    IconFile    *iconFile = ActiveIconFile();  
    Point       textPos;
    int         dx = 0, dy =0;
    int         tool = GetCurrentTool();
    
    switch (event)
        {
        case EVENT_KEYPRESS:
            if (icon)
                {
                switch (eventData1)
                    {
                    case VAL_UP_ARROW_VKEY:
                        dy = -1;
                        break;
                    case VAL_DOWN_ARROW_VKEY:
                        dy = 1;
                        break;
                    case VAL_LEFT_ARROW_VKEY:
                        dx = -1;
                        break;
                    case VAL_RIGHT_ARROW_VKEY:
                        dx = 1;
                        break;
                    default:
                        break;
                    }
                        
                if (dx != 0 || dy != 0)
                    {
                    switch (tool)
                        {
                        case kTextTool:
                        case kSelectTool:
                        case kSelectSquareTool:
                            if (Sel_ShiftSelection(&gEditSelection, icon, dx, dy))
                                {
                                IconFile_SaveUndoInfo(iconFile, kSelectionMoveAction);
                                
                                if (tool == kTextTool)
                                    {
                                    textPos = TextTool_GetPosition();
                                    textPos.x += dx;
                                    textPos.y += dy;
                                    TextTool_SetPosition(textPos);
                                    }
                                Sel_Apply(&gEditSelection, icon);
                                Display_Update(icon, kEditingIcon | kSampleIcon);   
                                }
                            else
                                Beep();
                            break;
                        default:
                            break;
                        }
                    }
                else
                    {
                    switch (tool)
                        {
                        case kTextTool:
                            TextTool_HandleKey(&gEditSelection, icon, eventData1);
                        break;
                        }
                    }
                }
            break;
        case EVENT_CLOSE:
            ExitCallback(0, 0, 0, 0);
        }
    return 0;
}

/*************************************************************************************/

void CVICALLBACK NewCallback(int menubar, int menuItem, void *callbackData, int panel)
{
    IconFile    *iconFile = ActiveIconFile();  
    IconFile    *newIconFile;

    if (!iconFile || !iconFile->needsSave || !IconFile_PromptToSave(iconFile))
        {
        IconFile_Dispose(iconFile);
        newIconFile = IconFile_New();
        if (!newIconFile)
            FatalError("Out of memory", 0);
            
        IconFile_AddDefaultIcons(newIconFile);  
        IconFile_SetActive(newIconFile);
        SetSelectedIcon(NULL);
        }
}

/*************************************************************************************/

static int LoadIconsToActiveIconFile(char *fileName)
{
    IconFile    *iconFile = ActiveIconFile();  
    IconFile    *newIconFile = 0;
    int         numUnsupportedIcons; 
    char        msg[400];
    int         result;

    newIconFile = IconFile_New();
    if (!newIconFile)
        FatalError("Out of memory", 0);
        
    switch (result = IconFile_Load(newIconFile, fileName, &numUnsupportedIcons))
        {
        case kFileOpSuccess:
            if (numUnsupportedIcons > 0)
                {
                sprintf(msg,    "The file '%s' contains\n"
                                "%d icon(s) of sizes or color depths that are\n"
                                "not supported by the icon editor\n\n"
                                "If you save changes to this file, the unsupported icons will be deleted.\n\n"
                                "Do you want to load this file?",
                                newIconFile->fileName, numUnsupportedIcons);
                if (!ConfirmPopup("Icon Editor Message", msg))
                    break;  /* break out before switching to use the loaded icon file */
                }
                
                
            IconFile_Dispose(iconFile);     /* discard current file in favor of the newly loaded file */
            IconFile_SetActive(newIconFile);
            newIconFile = 0;    /* avoid dispose at end of function */
            break;
        case kFileOpIOError:
            sprintf(msg, "An error occurred while trying to load '%s'.\n\nThis file may be corrupted or may not contain any supported icon types", fileName);
            MessagePopup("Icon Editor Message", msg);
            break;
        case kFileOpCanceled:
            break;
        }

    IconFile_Dispose(newIconFile);
    
    return result;
}

/*************************************************************************************/

void CVICALLBACK OpenCallback(int menubar, int menuItem, void *callbackData, int panel)
{
    int         ret;
    IconFile    *iconFile = ActiveIconFile();  
    IconFile    *newIconFile = 0;
    
    if (iconFile && (!iconFile->needsSave || !IconFile_PromptToSave(iconFile)))
        LoadIconsToActiveIconFile(NULL);

    return;
}

/*************************************************************************************/

void CVICALLBACK SaveAsCallback(int menubar, int menuItem, void *callbackData, int panel)
{
    IconFile    *iconFile = ActiveIconFile();  
    
    if (iconFile)
        iconFile->needsSave = !IconFile_Save(iconFile, TRUE);
        
    return;
}

/*************************************************************************************/

void CVICALLBACK SaveCallback(int menubar, int menuItem, void *callbackData, int panel)
{
    IconFile    *iconFile = ActiveIconFile();  
    
    if (iconFile && iconFile->needsSave)
        iconFile->needsSave = !IconFile_Save(iconFile, FALSE);
        
    return;
}

/*************************************************************************************/

int CVICALLBACK EditColorCallback(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    int         val;
    int         currentColorIndex = ColorSelect_GetIndex(&gColorSelector);
    Icon        *icon = SelectedIcon();
    IconDisplay whichDisplay;
    IconFile    *iconFile = ActiveIconFile();  
    
    if ((event == EVENT_VAL_CHANGED) || (event == EVENT_COMMIT))
        {
        GetCtrlVal(panel, control, &val);
        
        ColorSelect_SetIndexColor(&gColorSelector, currentColorIndex, val);

        if (event == EVENT_COMMIT)     /* for display speed, only update the sample when the color is committed */
            {
            whichDisplay = kSampleAndEditIcons;

            if (iconFile)
                iconFile->needsSave = TRUE;

            SetActivePanel(editPanel);  /* color edit control doesn't want to keep the focus (in case the text tool should need it) */
            }
        else
            whichDisplay = kEditingIcon;
            
        if (icon)
            {
            Icon_InvalDisplay(icon, whichDisplay, currentColorIndex);                   
            if (event == EVENT_COMMIT && currentColorIndex == kTransparentColorIndex)
                SetPanelAttribute(sampleIconPanel, ATTR_BACKCOLOR, val);

            Display_Update(icon, whichDisplay); 
            }
        }
    else
    if (event == EVENT_RIGHT_CLICK)
        SystemHelp (gHelpFile, HELP_CONTEXT, 10, 0);    
        
    return 0;
}

/*************************************************************************************/

void CVICALLBACK ShowGridCallback(int menubar, int menuItem, void *callbackData, int panel)
{
    Icon    *icon = SelectedIcon();
    
    Grid_SetVisible(!Grid_Visible());
    if (icon)
        Display_Update(icon, kEditingIcon);     /* cause grid to redraw */
    
    SetMenuBarAttribute(menubar, menuItem, ATTR_CHECKED, Grid_Visible());
    return;
}

/*************************************************************************************/

void CVICALLBACK FlipHCallback(int menubar, int menuItem, void *callbackData, int panel)
{
    int         i, j;
    int         tempColorIndex[kMaxIconDimension][kMaxIconDimension];
    IconFile    *iconFile = ActiveIconFile();  
    Icon        *icon = SelectedIcon();

    if (!iconFile || !icon) 
        return;
        
    IconFile_SaveUndoInfo(iconFile, kDrawingAction);
    CancelSelectionDueToOperation(&gEditSelection, icon);

    for (j=0; j < icon->descr.height; j++)
        for (i=0; i < icon->descr.height; i++)
            tempColorIndex[(icon->descr.width-1)-i][j] = icon->pixels[i][j].colorIndex;
            
    for (j=0; j < icon->descr.height; j++)
        for (i=0; i < icon->descr.width; i++)
            icon->pixels[i][j].colorIndex = tempColorIndex[i][j];
            
    Display_Update(icon, kSampleAndEditIcons);
}

/*************************************************************************************/

void CVICALLBACK FlipVCallback(int menubar, int menuItem, void *callbackData, int panel)
{
    int         i, j;
    int         tempColorIndex[kMaxIconDimension][kMaxIconDimension];
    IconFile    *iconFile = ActiveIconFile();  
    Icon        *icon = SelectedIcon();

    if (!iconFile || !icon) 
        return;
        
    IconFile_SaveUndoInfo(iconFile, kDrawingAction);
    CancelSelectionDueToOperation(&gEditSelection, icon);

    for (j=0; j < icon->descr.height; j++)
        for (i=0; i < icon->descr.width; i++)
            tempColorIndex[i][(icon->descr.width - 1) - j] = icon->pixels[i][j].colorIndex;
            
    for (j=0; j < icon->descr.height; j++)
        for (i=0; i < icon->descr.width; i++)
            icon->pixels[i][j].colorIndex = tempColorIndex[i][j];
            
    Display_Update(icon, kSampleAndEditIcons);
}

/*************************************************************************************/

void CVICALLBACK RedoCallback(int menubar, int menuItem, void *callbackData, int panel)
{
    IconFile    *iconFile = ActiveIconFile();  
    Icon        *icon = SelectedIcon();

    if (!iconFile || !icon) 
        return;

    SetWaitCursor(1);
    CancelSelectionDueToOperation(&gEditSelection, icon);
    Icon_RestoreState(icon, kUndoSavedState, TRUE);
    SetWaitCursor(0);
    iconFile->canRedo = FALSE;
    iconFile->canUndo = TRUE;
    iconFile->lastUndoableAction = kRedoAction;
    Display_Update(icon, kSampleAndEditIcons);
}

/*************************************************************************************/

void CVICALLBACK UndoCallback(int menubar, int menuItem, void *callbackData, int panel)
{
    IconFile    *iconFile = ActiveIconFile();  
    Icon        *icon = SelectedIcon();
    int         cancelTextOperation = GetCurrentTool() == kTextTool && Sel_IsActive(&gEditSelection);

    if (!iconFile || !icon) 
        return;
        
    SetWaitCursor(1);
    if (cancelTextOperation)
        {
        Sel_Delete(&gEditSelection, icon);
        }
    else
        {
        CancelSelectionDueToOperation(&gEditSelection, icon);
        Icon_RestoreState(icon, kUndoSavedState, TRUE);
        iconFile->canUndo = FALSE;
        iconFile->canRedo = TRUE;
        iconFile->lastUndoableAction = kNoAction;
        }
    Display_Update(icon, kSampleAndEditIcons);
    SetWaitCursor(0);
}

/*************************************************************************************/

void CVICALLBACK AboutCallback(int menubar, int menuItem, void *callbackData, int panel)
{
    int aboutBoxPanel;
    
    aboutBoxPanel = LoadPanel(0, "iconui.uir", ABOUT);
    if (aboutBoxPanel < 0)  FatalError("Error loading panel from .uir file", aboutBoxPanel);        
    InstallPopup(aboutBoxPanel);
}

/*************************************************************************************/

int CVICALLBACK OkCallback(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    if (event == EVENT_COMMIT)
        DiscardPanel(panel);
    return 0;
}

/*************************************************************************************/

void CVICALLBACK ContentsCallback(int menubar, int menuItem, void *callbackData, int panel)
{
    SystemHelp (gHelpFile, HELP_CONTENTS, 0, 0);
}

/*************************************************************************************/

void CVICALLBACK SearchCallback(int menubar, int menuItem, void *callbackData, int panel)
{
    SystemHelp (gHelpFile, HELP_PARTIALKEY, 0, "");
}

/*************************************************************************************/

int CVICALLBACK ColorPanelCallback(int panel, int event, void *callbackData, int eventData1, int eventData2)
{
    if (event == EVENT_RIGHT_CLICK)
        SystemHelp (gHelpFile, HELP_CONTEXT, 9, 0);

    return 0;
}

/*************************************************************************************/

int CVICALLBACK IconSelectionPanelCallback(int panel, int event, void *callbackData, int eventData1, int eventData2)
{
    if (event == EVENT_RIGHT_CLICK)
        SystemHelp (gHelpFile, HELP_CONTEXT, 11, 0);

    return 0;
}
/*************************************************************************************/

int CVICALLBACK EditPanelCallback(int panel, int event, void *callbackData, int eventData1, int eventData2)
{
    if (event == EVENT_RIGHT_CLICK)
        SystemHelp (gHelpFile, HELP_CONTEXT, 6, 0);

    return 0;
}

/*************************************************************************************/

int CVICALLBACK ToolPanelCallback(int panel, int event, void *callbackData, int eventData1, int eventData2)
{
    if (event == EVENT_RIGHT_CLICK)
        SystemHelp (gHelpFile, HELP_CONTEXT, 5, 0);

    return 0;
}

/*************************************************************************************/

int CVICALLBACK SamplePanelCallback(int panel, int event, void *callbackData, int eventData1, int eventData2)
{
    if (event == EVENT_RIGHT_CLICK)
        SystemHelp (gHelpFile, HELP_CONTEXT, 7, 0);

    return 0;
}

/*************************************************************************************/

int CVICALLBACK WindowColorPanelCallback(int panel, int event, void *callbackData, int eventData1, int eventData2)
{
    if (event == EVENT_RIGHT_CLICK)
        SystemHelp (gHelpFile, HELP_CONTEXT, 8, 0);

    return 0;
}

/*************************************************************************************/

void CVICALLBACK RotateRightCallback(int menubar, int menuItem, void *callbackData, int panel)
{
    int         i, j;
    int         tempColorIndex[kMaxIconDimension][kMaxIconDimension];
    IconFile    *iconFile = ActiveIconFile();  
    Icon        *icon = SelectedIcon();

    if (!iconFile || !icon) 
        return;
        
    IconFile_SaveUndoInfo(iconFile,kDrawingAction);
    CancelSelectionDueToOperation(&gEditSelection, icon);
    
    for (j=0; j < icon->descr.height; j++)
        for (i=0; i < icon->descr.width; i++)
            tempColorIndex[(icon->descr.width-1) - j][i] = icon->pixels[i][j].colorIndex;

    for (j=0; j < icon->descr.height; j++)
        for (i=0; i < icon->descr.width; i++)
            icon->pixels[i][j].colorIndex = tempColorIndex[i][j];
            
    Display_Update(icon, kSampleAndEditIcons);
}

/*************************************************************************************/

void CVICALLBACK RotateLeftCallback(int menubar, int menuItem, void *callbackData, int panel)
{
    int         i, j;
    int         tempColorIndex[kMaxIconDimension][kMaxIconDimension];
    IconFile    *iconFile = ActiveIconFile();  
    Icon        *icon = SelectedIcon();

    if (!iconFile || !icon) 
        return;

    IconFile_SaveUndoInfo(iconFile, kDrawingAction);
    CancelSelectionDueToOperation(&gEditSelection, icon);
    
    for (j=0; j < icon->descr.height; j++)
        for (i=0; i < icon->descr.width; i++)
            tempColorIndex[j][(icon->descr.width - 1)-i] = icon->pixels[i][j].colorIndex;
            
    for (j=0; j < icon->descr.height; j++)
        for (i=0; i < icon->descr.width; i++)
            icon->pixels[i][j].colorIndex = tempColorIndex[i][j];
            
    Display_Update(icon, kSampleAndEditIcons);
}


/*************************************************************************************/

    /* if error code is less than zero, the error code and description is added to the message */
void FatalError(char *msg, int uiErrorCode)
{
    char    errMsg[400];
    
    if (uiErrorCode > 0)
        strcpy(errMsg, msg);
    else
        sprintf(errMsg, "%s\nError Code = %d\n'%s'\n", msg, uiErrorCode, GetUILErrorString (uiErrorCode));
        
    MessagePopup("Fatal Icon Editor Error", errMsg);
    exit(0);
}


/*************************************************************************************/

static void SetSelectedIconDeferredCallback(void *data)
{
    int btnDown;

    GetGlobalMouseState (0, 0, 0, &btnDown, 0, 0);
    if (btnDown)    /* allow hilite to be dragged unimpeded, try again later */
        PostDeferredCall(SetSelectedIconDeferredCallback, 0);
    else
        SetSelectedIcon(NULL);
}

/*************************************************************************************/

int CVICALLBACK IconSelectionCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) 
        {
        case EVENT_VAL_CHANGED:
        case EVENT_COMMIT:
            SetSelectedIconDeferredCallback(0);
            break;
        }
    return 0;
}

/*************************************************************************************/

int CVICALLBACK SelectIconCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) 
        {
        case EVENT_COMMIT:
            SetSelectedIcon(NULL);
            break;
        }
    return 0;
}

/*************************************************************************************/

int CVICALLBACK InsertIconCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    int             insertPanel;
    IconType        type;
    char            name[kMaxIconNameLength];
    IconFile        *iconFile = ActiveIconFile();  
    int             evtCtrl;
    IconDescription description;
    
    switch (event) 
        {
        case EVENT_COMMIT:
            if (!iconFile)
                return 0;
                
            insertPanel = LoadPanel (0, "iconui.uir", INSERT);
            if (insertPanel < 0)
                FatalError("Error loading panel from .uir file", insertPanel);
                
            for (type = kFirstIconType; type <= kLastIconType; type++)
                {
                Icon_TypeToDescription(type, &description);
#ifdef SQUARE_ICONS_ONLY
                if (description.height == description.width)
#endif
                InsertListItem (insertPanel, INSERT_ICONTYPE, -1, description.name, (int)type);
                }
        
            InstallPopup(insertPanel);
            GetUserEvent (TRUE, 0, &evtCtrl);
            if (evtCtrl == INSERT_INSERT)
                {
                GetCtrlVal(insertPanel, INSERT_ICONTYPE, (int *)&type);
                SetSelectedIcon(Icon_New(iconFile, type));
                iconFile->needsSave = TRUE;
                }
            DiscardPanel(insertPanel);
            break;
        }
    return 0;
}

/*************************************************************************************/

int CVICALLBACK DeleteIconCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    IconFile    *iconFile = ActiveIconFile();  
    Icon        *icon = NULL;
    int         selectedItem;

    switch (event) 
        {
        case EVENT_COMMIT:
            GetCtrlIndex (iconSelectPanel, ICONSELECT_LIST, &selectedItem);     
            if (selectedItem >= 0)
                ListGetItem(iconFile->iconList, &icon, selectedItem + 1);           

            if (!iconFile || !icon) 
                {
                Beep();
                return 0;
                }
                
            if (ListNumItems(iconFile->iconList) == 1)
                MessagePopup("Icon Editor Message", "The last icon cannot be deleted.\n\nThe icon file must contain at least one icon.");
            else                
                {
                IconFile_DiscardIcon(iconFile, icon); 
                iconFile->needsSave = TRUE;
                }
            break;
        }
    return 0;
}

/*************************************************************************************/

static void SizeAndPositionSampleIcon(void)
{
    Icon    *icon = SelectedIcon();
    int     panelWidth;
    int     labelWidth;
    
    if (icon)
        {
        GetPanelAttribute(sampleIconPanel, ATTR_WIDTH, &panelWidth);
        GetCtrlAttribute(sampleIconPanel, SAMPLE_CANVAS, ATTR_LABEL_WIDTH, &labelWidth);
        SetCtrlAttribute(sampleIconPanel, SAMPLE_CANVAS, ATTR_WIDTH, icon->descr.width);
        SetCtrlAttribute(sampleIconPanel, SAMPLE_CANVAS, ATTR_HEIGHT, icon->descr.height);
        SetCtrlAttribute(sampleIconPanel, SAMPLE_CANVAS, ATTR_LEFT, (panelWidth - icon->descr.width) / 2);
        SetCtrlAttribute(sampleIconPanel, SAMPLE_CANVAS, ATTR_LABEL_LEFT, (panelWidth - labelWidth) / 2); 
        }
}

/*************************************************************************************/

int CVICALLBACK AnimateSelectionCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event)
        {
        case EVENT_TIMER_TICK:
            {
            Icon *icon = SelectedIcon();

            if (icon)
                Display_AnimateSelection(icon, &gEditSelection);
            break;
            }
        }
    return 0;
}

/*************************************************************************************/

void CVICALLBACK CutCallback (int menuBar, int menuItem, void *callbackData, int panel)
{
    IconFile    *iconFile = ActiveIconFile();
    Icon        *icon = SelectedIcon();

    if (icon)
        {
        IconFile_SaveUndoInfo(iconFile, kCutAction);
        
        Sel_CutToClipboard(&gEditSelection, icon);
        Display_Update(icon, kEditingIcon | kSampleIcon);   
        }
}

/*************************************************************************************/

void CVICALLBACK CopyCallback (int menuBar, int menuItem, void *callbackData, int panel)
{
    IconFile    *iconFile = ActiveIconFile();
    Icon        *icon = SelectedIcon();

    if (icon)
        {
        Sel_CopyToClipboard(&gEditSelection);
        }
}

/*************************************************************************************/

void CVICALLBACK PasteCallback (int menuBar, int menuItem, void *callbackData, int panel)
{
    IconFile    *iconFile = ActiveIconFile();
    Icon        *icon = SelectedIcon();

    if (icon)
        {
        IconFile_SaveUndoInfo(iconFile, kPasteAction);
        Sel_PasteFromClipboard(&gEditSelection,icon);
        Display_Update(icon, kEditingIcon | kSampleIcon);   
        }
}

/*************************************************************************************/

void CVICALLBACK DeleteCallback (int menuBar, int menuItem, void *callbackData, int panel)
{
    IconFile    *iconFile = ActiveIconFile();
    Icon        *icon = SelectedIcon();

    if (icon)
        {
            /* don't remember undo if the text tool has uncommitted text */
        if (GetCurrentTool() != kTextTool ||  !Sel_IsActive(&gEditSelection))
            IconFile_SaveUndoInfo(iconFile, kDeleteAction);
        
        if (!Sel_IsActive(&gEditSelection))
            {
            Sel_SelectAll(&gEditSelection, icon);
            Sel_LiftUp(&gEditSelection, icon);
            }
            
        Sel_Delete(&gEditSelection,icon);
        Display_Update(icon, kEditingIcon | kSampleIcon);   
        }
}

/*************************************************************************************/

void CVICALLBACK SelectAllCallback (int menuBar, int menuItem, void *callbackData, int panel)
{
    IconFile    *iconFile = ActiveIconFile();
    Icon        *icon = SelectedIcon();

    if (icon)
        {
        switch (GetCurrentTool())
            {
            case kSelectTool:
            case kSelectSquareTool:
                break;
            default:
                SetCurrentTool(kSelectTool);
                break;
            }

        Sel_SelectAll(&gEditSelection, icon);
            
        Display_Update(icon, kEditingIcon | kSampleIcon);   
        }
}

/*************************************************************************************/

void CVICALLBACK SetFontCallback (int menuBar, int menuItem, void *callbackData,
        int panel)
{
    Icon        *icon = SelectedIcon();
    char        currentMetaFont[256];
    char        typeface[256];
    int         bold, underline, strikeout, italic, size;

    if (icon)
        {
        TextTool_GetMetaFont(currentMetaFont);
        GetMetaFontInfo(currentMetaFont, typeface, &bold, &underline, &strikeout, &italic, &size);
        FontSelectPopup ("Select Text Tool Font", "Sample Text", 0, typeface,
                         &bold, &underline, &strikeout, &italic, NULL, NULL,
                         &size, 3, 48, 1, 0);
        CreateMetaFont ("IconEditorMetaFont", typeface, size, bold, italic, underline, strikeout);
        TextTool_SetMetaFont(icon, "IconEditorMetaFont");
        }
}
