#ifndef ICONEDIT_HEADER
#define ICONEDIT_HEADER

#include "toolbox.h"

extern int  mainPanel, editPanel, sampleIconPanel, colorPalettePanel;

#define kMaxIconDimension   48
#define kMaxIconNameLength  48
#define kUntitledStr        "[Untitled]"   


#define SQUARE_ICONS_ONLY   /* the flip and rotate and load and save code must be modified to support non-square icons */ 

typedef enum
    {
    kUnsupportedIcon = -1,
    kFirstIconType  = 0,        /* supported types must start at zero and proceed without gaps so they can be iterated through */      
    k32By32_16ColorIcon = kFirstIconType,       
    k32By32_2ColorIcon,     
    k32By32_256ColorIcon,       

    k32By16_2ColorIcon,
    k32By16_16ColorIcon,
    k32By16_256ColorIcon,

    k32By48_2ColorIcon,
    k32By48_16ColorIcon,
    k32By48_256ColorIcon,

    k16By16_2ColorIcon,
    k16By16_16ColorIcon,
    k16By16_256ColorIcon,
    
    k16By32_2ColorIcon,
    k16By32_16ColorIcon,
    k16By32_256ColorIcon,
    
    k16By48_2ColorIcon,
    k16By48_16ColorIcon,
    k16By48_256ColorIcon,

    k48By16_2ColorIcon,
    k48By16_16ColorIcon,
    k48By16_256ColorIcon,

    k48By32_2ColorIcon,
    k48By32_16ColorIcon,
    k48By32_256ColorIcon,

    k48By48_2ColorIcon,
    k48By48_16ColorIcon,
    k48By48_256ColorIcon,

    kLastIconType           = k48By48_256ColorIcon
    } IconType;

    
typedef struct
    {
    int     numColors;
    int     colorDepth;             
    int     height;
    int     width;
    int     pixelDataSizeInFile;
    int     maskSizeInFile;
    int     pixelRowSize;
    int     maskRowSize;
    char    name[kMaxIconNameLength];
    }   IconDescription;

typedef enum
    {
    kUndoSavedState = 0,    /* used as an array index into  icon.pixels.savedColorTableIndex[] */
    kTextUndoSavedState,
    kTempState,
    kNumSavedStates,
    } SavedState;
    
typedef struct 
    {
    int     colorIndex;             /* actual color value of the pixel */
    int     savedColorTableIndex[kNumSavedStates];  /* saved values of colorIndex */
    } Pixel;

typedef struct
    {
    IconDescription descr;
    IconType        type;
    int             pixelWidth;     /* width of a icon pixel in the editing area (in pixels) */
    int             pixelHeight;    /* height of a icon pixel in the editing area (in pixels) */
    int             *colorTable;
    Pixel           pixels[kMaxIconDimension][kMaxIconDimension];
    }  Icon;
    
typedef struct 
    {
    int         canRedo;
    int         canUndo;
    int         lastUndoableAction;
    int         needsSave;
    ListType    iconList;           /* list of pointers to Icons in the this file */
    Icon        *selectedIcon;      /* should point to an icon in the icon list, or NULL */
    char        fileName[MAX_PATHNAME_LEN]; 
    } IconFile; 


typedef     IconType (*IconTypeFunction)(int height, int width, int numberOfColors);

typedef enum    
    {
    kNoAction = 0,
    kSelectionMoveAction,
    kDrawingAction,
    kCutAction,
    kPasteAction,
    kDeleteAction,
    kRedoAction,
    kTextDrawingAction
    } UndoableAction;

void        FatalError(char *msg, int uiErrorCode); 

IconFile    *ActiveIconFile(void);    
Icon        *SelectedIcon(void);

IconFile    *IconFile_New(void); 
void        IconFile_Dispose(IconFile *iconFile);
void        IconFile_UpdateWindowTitle(IconFile *iconFile);

IconType    Icon_DescriptionToType(int height, int width, int numberOfColors);
int         Icon_TypeToDescription(IconType type, IconDescription *description);
Icon        *Icon_New(IconFile *iconFile, int type);
void        IconFile_SaveUndoInfo(IconFile *iconFile, UndoableAction action);

#endif /* ICONEDIT_HEADER */
