#include    <userint.h>
#include    <ansi_c.h>
#include    "iconui.h"
#include    "iconutil.h"
#include    "iconcolr.h"
#include    "iconio.h"


static void ReadIconData (FILE *file, Icon *icon, int imageOffset);
static int  WriteIconDirEntry (Icon *icon, FILE *file, int bytesInRes, int imageOffset);
static int  WriteIconData (Icon *icon, FILE *file);

extern int  editPanel, sampleIconPanel, mainPanel;

#define kHeaderSize         6
#define kIconDirEntrySize   16
#define kImageHeaderSize    40


/*********************************************************************************/

static void ReadIconData (FILE *file, Icon *icon, int imageOffset)
{
    int             i, j, k;
    unsigned char   readByte;
    int             maskStart;
    int             dataStart;
    int             row;
    
        /* move to where the image offset is stored, then move to start of image */
    fseek (file, imageOffset + kImageHeaderSize, SEEK_SET);

        /* read the color table */
    fread (icon->colorTable, icon->descr.numColors * sizeof(int), 1, file);

        /* clear out icon data */
    for (j= 0; j < icon->descr.height; j++)
        for (i=0; i < icon->descr.width; i++)
            icon->pixels[i][j].colorIndex = kNotAColorIndex;

        /* Seek past image to mask and read it */
    fseek (file, icon->descr.pixelDataSizeInFile, SEEK_CUR);
    maskStart = ftell(file);
    for (row = 1, j= icon->descr.height - 1; j >= 0; row++, j--)
        {
        for (i=0; i < icon->descr.width; i+=8)
            {
            fread (&readByte, 1, 1, file);
            for (k=0; k<=7; k++)
                if ((readByte >> k) & 1)
                    icon->pixels[i+7-k][j].colorIndex = kTransparentColorIndex;
            }
            
        fseek (file, maskStart + icon->descr.maskRowSize * row, SEEK_SET);
        }

        /* set pixels to color index (seek back past mask and image) */
    fseek (file, -1 * (icon->descr.maskSizeInFile + icon->descr.pixelDataSizeInFile), SEEK_CUR);
    dataStart = ftell(file);
    switch (icon->descr.colorDepth)
        {
        case 1:
            for (row = 1, j= icon->descr.height - 1; j >= 0; row++, j--)
                {
                for (i=0; i < icon->descr.width; i+=8)
                    {
                    fread (&readByte, 1, 1, file);
                    for (k=0; k<=7; k++)
                        if (icon->pixels[i+7-k][j].colorIndex == kNotAColorIndex) 
                            {
                            if ((readByte >> k) & 1)
                                icon->pixels[i+7-k][j].colorIndex = 1;
                            else
                                icon->pixels[i+7-k][j].colorIndex = 0;
                            }
                    }
                fseek (file, dataStart + icon->descr.pixelRowSize * row, SEEK_SET);
                }
            break;
        case 4:
            for (row = 1, j= icon->descr.height - 1; j >= 0; row++, j--)
                {
                for (i=0; i < icon->descr.width; i+=2)
                    {
                    fread (&readByte, 1, 1, file);

                    if (icon->pixels[i][j].colorIndex == kNotAColorIndex)
                        icon->pixels[i][j].colorIndex = (readByte&0xf0) >> 4;

                    if (icon->pixels[i+1][j].colorIndex == kNotAColorIndex)
                        icon->pixels[i+1][j].colorIndex = readByte & 0x0f;
                    }
                fseek (file, dataStart + icon->descr.pixelRowSize * row, SEEK_SET);
                }
            break;
        case 8:
            for (row = 1, j= icon->descr.height - 1; j >= 0; row++, j--)
                {
                for (i=0; i < icon->descr.width; i++)
                    {
                    fread (&readByte, 1, 1, file);

                    if (icon->pixels[i][j].colorIndex == kNotAColorIndex)
                        icon->pixels[i][j].colorIndex = readByte;
                    }
                fseek (file, dataStart + icon->descr.pixelRowSize * row, SEEK_SET);
                }
            break;
        }
 }
    
/*********************************************************************************/

    /*  pass NULL for fileName to cause the file dialog to prompt the user for the file name */
FileOpResult    IconFile_Load(IconFile *iconFile, char fileName[MAX_PATHNAME_LEN], int *numUnsupportedIconsNotLoaded)
{
    FILE            *file=NULL;
    FileOpResult    result = kFileOpSuccess;
    int             bytesRead, i, j, readInt;
    short           readWord, numIcons;
    unsigned char   height, width, colorsByte;
    int             colors;
    int             imageOffset;    
    IconType        iconType;
    Icon            *icon;
    int             numUnsupportedIcons = 0;
    int             fileSelectResult;
    char            fileNameToLoad[MAX_PATHNAME_LEN];
    
    if (!fileName)
        {
        fileSelectResult = FileSelectPopup ("", "*.ico", "*.ico", "Open Icon", VAL_LOAD_BUTTON, 0, 1, 1, 0, fileNameToLoad);
        if (fileSelectResult != VAL_EXISTING_FILE_SELECTED)
            {
            result = kFileOpCanceled;
            goto Error;
            }
        }
    else
        strcpy(fileNameToLoad, fileName);

    if (!iconFile->fileName[0])
        {
        result = kFileOpCanceled;
        goto Error;
        }
        
    SetWaitCursor(1);
    file = fopen (fileNameToLoad, "rb");
    if (!file)
        {
        result = kFileOpIOError;
        goto Error;
        }       

    /* first four bytes must be 00 00 01 00 */
    fread (&readInt, 4, 1, file);
    if (readInt != 0x10000)
        {
        result = kFileOpIOError;
        goto Error;
        }

    /* read all the supported icons we can understand out of the file */
    fread (&numIcons, 2, 1, file);
    for (i=1; i<=numIcons; i++)
        {
        fseek (file, kHeaderSize+(i-1)*kIconDirEntrySize, SEEK_SET);
        fread (&width, 1, 1, file);
        fread (&height, 1, 1, file);
        fread (&colorsByte, 1, 1, file);
        colors = colorsByte;
        if (colors == 0)
            colors = 256;
        
        iconType = Icon_DescriptionToType(height, width, colors);
        
        if (iconType != kUnsupportedIcon)
            {
            icon = Icon_New(iconFile, iconType);
            fseek (file, 9, SEEK_CUR);
            fread (&imageOffset, 4, 1, file);
            ReadIconData (file, icon, imageOffset);
            }
        else
            numUnsupportedIcons++;
        }

Error:
    if (file)
        fclose(file);
        
    SetWaitCursor(0);
    
    if (result == kFileOpSuccess && ListNumItems(iconFile->iconList) == 0)
        result = kFileOpIOError;    /* .ico file must have one supported image */
    
    if (result == kFileOpSuccess)
        {
        strcpy(iconFile->fileName, fileNameToLoad);
        if (numUnsupportedIconsNotLoaded)
            *numUnsupportedIconsNotLoaded = numUnsupportedIcons;
        }
        
    return result;
}

/*********************************************************************************/

static int WriteIconDirEntry (Icon *icon, FILE *file, int bytesInRes, int imageOffset)
{
    int     writeInt;
    char    writeByte;

    /* set size and colors */
    writeByte = icon->descr.width;
    fwrite (&writeByte, 1, 1, file);
    writeByte = icon->descr.height;
    fwrite (&writeByte, 1, 1, file);
    writeByte = icon->descr.numColors;
    fwrite (&writeByte, 1, 1, file);

    /* set some reserved to 0 */
    writeByte = 0;
    fwrite (&writeByte, 1, 1, file);
    writeInt = 0;
    fwrite (&writeInt, 4, 1, file);

    /* set number of bytes in icon resource */
    writeInt = bytesInRes;
    fwrite (&writeInt, 4, 1, file);

    /* offset to image from start of file */
    writeInt = imageOffset;
    fwrite (&writeInt, 4, 1, file);

    return 0;
}
    
/*********************************************************************************/

static int WriteIconData (Icon *icon, FILE *file)
{
    int             i, j, k;
    unsigned int    writeInt;
    unsigned char   writeByte, hiByte, lowByte;
    unsigned short  writeWord = 0;
    int             padding;
    
    /* write DIB header */
    writeInt = 0x28;        /* size of DIB header = 40 */
    fwrite (&writeInt, 4, 1, file);
    writeInt = icon->descr.width;
    fwrite (&writeInt, 4, 1, file);
    writeInt = icon->descr.height * 2;
    fwrite (&writeInt, 4, 1, file);
    writeInt = icon->descr.colorDepth << 16 | 0x0001;       /* biPlanes (WORD) = 1, biBitCount (WORD) = 4 */
    fwrite (&writeInt, 4, 1, file);
    writeInt = 0;               
    fwrite (&writeInt, 4, 1, file);                         /* compression type */
    writeInt = icon->descr.pixelDataSizeInFile;
    fwrite (&writeInt, 4, 1, file);
    writeInt = 0;                                           
    fwrite (&writeInt, 4, 1, file);                         /* xPelsPerMeter */
    writeInt = 0;                                           
    fwrite (&writeInt, 4, 1, file);                         /* yPelsPerMeter */
    writeInt = 0;                                           
    fwrite (&writeInt, 4, 1, file);                         /* colors used, zero == max for color depth */
    writeInt = 0;
    fwrite (&writeInt, 4, 1, file);                         /* num significant colors, not used when previous field is zero */

    /* write colors in color table */
    for (i=0; i < icon->descr.numColors; i++)
        {
        writeInt = icon->colorTable[i];
        fwrite (&writeInt, 4, 1, file);
        }

    /* write pixel color indices */
    switch (icon->descr.colorDepth)
        {
        case 1:
            padding = icon->descr.pixelRowSize - icon->descr.width / 8;
            for (j= icon->descr.height - 1; j >= 0; j--)
                {
                for (i=0; i < icon->descr.width; i+=8)
                    {
                    writeByte = 0;
                    for (k=0; k<=7; k++)
                        if (icon->pixels[i+7-k][j].colorIndex & 0x01)
                            writeByte |= (1 << k);
                    fwrite (&writeByte, 1, 1, file);
                    }
                   
                if (padding > 0)
                    {
                    writeByte = 0;
                    for (i = 0; i < padding; i++)           /* pad to 4 byte boundary */
                        fwrite (&writeByte, 1, 1, file);
                    }
                }
            break;
        case 4:
            for (j= icon->descr.height - 1; j >= 0; j--)
                for (i=0; i < icon->descr.width; i+=2)
                    {
                    hiByte = icon->pixels[i][j].colorIndex == kTransparentColorIndex ? 0 : icon->pixels[i][j].colorIndex;
                    lowByte = icon->pixels[i + 1][j].colorIndex == kTransparentColorIndex ? 0 : icon->pixels[i + 1][j].colorIndex;
                    writeByte = hiByte << 4 | lowByte;
                    fwrite (&writeByte, 1, 1, file);
                    }
            break;
        case 8:
            for (j= icon->descr.height - 1; j >= 0; j--)
                for (i=0; i < icon->descr.width; i++)
                    {
                    writeByte = icon->pixels[i][j].colorIndex == kTransparentColorIndex ? 0 : icon->pixels[i][j].colorIndex;
                    fwrite (&writeByte, 1, 1, file);
                    }
            break;
        }
        
    /* write mask */
    padding = icon->descr.maskRowSize - icon->descr.width / 8;
    for (j= icon->descr.height - 1; j >= 0; j--)
        {
        for (i=0; i < icon->descr.width; i+=8)
            {
            writeByte = 0;
            for (k=0; k<=7; k++)
                if (icon->pixels[i+7-k][j].colorIndex == kTransparentColorIndex)
                    writeByte |= (1 << k);
            fwrite (&writeByte, 1, 1, file);
            }
         
        if (padding)
            {
            writeByte = 0;
            for (i = 0; i < padding; i++)           /* pad to 4 byte boundary */ 
                fwrite (&writeByte, 1, 1, file);
            }
        }

    return 0;            
}
    
    
/*********************************************************************************/

int IconFile_Save(IconFile *iconFile, int popup)
{
    FILE            *file=NULL;
    int             selected, writeInt;
    int             currentUntitled = !strcmp(iconFile->fileName, kUntitledStr);
    int             bytesInResource;
    int             imageOffset;
    int             numIcons;
    int             index;
    Icon            *icon;
    
    if (popup || currentUntitled)
        selected = FileSelectPopup ("", currentUntitled ? "*.ico" : iconFile->fileName, "*.ico", "Save Icon", VAL_SAVE_BUTTON, 0, 1, 1, 1, iconFile->fileName);
    else
        selected = VAL_EXISTING_FILE_SELECTED;
    if ((selected == VAL_EXISTING_FILE_SELECTED) ||
            (selected == VAL_NEW_FILE_SELECTED))
        {
        SetWaitCursor(1);
        file = fopen (iconFile->fileName, "wb");

        /* first four bytes must be 00 00 01 00 */
        writeInt = 0x10000;
        fwrite (&writeInt, 4, 1, file);

        numIcons = ListNumItems(iconFile->iconList);
        fwrite (&numIcons, 2, 1, file);

        imageOffset = kHeaderSize + kIconDirEntrySize * numIcons;
        
        for (index = 1; index <= numIcons; index++)
            {
            ListGetItem(iconFile->iconList, &icon, index);
            
            bytesInResource = kImageHeaderSize + (icon->descr.numColors * sizeof(int)) + icon->descr.pixelDataSizeInFile + icon->descr.maskSizeInFile;
        
            WriteIconDirEntry (icon, file, bytesInResource, imageOffset);
            imageOffset += bytesInResource;
            }
            
        for (index = 1; index <= numIcons; index++)
            {
            ListGetItem(iconFile->iconList, &icon, index);
            WriteIconData (icon, file);
            }
            
            
        IconFile_UpdateWindowTitle(iconFile);
        }

    if (file)
        fclose(file);
    SetWaitCursor(0);
    
    return selected;
}

/*********************************************************************************/

/* returns 1 if the action was cancelled */
int IconFile_PromptToSave(IconFile *iconFile)
{
    char    messageBuf[300];
    int     selected;

    sprintf(messageBuf, "Do you want to save changes to %s?", iconFile->fileName);
    selected = GenericMessagePopup ("Icon Editor Message", messageBuf,"__Yes", "__No", "Cancel", NULL, 0, 0,
                                    VAL_GENERIC_POPUP_BTN1,
                                    VAL_GENERIC_POPUP_BTN1,
                                    VAL_GENERIC_POPUP_BTN3);
    if (selected == VAL_GENERIC_POPUP_BTN1)
        return !IconFile_Save(iconFile, FALSE);
        
    return (selected == VAL_GENERIC_POPUP_BTN3);
}
