#include    <userint.h>
#include    <utility.h>
#include    <ansi_c.h>
#include    "iconedit.h"
#include    "icontool.h"
#include    "iconui.h"
#include    "iconutil.h"
#include    "iconcolr.h"

typedef struct 
    {
    char            metaFont[256];
    int             fontHeight;
    char            text[kMaxIconDimension + 1];        /* up to a ridiculous worst case of one pixel characters */
    int             numChars;
    Point           position;
    } TextTool;
    
TextTool    gTextTool;

typedef struct
    {
    Icon        *icon;
    IconDisplay destination;
    } IconSetGetPixelData;      

typedef struct
    {
    PixelDisplayState   state[kMaxIconDimension][kMaxIconDimension];
    } Display;


static unsigned char    checkerPat[8] = {0xAA, 0x55, 0xAA, 0x55, 0xAA, 0x55, 0xAA, 0x55};
static unsigned char    selectionPat[8] = {0x99, 0x33, 0x66, 0xCC, 0x99, 0x33, 0x66, 0xCC};
static unsigned char    solidPat[8] = {0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF, 0xFF};

Selection   gEditSelection;  
Display     gDisplay;

static gGridVisible = TRUE;

static void GenericFloodFillPoint(Point p);
void        GenericFloodFill(void *callbackData, Point fillStart, Rect floodLimits, GenericGetPixelCallback getPixel, GenericSetPixelCallback setPixel, int fillValue);

static void Display_DrawSamplePixel(Display *display, Icon *icon, Point p, int colorIndex);
static void Display_DrawEditPixel(Display *display, Icon *icon, Point p, int colorIndex, int leaveRoomForGrid);
static void Display_UpdateEditWindow(Display *display, Icon *icon);
static void Display_UpdateSample(Display *display, Icon *icon);
static void Display_InvalPixelColor(Display *display, Icon *icon,  int colorIndexToInval);
static void Display_InvalPixel(Display *display, Icon *icon, Point p, int invalGrid);
static void Display_UpdateSelection(Display *display, Icon *icon, Selection *sel);

static void Icon_PlotEllipsePoints (Icon *icon, int xc, int yc, int x, int y, int colorIndex, IconDisplay destination, int fill, int oddX, int oddY);
static void Icon_InvalSampleDisplay(Display *display, Icon *icon, int colorIndexToInval);
static void Icon_SetPixelCallback(void *callbackData, Point p, int value);
static int  Icon_GetPixelCallback(void *callbackData, Point p);

static void Sel_CalcInterior(Selection *sel);
static void Sel_DeselectTransparentPixels(Selection *sel, Icon *icon);
static void Sel_CopyToTemp(Selection *sel);
static int  Sel_FindUnselectedEdgePoint(Selection *sel, Point *start);
static int  Sel_GetPixelCallback(void *callbackData, Point p);
static void Sel_SavePixelColors(Selection *sel, Icon *icon);
static void Sel_SaveBGPixelColors(Selection *sel, Icon *icon);
static void Sel_RestoreBGPixelColors(Selection *sel, Icon *icon);
static void Sel_PasteCanvasContents(Selection *sel, Icon *icon, int panel, int canvas);
static void Sel_SetText(Selection *sel, Icon *icon, char *text, Point position);
static int  Sel_VisibleAtPosition(Selection *sel, Icon *icon, Point position);

static int Clipboard_GetCanvas(int *clipboardPanel, int *clipboardCanvas);

/***************************************************************************/

int Grid_Visible(void)
{   
    return gGridVisible;
}

/***************************************************************************/

void Grid_SetVisible(int visible)
{
    gGridVisible = visible;
    Grid_Invalidate();
}

/***************************************************************************/

void Grid_Invalidate(void)
{
    int     x, y;
    Icon    *icon = SelectedIcon();

    if (icon)
        for (x = 0; x < icon->descr.width; x++)
            for (y = 0; y < icon->descr.height; y++)
                gDisplay.state[x][y].displayedGrid = kGridInvalled;
}

/***************************************************************************/

static void Display_DrawEditPixel(Display *display, Icon *icon, Point p, int colorIndex, int leaveRoomForGrid)
{
    int     color = ColorSelect_GetIndexColor(&gColorSelector, colorIndex);
    int     gridAdd = leaveRoomForGrid ? 1 : 0;
    Rect    r;
    
    RectSet(&r, p.y*icon->pixelHeight, p.x*icon->pixelWidth, icon->pixelHeight - gridAdd, icon->pixelWidth - gridAdd);
    SetCtrlAttribute(editPanel, EDIT_ICON_CANVAS, ATTR_PEN_FILL_COLOR, color);
    CanvasDrawRect(editPanel, EDIT_ICON_CANVAS, r, VAL_DRAW_INTERIOR);
    display->state[p.x][p.y].displayedIndex = colorIndex;
    if (!leaveRoomForGrid)
        display->state[p.x][p.y].displayedGrid = kGridHidden;   /* just drew over the grid area */
    display->state[p.x][p.y].displayedSelection = kSelNone;     
}

/***************************************************************************/

static void Display_DrawSamplePixel(Display *display, Icon *icon, Point p, int colorIndex)
    {
    int     color = ColorSelect_GetIndexColor(&gColorSelector, colorIndex);

    SetCtrlAttribute(sampleIconPanel, SAMPLE_CANVAS, ATTR_PEN_COLOR, color);
    CanvasDrawPoint(sampleIconPanel, SAMPLE_CANVAS, p);
    display->state[p.x][p.y].displayedSampleIndex = colorIndex;
    }

/*************************************************************************************/

void Display_AnimateSelection(Icon *icon, Selection *sel)
{
    unsigned char   temp;
    
    temp = selectionPat[7];
    memmove(selectionPat + 1, selectionPat, 7);
    selectionPat[0] = temp;
        
    if (Sel_IsActive(sel))
        {
        Sel_Invalidate(sel);
        StartIconBatchDraw(kEditingIcon);
        Display_UpdateSelection(&gDisplay, icon, sel);
        EndIconBatchDraw(kEditingIcon);
        }
}

/*************************************************************************************/

static void Display_UpdateSelection(Display *display, Icon *icon, Selection *sel)
{
    SelAppearance   selApp;
    Point           start, end;
    int             xCoord; 
    int             yCoord;
    int             gridVisible = Grid_Visible();
    int             gridAdd =  gridVisible ? 1 : 0;
    int             pixelWidth = icon->pixelWidth - gridAdd - 2;
    int             pixelHeight = icon->pixelHeight - gridAdd - 2;
    Rect            r;
    int             x, y;
    int             selX, selY;
    int             width = icon->descr.width;
    int             height = icon->descr.height;
   
        /*  clear out any selection that isn't current drawn correctly */
   for (x = 0; x < width; x++)
        for (y = 0; y < height; y++)
            {
            selX = x - sel->position.x;
            selY = y - sel->position.y;
            
            if (selY >= 0 && selY < height && selX >= 0 && selX < width)
                selApp = sel->selApp[selX][selY] & ~kSelInvalled;
            else
                selApp = kSelNone;

            if (display->state[x][y].displayedSelection != selApp)
                Display_DrawEditPixel(display, icon, MakePoint(x,y), icon->pixels[x][y].colorIndex, gridVisible);
            }           
                
        /*  draw all of the selection that isn't already drawn */
    SetCtrlAttribute(editPanel, EDIT_ICON_CANVAS, ATTR_PEN_FILL_COLOR, 0xD0D0FF);
    SetCtrlAttribute(editPanel, EDIT_ICON_CANVAS, ATTR_PEN_PATTERN, selectionPat);      

    for (x = 0; x < width; x++)
        for (y = 0; y < height; y++)
            {
            selX = x - sel->position.x;
            selY = y - sel->position.y;
            
            if (selY >= 0 && selY < height && selX >= 0 && selX < width)
                selApp =  sel->selApp[selX][selY];
            else
                selApp = kSelNone;

            if (display->state[x][y].displayedSelection != selApp)
                {
                xCoord = x * icon->pixelWidth + 1;
                yCoord = y * icon->pixelHeight + 1;
    
                if (selApp == kSelUnknown)
                    {
                    RectSet(&r, yCoord, xCoord, pixelHeight, pixelWidth);
                    CanvasDrawRect(editPanel, EDIT_ICON_CANVAS, r, VAL_DRAW_INTERIOR);
                    }
                else
                    {
                    if (selApp & kSelTop)
                        CanvasDrawRect(editPanel, EDIT_ICON_CANVAS, MakeRect(yCoord - 1, xCoord - 1, 2, pixelWidth + 2), VAL_DRAW_INTERIOR);
    
                    if (selApp & kSelRight)
                        CanvasDrawRect(editPanel, EDIT_ICON_CANVAS, MakeRect(yCoord - 1, xCoord + pixelWidth - 1,
                                    pixelHeight + 2, 2), VAL_DRAW_INTERIOR);

                    if (selApp & kSelBottom)
                        CanvasDrawRect(editPanel, EDIT_ICON_CANVAS, MakeRect(yCoord + pixelHeight - 1, xCoord - 1,
                                    2, pixelWidth + 2), VAL_DRAW_INTERIOR);

                    if (selApp & kSelLeft)
                        CanvasDrawRect(editPanel, EDIT_ICON_CANVAS, MakeRect(yCoord - 1, xCoord - 1, 
                                    pixelHeight + 2, 2), VAL_DRAW_INTERIOR);
                    }
                }
                
            display->state[x][y].displayedSelection = selApp & ~kSelInvalled;
            }

    SetCtrlAttribute(editPanel, EDIT_ICON_CANVAS, ATTR_PEN_PATTERN, solidPat);      
}

/***************************************************************************/

void Icon_SetPixel(Icon *icon, Point p, int colorIndex, IconDisplay destination)
{
    icon->pixels[p.x][p.y].colorIndex = colorIndex;
    
    if (destination & kEditingIcon)
        Display_DrawEditPixel(&gDisplay, icon, p, colorIndex, Grid_Visible());

    if (destination & kSampleIcon)
        Display_DrawSamplePixel(&gDisplay, icon, p, colorIndex);
}

/***************************************************************************/

void Display_Update(Icon *icon, IconDisplay which)
{
    if (which & kSampleIcon)        /* sample uses delayed drawing mode, so update it first, and they will appear to update at the same time */
        Display_UpdateSample(&gDisplay, icon);

    if (which & kEditingIcon)
        Display_UpdateEditWindow(&gDisplay, icon);
}

/***************************************************************************/

static void Display_UpdateGrid(Display *display, Icon *icon)
{
    Rect                    r;
    int                     pixelHeight = icon->pixelHeight;
    int                     pixelWidth = icon->pixelWidth;
    int                     x, y;
    int                     height = icon->descr.height;
    int                     width = icon->descr.width;
    int                     expectedGridState;
    int                     displayedState;

    if (Grid_Visible())
        {
        SetCtrlAttribute(editPanel, EDIT_ICON_CANVAS, ATTR_PEN_FILL_COLOR, VAL_GRAY);
        SetCtrlAttribute(editPanel, EDIT_ICON_CANVAS, ATTR_PEN_PATTERN, checkerPat);        
        
        expectedGridState = kGridShown;
        }
    else
        expectedGridState = kGridHidden;
        
    for (x = 0; x < width; x++)
        for (y = 0; y < height; y++)
            {
            displayedState = display->state[x][y].displayedGrid;
            if (displayedState != expectedGridState)
                {
                if (expectedGridState == kGridShown)
                    {
                    display->state[x][y].displayedGrid = kGridShown;
                    RectSet(&r, y * pixelHeight - 1, x * pixelWidth - 1, pixelHeight + 2, 1);
                    CanvasDrawRect(editPanel, EDIT_ICON_CANVAS, r, VAL_DRAW_INTERIOR);
                    r.width = pixelWidth + 2;
                    r.height = 1;
                    CanvasDrawRect(editPanel, EDIT_ICON_CANVAS, r, VAL_DRAW_INTERIOR);
                    }
                else
                    {
                    Display_DrawEditPixel(display, icon, MakePoint(x,y), icon->pixels[x][y].colorIndex, FALSE);
                    }
                }
            }
            
    SetCtrlAttribute(editPanel, EDIT_ICON_CANVAS, ATTR_PEN_PATTERN, solidPat);      
}       

/***************************************************************************/

static void Display_UpdateEditPixelColors(Display *display, Icon *icon)
{
    int x, y;
    int width = icon->descr.width;
    int height = icon->descr.height;
    int colorIndex;
    int gridVisible;
    
    gridVisible = Grid_Visible();

    for (x = 0; x < width; x++)
        for (y = 0; y < height; y++)
            {
            colorIndex = icon->pixels[x][y].colorIndex;
            if (display->state[x][y].displayedIndex != colorIndex)
                Display_DrawEditPixel(display, icon, MakePoint(x,y), colorIndex, gridVisible);
            }
}

/***************************************************************************/

static void Display_UpdateEditWindow(Display *display, Icon *icon)
{
    StartIconBatchDraw(kEditingIcon);
                
    Display_UpdateEditPixelColors(display, icon);
    Display_UpdateGrid(display, icon);
        
    Display_UpdateSelection(display, icon, &gEditSelection);                
    EndIconBatchDraw(kEditingIcon);
}

/***************************************************************************/

static void Display_UpdateSample(Display *display, Icon *icon)
{
    int x, y;
    
    StartIconBatchDraw(kSampleIcon);
    for (x = 0; x < icon->descr.width; x++)
        for (y = 0; y < icon->descr.height; y++)
            if (display->state[x][y].displayedSampleIndex != icon->pixels[x][y].colorIndex)
                Display_DrawSamplePixel(display, icon, MakePoint(x,y), icon->pixels[x][y].colorIndex);                  
    EndIconBatchDraw(kSampleIcon);
}

/***************************************************************************/

    /* pass kNotAColorIndex for colorIndexToInval to invalidate all pixels */
void Icon_InvalDisplay(Icon *icon, IconDisplay which, int colorIndexToInval)
{
    if (which & kEditingIcon)
        Display_InvalPixelColor(&gDisplay, icon, colorIndexToInval);
        
    if (which & kSampleIcon)
        Icon_InvalSampleDisplay(&gDisplay, icon, colorIndexToInval);
}

/***************************************************************************/

    /* pass kNotAColorIndex for colorIndexToInval to invalidate all pixels */
static void Display_InvalPixelColor(Display *display, Icon *icon, int colorIndexToInval)
{
    int x, y;
    int width = icon->descr.width;
    int height = icon->descr.height;

        for (x = 0; x < width; x++)
            for (y = 0; y < height; y++)
                if ((colorIndexToInval == kNotAColorIndex) || (icon->pixels[x][y].colorIndex == colorIndexToInval))
                    display->state[x][y].displayedIndex = kNotAColorIndex;
}

/***************************************************************************/

static void Display_InvalPixel(Display *display, Icon *icon, Point p, int invalGrid)
{
    display->state[p.x][p.y].displayedIndex = kNotAColorIndex;
    if (invalGrid)
        gDisplay.state[p.x][p.y].displayedGrid = kGridInvalled;
}

/***************************************************************************/

    /* pass kNotAColorIndex for colorIndexToInval to invalidate all pixels */
static void Icon_InvalSampleDisplay(Display *display, Icon *icon, int colorIndexToInval)
{
    int x, y;
    
    for (x = 0; x < icon->descr.width; x++)
        for (y = 0; y < icon->descr.height; y++)
            if ((colorIndexToInval == kNotAColorIndex) || (icon->pixels[x][y].colorIndex == colorIndexToInval))
                display->state[x][y].displayedSampleIndex = kNotAColorIndex;                    
}

/***************************************************************************/

void Icon_SaveState(Icon *icon, SavedState state)
{
    int x, y;

    for (x = 0; x < icon->descr.width; x++)
        for (y = 0; y < icon->descr.height; y++)
            icon->pixels[x][y].savedColorTableIndex[state] = icon->pixels[x][y].colorIndex;
}

/***************************************************************************/

void Icon_RestoreState(Icon *icon, SavedState whichState, int saveCurrentState)
{
    int x, y;
    int currentColorIndex;

    for (x = 0; x < icon->descr.width; x++)
        for (y = 0; y < icon->descr.height; y++)
            {
            if (saveCurrentState)
                currentColorIndex = icon->pixels[x][y].colorIndex;
                
            icon->pixels[x][y].colorIndex = icon->pixels[x][y].savedColorTableIndex[whichState];
            
            if (saveCurrentState)
                icon->pixels[x][y].savedColorTableIndex[whichState] = currentColorIndex;
            }   
}

/***************************************************************************/

static void Icon_SetPixelCallback(void *callbackData, Point p, int value)
{
    IconSetGetPixelData *pixelData = (IconSetGetPixelData *)callbackData;
    
    Icon_SetPixel(pixelData->icon, p, value, pixelData->destination);
}

/***************************************************************************/

static int Icon_GetPixelCallback(void *callbackData, Point p)
{
    IconSetGetPixelData *pixelData = (IconSetGetPixelData *)callbackData;

    return pixelData->icon->pixels[p.x][p.y].colorIndex;
}

/***************************************************************************/

void Icon_DrawLine (Icon *icon, Point start, Point end, int colorIndex, IconDisplay destination)
{
    IconSetGetPixelData pixelData;
    
    pixelData.icon = icon;
    pixelData.destination = destination;
    
    GenericDrawLine(start, end, Icon_SetPixelCallback, &pixelData, colorIndex);
}   

/***************************************************************************/
    
void GenericDrawLine (Point start, Point end, GenericSetPixelCallback setPixel, void *callbackData, int value)
{
    int     p;
    int     deltaX, deltaY, maxX, maxY;
    double  slope;
    int     i, yInc, xInc;
    int     lineX, lineY;

    deltaX = start.x - end.x;
    deltaY = start.y - end.y;
    if (!deltaX)
        slope = 999.0;
    else
        slope = (double)deltaY/(double)deltaX;
    deltaX = abs(deltaX);
    deltaY = abs(deltaY);

    if (slope >= -1 && slope <= 1)
        {
        if (start.x < end.x)
            {
            lineX = start.x;
            lineY = start.y;
            yInc = start.y < end.y ? 1 : -1;
            maxX = end.x;
            }
        else
            {
            lineX = end.x;
            lineY = end.y;
            yInc = start.y < end.y ? -1 : 1;
            maxX = start.x;
            }

        setPixel(callbackData, MakePoint(lineX, lineY), value);
        p = 2*deltaY - deltaX;
        i = 0;
        
        while (lineX + i  < maxX)
            {
            i++;
            if (p < 0)
                {
                p += 2*deltaY;
                }
            else
                {
                lineY += yInc;
                p += 2 * (deltaY - deltaX);
                }
            setPixel(callbackData, MakePoint(lineX + i, lineY), value); 
            }
        }
    else
        {
        if (start.y < end.y)
            {
            lineY = start.y;
            lineX = start.x;
            xInc = start.x < end.x ? 1 : -1;
            maxY = end.y;
            }
        else
            {
            lineY = end.y;
            lineX = end.x;
            xInc = start.x < end.x ? -1 : 1;
            maxY = start.y;
            }
            
        setPixel(callbackData, MakePoint(lineX, lineY), value); 
        p = 2*deltaX - deltaY;
        i = 0;
        
        while (lineY + i  < maxY)
            {
            i++;
            if (p < 0)
                {
                p += 2*deltaX;
                }
            else
                {
                lineX += xInc;
                p += 2 * (deltaX - deltaY);
                }
            setPixel(callbackData, MakePoint(lineX, lineY + i), value); 
            }
        }
}

/***************************************************************************/

void Icon_DrawRect(Icon *icon, Point start, Point end, int colorIndex, IconDisplay destination, int fill)
{
    IconSetGetPixelData pixelData;
    
    pixelData.icon = icon;
    pixelData.destination = destination;
    
    GenericDrawRect(start, end, fill, Icon_SetPixelCallback, &pixelData, colorIndex);
}

/***************************************************************************/

void GenericDrawRect(Point start, Point end, int fill, GenericSetPixelCallback setPixel, void *callbackData, int value)
{
    int i, j, temp;

    if (start.x > end.x)
        {
        temp = start.x;
        start.x = end.x;
        end.x = temp;
        }

    if (start.y > end.y)
        {
        temp = start.y;
        start.y = end.y;
        end.y = temp;
        }
        
    if (!fill)
        {
        for (i=start.x; i<=end.x; i++)
            {
            setPixel(callbackData, MakePoint(i, start.y), value); 
            setPixel(callbackData, MakePoint(i, end.y), value); 
            }
            
        for (i=start.y+1; i<=end.y-1; i++)
            {
            setPixel(callbackData, MakePoint(start.x, i), value); 
            setPixel(callbackData, MakePoint(end.x, i), value); 
            }
        }
    else
        {
        for (i=start.x; i<=end.x; i++)
            for (j=start.y; j<=end.y; j++)
                setPixel(callbackData, MakePoint(i, j), value); 
        }
}

/***************************************************************************/

void Icon_FloodFill(Icon *icon, Point p, int colorIndex, int colorIndexToReplace, IconDisplay destination)
{
    IconSetGetPixelData pixelData;
    
    pixelData.icon = icon;
    pixelData.destination = destination;
    
    GenericFloodFill(&pixelData, p, MakeRect(0,0, icon->descr.height, icon->descr.width), 
        Icon_GetPixelCallback, Icon_SetPixelCallback, colorIndex);
}

/***************************************************************************/

typedef struct  
    {
    int                     floodFillStackPtr;
    Rect                    floodLimits;    
    Point                   fillStack[kMaxIconDimension * kMaxIconDimension];
    GenericGetPixelCallback getPixel;
    GenericSetPixelCallback setPixel;
    void                    *callbackData;
    int                     fillValue;
    int                     valueToReplace;
    } FloodInfo;
    
static FloodInfo floodInfo;

static void GenericFloodFillPoint(Point p)
{
    if (RectContainsPoint(floodInfo.floodLimits, p))
        if (floodInfo.getPixel(floodInfo.callbackData, p) == floodInfo.valueToReplace)
            {
            floodInfo.setPixel(floodInfo.callbackData, p, floodInfo.fillValue);
            floodInfo.floodFillStackPtr++;
            floodInfo.fillStack[floodInfo.floodFillStackPtr] = p;
            }
}

/*********/

    /* Note: getPixel and setPixel will be called with the same callbackData and callbackFlags */
void GenericFloodFill(void *callbackData, Point fillStart, Rect floodLimits, GenericGetPixelCallback getPixel, GenericSetPixelCallback setPixel, int fillValue)
{
    int     currPtr = 0;
    int     valueToReplace;
    Point   nextPoint;
    Point   neighbor;
    
    floodInfo.floodFillStackPtr = -1;
    floodInfo.floodLimits = floodLimits;
    floodInfo.getPixel = getPixel;
    floodInfo.setPixel = setPixel;
    floodInfo.callbackData = callbackData;
    floodInfo.fillValue = fillValue;
    floodInfo.valueToReplace = getPixel(callbackData, fillStart);

    if(floodInfo.valueToReplace == floodInfo.fillValue)
        return;

    GenericFloodFillPoint(fillStart);

    while (currPtr <= floodInfo.floodFillStackPtr)
        {
        nextPoint = floodInfo.fillStack[currPtr];

        PointSet(&neighbor, nextPoint.x - 1, nextPoint.y);
        GenericFloodFillPoint(neighbor);
        PointSet(&neighbor, nextPoint.x + 1, nextPoint.y);
        GenericFloodFillPoint(neighbor);
        PointSet(&neighbor, nextPoint.x, nextPoint.y - 1);
        GenericFloodFillPoint(neighbor);
        PointSet(&neighbor, nextPoint.x, nextPoint.y + 1);
        GenericFloodFillPoint(neighbor);
        currPtr++;
        }
}

/***************************************************************************/

static void Icon_PlotEllipsePoints (Icon *icon, int xc, int yc, int x, int y, int colorIndex, IconDisplay destination, int fill, int oddX, int oddY)
    {
    if (fill)
        {
        int i;

        for (i = xc - x; i <= xc + x + oddX; i++)
            {
            Icon_SetPixel(icon, MakePoint(i, yc + y + oddY), colorIndex, destination);
            Icon_SetPixel(icon, MakePoint(i, yc - y), colorIndex, destination);
            }
        }
    else
        {
        Icon_SetPixel(icon, MakePoint(xc + x + oddX, yc + y + oddY), colorIndex, destination);
        Icon_SetPixel(icon, MakePoint(xc + x + oddX, yc - y), colorIndex, destination);
        Icon_SetPixel(icon, MakePoint(xc - x, yc + y + oddY), colorIndex, destination);
        Icon_SetPixel(icon, MakePoint(xc - x, yc - y), colorIndex, destination);
        }
    }

/***************************************************************************/

void Icon_DrawOval(Icon *icon, Point start, Point end, int colorIndex, IconDisplay destination, int fill)
{
    double  a, b, d, da, db, xc, yc, x, y, newY;
    int     oddX, oddY;

    if (start.x < end.x)
        {
        a = (end.x - start.x)/2;
        oddX = (end.x - start.x)%2 ? 1 : 0;
        xc = start.x + a;
        }
    else
        {
        a = (start.x - end.x)/2;
        oddX = (start.x - end.x)%2 ? 1 : 0;
        xc = end.x + a;
        }
        
    if (start.y < end.y)
        {
        b = (end.y - start.y)/2;
        oddY = (end.y - start.y)%2 ? 1 : 0;
        yc = start.y + b;
        }
    else
        {
        b = (start.y - end.y)/2;
        oddY = (start.y - end.y)%2 ? 1 : 0;
        yc = end.y + b;
        }

    x = a;
    y = 0;
    Icon_PlotEllipsePoints (icon, xc, yc, x, y, colorIndex, destination, fill, oddX, oddY);
    da = 4*a*a*b*b - b*b*(4*x*x-4*x + 1) -
            a*a*(4*y*y + 8*y + 4);
    d = -2*b*b*a + 2*a*a + b*b;
    while (d < 0 && x > 0)
        {
        if (da > 0)
            {
            y++;
            d = d + 2 * a * a;
            }
        else
            {
            x--;
            y++;
            d = d + 2 * a * a + 2 * b * b;
            }
        Icon_PlotEllipsePoints (icon, xc, yc, x, y, colorIndex, destination, fill, oddX, oddY);
        da = 4*a*a*b*b - b*b*(4*x*x-4*x + 1) -
                a*a*(4*y*y + 8*y + 4);
        }
    db = 4*a*a*b*b - b*b*(4*x*x - 8*x + 4) -
            a*a*(4*y*y + 4*y + 1);
    while (x > 0)
        {
        if (db > 0)
            {
            x--;
            y++;
            newY = 1;
            }
        else
            {
            x--;
            newY = 0;
            }
        if (newY || !fill)
            Icon_PlotEllipsePoints (icon, xc, yc, x, y, colorIndex, destination, fill, oddX, oddY);
        db = 4*a*a*b*b - b*b*(4*x*x - 8*x + 4) -
                a*a*(4*y*y + 4*y + 1);
        }
}

/*************************************************************************************/

void StartIconBatchDraw(IconDisplay which)
{
    if (which & kSampleIcon)
        CanvasStartBatchDraw(sampleIconPanel, SAMPLE_CANVAS);
    if (which & kEditingIcon)
        CanvasStartBatchDraw(editPanel, EDIT_ICON_CANVAS);
}

/*************************************************************************************/

void EndIconBatchDraw(IconDisplay which)
{
    if (which & kEditingIcon)
        CanvasEndBatchDraw(editPanel, EDIT_ICON_CANVAS);
    if (which & kSampleIcon)
        CanvasEndBatchDraw(sampleIconPanel, SAMPLE_CANVAS);
}

/*************************************************************************************/

int Sel_IsActive(Selection *sel)
{
    return sel->active;
}

/*************************************************************************************/

    /*  Clears a selection and (if icon != NULL) adapts its allowable size to the icon */
void Sel_ClearSelection(Selection *sel, Icon *icon)
{
    int x, y;

    PointSet(&sel->position, 0, 0);
    sel->active = FALSE;
    
    if (icon)
        {
        sel->width = icon->descr.width;
        sel->height = icon->descr.height;
        }
    else
        {
        sel->width = kMaxIconDimension;
        sel->height = kMaxIconDimension;
        }

    for (y = 0; y < sel->height; y++)
        for (x = 0; x < sel->width; x++)
            sel->selApp[x][y]  = kSelNone;
}

/*************************************************************************************/

void Sel_Invalidate(Selection *sel)
{
    int x, y;

    for (y = 0; y < sel->height; y++)
        for (x = 0; x < sel->width; x++)
            if (sel->selApp[x][y] != kSelNone)
                sel->selApp[x][y] |= kSelInvalled;
}
    
/*************************************************************************************/

void Sel_SelectPixel(Selection *sel, Icon *icon, Point p)
{
    sel->selApp[p.x][p.y] = kSelUnknown;
}

/*************************************************************************************/

void Sel_SelectAll(Selection *sel, Icon *icon)
{
    int x, y;

    for (y = 0; y < sel->height; y++)
        for (x = 0; x < sel->width; x++)
            sel->selApp[x][y] = kSelUnknown;
            
    Sel_Compute(&gEditSelection, icon, FALSE);
    Sel_LiftUp(&gEditSelection, icon);          
}

/*************************************************************************************/

int Sel_IsIconPixelSelected(Selection *sel, Point p)
{
    int selApp;
    
    p.x -= sel->position.x; /* convert to selection coordinates */
    p.y -= sel->position.y;
    
    if (p.x < 0 || p.y < 0 || p.x >= sel->width || p.y >= sel->height)
        return FALSE;
    else
        {
        selApp = sel->selApp[p.x][p.y];
        return selApp != kSelNone && selApp != kSelUnknown;
        }
}

/*************************************************************************************/

void Sel_SetPosition(Selection *sel, Point p)
{
    sel->position = p;
    Sel_Invalidate(sel);
}

/*************************************************************************************/

void Sel_GetPosition(Selection *sel, Point *p)
{
    *p = sel->position;
}

/*************************************************************************************/

    /*  causes the data (colors) in the selection to be copied to the icon */
void Sel_Apply(Selection *sel, Icon *icon)
{
    int x, y;
    int selX, selY;
    int width = icon->descr.width;
    int height = icon->descr.height;
    
    Sel_RestoreBGPixelColors(sel, icon);
    
    for (y = 0; y < height; y++)        /* apply selection to current position */
        for (x = 0; x < width; x++)
            {
            selX = x - sel->position.x;
            selY = y - sel->position.y;
            
            if (selX >= 0 && selX < sel->width && selY >= 0 && selY < sel->height)
                if (sel->selApp[selX][selY] != kSelNone)
                    Icon_SetPixel(icon, MakePoint(x, y), sel->colorIndex[selX][selY], kNoDisplayIcon);
            }
}

/*************************************************************************************/

    /*  Also sets the sel->active flag to reflect whether there are any selected pixels */
int Sel_CalcOutlineAppearance(Selection *sel)
{
    int             x, y;
    SelAppearance   selApp;
    int             widthMinusOne  = sel->width - 1;
    int             heightMinusOne = sel->height - 1;
    int             numSelected = 0;
    
    Sel_CopyToTemp(sel);
    
    for (y = 0; y < sel->height; y++)
        for (x = 0; x < sel->width; x++)
            if (sel->selApp[x][y] & kSelUnknown)
            {
            selApp = kSelAll;
            if (x > 0 && sel->selAppTemp[x - 1][y] != kSelNone)
                selApp &= ~kSelLeft;
            if (x < widthMinusOne && sel->selAppTemp[x + 1][y] != kSelNone)
                selApp &= ~kSelRight;
            if (y > 0 && sel->selAppTemp[x][y - 1] != kSelNone)
                selApp &= ~kSelTop;
            if (y < heightMinusOne && sel->selAppTemp[x][y + 1] != kSelNone)
                selApp &= ~kSelBottom;
                
            sel->selApp[x][y] = selApp;
            if (selApp != kSelNone)
                numSelected++;
            }
        
    if (numSelected > 0)
        sel->active = TRUE;
    else
        sel->active = FALSE;            
        
    return numSelected;
}

/*************************************************************************************/

static void Sel_DeselectTransparentPixels(Selection *sel, Icon *icon)
{
    int x, y;

    Assert(icon->descr.width == sel->width);
    Assert(icon->descr.height == sel->height);

    for (y = 0; y < sel->height; y++)
        for (x = 0; x < sel->width; x++)
            if (icon->pixels[x][y].colorIndex == kTransparentColorIndex)
                sel->selApp[x][y] = kSelNone;
}

/*************************************************************************************/

static void Sel_SavePixelColors(Selection *sel, Icon *icon)
{
    int x, y;

    for (y = 0; y < sel->height; y++)
        for (x = 0; x < sel->width; x++)
            sel->colorIndex[x][y] = icon->pixels[x][y].colorIndex;
}

/*************************************************************************************/

static void Sel_SaveBGPixelColors(Selection *sel, Icon *icon)
{
    int x, y;

    for (y = 0; y < sel->height; y++)
        for (x = 0; x < sel->width; x++)
            sel->bgColorIndex[x][y] = icon->pixels[x][y].colorIndex;
}

/*************************************************************************************/

static void Sel_RestoreBGPixelColors(Selection *sel, Icon *icon)
{
    int x, y;

    for (y = 0; y < icon->descr.height; y++)        /* restore original pixels */  
        for (x = 0; x < icon->descr.width; x++)
            Icon_SetPixel(icon, MakePoint(x, y), sel->bgColorIndex[x][y], kNoDisplayIcon);  
}

/*************************************************************************************/

void Sel_Compute(Selection *sel, Icon *icon, int selectTransparentPixels)
{
    Sel_CalcInterior(sel);
    if (!selectTransparentPixels)
        Sel_DeselectTransparentPixels(sel, icon);
    Sel_CalcOutlineAppearance(sel);
    Sel_SavePixelColors(sel, icon);
    Sel_SaveBGPixelColors(sel, icon);
}

/*************************************************************************************/

    /*  Finds and unselected point on the edge of the icon.
        returns FALSE if no such point exists 
    */
static int Sel_FindUnselectedEdgePoint(Selection *sel, Point *start)
{
    int x, y;
    int width = sel->width;
    int height = sel->height;
    
    for (x = 0; x < width; x++)
        {
        if (sel->selApp[x][0] == kSelNone)
            {
            PointSet(start, x, 0);
            return TRUE;
            }

        if (sel->selApp[x][height - 1] == kSelNone)
            {
            PointSet(start, x, height - 1);
            return TRUE;
            }
        }
        
    for (y = 0; y < height; y++)
        {
        if (sel->selApp[0][y] == kSelNone)
            {
            PointSet(start, 0, y);
            return TRUE;
            }

        if (sel->selApp[width - 1][y] == kSelNone)
            {
            PointSet(start, width - 1, y);
            return TRUE;
            }
        }       

    return FALSE;
}

/*************************************************************************************/

static void Sel_CalcInterior(Selection *sel)
{
    int x, y;
    int width = sel->width;
    int height = sel->height;
    int startX, startY;
    Point   p;
    
    if (Sel_FindUnselectedEdgePoint(sel, &p))
        {
        do 
            {
            GenericFloodFill(sel, p, MakeRect(0,0,height,width), Sel_GetPixelCallback, 
                            Sel_SetPixelCallback, kSelNonSelectFill);
            }
            while (Sel_FindUnselectedEdgePoint(sel, &p));
            
        for (y = 0; y < sel->height; y++)
            for (x = 0; x < sel->width; x++)
                if (sel->selApp[x][y] == kSelNonSelectFill)
                    sel->selApp[x][y] = kSelNone;
                else
                    sel->selApp[x][y] = kSelUnknown;
        }
    else
        for (y = 0; y < sel->height; y++)   /* everything is selected */
            for (x = 0; x < sel->width; x++)
                sel->selApp[x][y] = kSelUnknown;
}

/*************************************************************************************/

static int Sel_GetPixelCallback(void *callbackData, Point p)
{
    Selection   *sel = (Selection *)callbackData;

    return sel->selApp[p.x][p.y];
}

/*************************************************************************************/

void Sel_SetPixelCallback(void *callbackData, Point p, int value)
{
    Selection   *sel = (Selection *)callbackData;
    
    sel->selApp[p.x][p.y] = value;
}


/*************************************************************************************/

static void Sel_CopyToTemp(Selection *sel)
{
    memmove(sel->selAppTemp, sel->selApp, sizeof(sel->selApp));
}

/*************************************************************************************/

static int Clipboard_GetCanvas(int *clipboardPanel, int *clipboardCanvas)
{
    int         error = 0;
    static int  panel = 0;
    static int  canvas = 0;
    
    if (panel <= 0)
        errChk( panel = NewPanel (0, "Clipboard panel", 40, 40, 100, 100));

    if (canvas <= 0)
        errChk( canvas = NewCtrl(panel, CTRL_CANVAS, 0, 50, 50));
    *clipboardPanel = panel;
    *clipboardCanvas = canvas;
        
Error:
    return error;
}

/*************************************************************************************/

static int Sel_VisibleAtPosition(Selection *sel, Icon *icon, Point position)
{
    Rect    iconRect;   
    int     x, y;
    
    iconRect = MakeRect(0,0, icon->descr.height, icon->descr.width);

    for (y = 0; y < sel->height; y++)
        for (x = 0; x < sel->width; x++)
            {
            if (sel->selApp[x][y] != kSelNone)
                if (RectContainsPoint(iconRect, MakePoint(x + position.x, y + position.y)))
                    return TRUE;    /* found visible selected pixel */
            }
            
    return FALSE;
}

/*************************************************************************************/

    /*  Returns TRUE is success, FALSE if the selection could not be moved because it would leave the icon */
int Sel_ShiftSelection(Selection *sel, Icon *icon, int dx, int dy)
{
    Point   pos;
    Point   newPos;
    int     shifted = TRUE;
    
    if (Sel_IsActive(sel))
        {
        Sel_GetPosition(sel, &pos);
        PointSet(&newPos, pos.x + dx, pos.y + dy);      
        
        if (Sel_VisibleAtPosition(sel, icon, newPos) || !Sel_VisibleAtPosition(sel, icon, pos))  /* part of the selection must remain on the icon (so the user can grab it) */
            Sel_SetPosition(sel, newPos);
        else
            shifted = FALSE;
        }

    return shifted;
}

/*************************************************************************************/

    /*  returns the bounds rectangle of the current selection in selection coordinates (not icon coordinates).
    */
Rect Sel_Bounds(Selection *sel)
{
    Rect    r = MakeRect(0,0,0,0);
    int     x, y;
    int     first = TRUE;

    for (y = 0; y < sel->height; y++)
        for (x = 0; x < sel->width; x++)
            {
            if (sel->selApp[x][y] != kSelNone)
                {
                if (first)
                    {
                    first = FALSE;
                    RectSet(&r, y, x, 1, 1);
                    }
                else
                    RectUnion(r, MakeRect(y, x, 1, 1), &r);
                }
            }
            
    return r;
}

/*************************************************************************************/

void Sel_CopyToClipboard(Selection *sel)
{
    int     panel, canvas;
    int     x, y;
    Rect    r;
    int     bitmap;
    
    Clipboard_GetCanvas(&panel, &canvas);
    
    r = Sel_Bounds(sel);
    SetCtrlAttribute(panel, canvas, ATTR_PICT_BGCOLOR, ColorSelect_GetIndexColor(&gColorSelector, kTransparentColorIndex));
    CanvasClear(panel, canvas, VAL_ENTIRE_OBJECT);
    SetCtrlAttribute(panel, canvas, ATTR_WIDTH, r.width);
    SetCtrlAttribute(panel, canvas, ATTR_HEIGHT, r.height);
    
    CanvasStartBatchDraw(panel, canvas);
    for (y = r.top; y < r.top + r.height; y++)
        for (x = r.left; x < r.left + r.width; x++)
            if (sel->selApp[x][y] != kSelNone)
                {
                SetCtrlAttribute(panel, canvas, ATTR_PEN_COLOR, ColorSelect_GetIndexColor(&gColorSelector, sel->colorIndex[x][y]));
                CanvasDrawPoint(panel, canvas, MakePoint(x - r.left, y - r.top));
                }
            
    CanvasEndBatchDraw(panel, canvas);
    
    if (GetCtrlBitmap (panel, canvas, 0, &bitmap) >= 0)
        {
        ClipboardPutBitmap(bitmap);
        DiscardBitmap(bitmap);
        }
}

/*************************************************************************************/

static void Sel_PasteCanvasContents(Selection *sel, Icon *icon, int panel, int canvas)
{
    int         height;
    int         width;
    static int  pixelColors[kMaxIconDimension * kMaxIconDimension]; 
    int         *colorTableWithTransparent = NULL;
    int         color;
    int         lastColor = 0xFFFFFFFF; /* invalid color */
    int         colorIndex;
    int         x, y;
    
    GetCtrlAttribute(panel, canvas, ATTR_HEIGHT, &height);
    GetCtrlAttribute(panel, canvas, ATTR_WIDTH, &width);

    Sel_SaveBGPixelColors(sel, icon);
    
    CanvasGetPixels(panel, canvas, MakeRect(0, 0, height, width), pixelColors);

    colorTableWithTransparent = malloc((icon->descr.numColors + 1) * sizeof(int));
    
    if (colorTableWithTransparent)
        {   
        memmove(colorTableWithTransparent + 1, icon->colorTable, icon->descr.numColors *sizeof(int));
        colorTableWithTransparent[0] = ColorSelect_GetIndexColor(&gColorSelector, kTransparentColorIndex);
        
        for (y = 0; y < height; y++)
            for (x = 0; x < width; x++)
                {
                color = pixelColors[y * width + x];
            
                if (color != lastColor)  
                    {
                    if (color == VAL_TRANSPARENT)
                        colorIndex = kTransparentColorIndex;    /* currently, Windows doesn't have a concept of a transparent bitmap, so this won't happen :( */
                    else
                        {   
                        colorIndex = FindClosestColorInTable(color, colorTableWithTransparent, icon->descr.numColors + 1);
                        if (colorIndex == 0)
                            colorIndex = kTransparentColorIndex;
                        else
                            colorIndex--;
                        }
                    lastColor = color;
                    }
                    
                if (colorIndex != kTransparentColorIndex)
                    {
                    sel->selApp[x][y] = kSelUnknown;
                    Icon_SetPixel(icon, MakePoint(x, y), colorIndex, kNoDisplayIcon);
                    }
                }
        }
            
    Sel_SavePixelColors(sel, icon);
    Sel_DeselectTransparentPixels(sel, icon);
    Sel_CalcOutlineAppearance(sel);
    free(colorTableWithTransparent);
}

/*************************************************************************************/

void Sel_PasteFromClipboard(Selection *sel, Icon *icon)
{
    int     panel, canvas;
    int     x, y;
    Rect    r;
    int     bitmap = 0;
    int     height;
    int     width;
    int     savedWaitCursor;

    savedWaitCursor = GetWaitCursorState();
    SetWaitCursor(TRUE);

    Sel_ClearSelection(sel, icon);  
    SetCurrentTool(kSelectTool);
    Clipboard_GetCanvas(&panel, &canvas);
    SetCtrlAttribute(panel, canvas, ATTR_PICT_BGCOLOR, ColorSelect_GetIndexColor(&gColorSelector, kTransparentColorIndex));
    ClipboardGetBitmap(&bitmap, NULL);
    
    if (bitmap) 
        {
        GetBitmapData (bitmap, NULL, NULL, &width, &height, NULL, NULL, NULL);
        width = Min(width, icon->descr.width);
        height = Min(height, icon->descr.height);
        
        SetCtrlAttribute(panel, canvas, ATTR_HEIGHT, height);
        SetCtrlAttribute(panel, canvas, ATTR_WIDTH, width);
        CanvasClear(panel, canvas, VAL_ENTIRE_OBJECT);
        CanvasDrawBitmap (panel, canvas, bitmap, VAL_ENTIRE_OBJECT, MakeRect(0,0, VAL_KEEP_SAME_SIZE, VAL_KEEP_SAME_SIZE));
        DiscardBitmap(bitmap);
        Sel_PasteCanvasContents(sel, icon, panel, canvas);
        }
        
    SetWaitCursor(savedWaitCursor);
}

/*************************************************************************************/

    /*  Call this function to specify that the currently selected pixels removed from
        the saved icon background pixels.  Conceptually, the pixels
        are lifted off the editing surface into the selection.
    */
void Sel_LiftUp(Selection *sel, Icon *icon)
{
    int x, y;
    int iconX, iconY;
    int width = sel->width;
    int height = sel->height;
    
    for (y = 0; y < height; y++)    
        for (x = 0; x < width; x++)
            {
            iconX = x + sel->position.x;
            iconY = y + sel->position.y;
            
            if (iconX >= 0 && iconX < icon->descr.width && iconY >= 0 && iconY < icon->descr.height)
                if (sel->selApp[x][y] != kSelNone)
                    sel->bgColorIndex[iconX][iconY] = kTransparentColorIndex;
            }
}

/*************************************************************************************/

void Sel_Delete(Selection *sel, Icon *icon)
{
    Sel_RestoreBGPixelColors(sel, icon);
    Sel_ClearSelection(sel, icon);
    TextTool_ClearText(icon);   /* if the text tool had text active, its gone now */
}

/*************************************************************************************/

void Sel_CutToClipboard(Selection *sel, Icon *icon)
{
    Sel_CopyToClipboard(sel);
    Sel_Delete(sel, icon);  
}

/*************************************************************************************/

void TextTool_HandleToolChange(Selection *sel, Icon *icon, int newTool,  int oldTool)
{
    if (newTool == kTextTool)
        {
        Icon_SaveState(icon, kTextUndoSavedState);
        TextTool_ClearText(icon);
        }
    else
    if (oldTool == kTextTool)
        TextTool_CommitText(sel, icon);

    if (icon)
        Sel_ClearSelection(sel, icon);
}

/*************************************************************************************/

void TextTool_CommitText(Selection *sel, Icon *icon)
{
    int textToCommit = Sel_IsActive(sel) && gTextTool.numChars > 0;
    
    if (textToCommit)
        {
        Icon_SaveState(icon, kTempState);   /* save current state */
        Icon_RestoreState(icon, kTextUndoSavedState, TRUE);  /* restore text undo state so we can make it the current official undo state */
        IconFile_SaveUndoInfo(ActiveIconFile(), kTextDrawingAction);  /* make current state the official undo state */
        Icon_RestoreState(icon, kTempState, FALSE);     /* restore the real current state */
        }

    TextTool_ClearText(icon);
    Sel_ClearSelection(sel, icon);
    
    if (textToCommit)
        Icon_SaveState(icon, kTextUndoSavedState);
}

/*************************************************************************************/

void TextTool_ClearText(Icon *icon)
{
    gTextTool.numChars = 0;
    gTextTool.text[0] = '\0';
    TextTool_SetPosition(MakePoint(0,icon->descr.height));
}

/*************************************************************************************/

void TextTool_SetPosition(Point p)
{
    gTextTool.position = p;
}

/*************************************************************************************/

Point TextTool_GetPosition(void)
{
    return gTextTool.position;
}

/*************************************************************************************/

void TextTool_GetMetaFont(char *metaFont)
{
    strcpy(metaFont, gTextTool.metaFont);
}

/*************************************************************************************/

    /* icon may be NULL */
void TextTool_SetMetaFont(Icon *icon, char *metaFont)
{
    strcpy(gTextTool.metaFont, metaFont);
    if ((gTextTool.numChars > 0) && (GetCurrentTool() == kTextTool))
        Sel_SetText(&gEditSelection, icon, gTextTool.text, TextTool_GetPosition());
}

/*************************************************************************************/

void TextColor_HandleColorChange(void)
{
    Icon        *icon = SelectedIcon();

    if (icon && (gTextTool.numChars > 0) && (GetCurrentTool() == kTextTool))
        Sel_SetText(&gEditSelection, icon, gTextTool.text, TextTool_GetPosition());
}

/*************************************************************************************/

    /*  returns whether a change was made */
int TextTool_HandleKey(Selection *sel, Icon *icon, int keyCode)
{
    int asciiKey = keyCode & VAL_ASCII_KEY_MASK;
    int width, height;
    int changed = FALSE;

    if (keyCode == VAL_BACKSPACE_VKEY)
        {
        if (gTextTool.numChars > 0)
            gTextTool.numChars--;
        }
    else
    if (asciiKey && gTextTool.numChars < kMaxIconDimension) 
        {
        gTextTool.text[gTextTool.numChars] = asciiKey;
        gTextTool.numChars++;
        }
        
    gTextTool.text[gTextTool.numChars] = '\0';
    GetTextDisplaySize (gTextTool.text, gTextTool.metaFont, &height, &width);
    if (width <= icon->descr.width)
        {
        Sel_SetText(sel, icon, gTextTool.text, TextTool_GetPosition());
        changed = TRUE;
        }
    else
        {
        Beep();
        TextTool_HandleKey(sel, icon, VAL_BACKSPACE_VKEY);
        }
    
    return changed;
}

/*************************************************************************************/

static void Sel_SetText(Selection *sel, Icon *icon, char *text, Point position)
{
    Point       p;
    int         height, width;
    int         panel, canvas;
    int         selectionWasActive;
    Rect        selBounds;
    Point       selPosition;
    
    selectionWasActive = Sel_IsActive(sel);
    
        /* Can't type text in the eraser color, you can't select erased pixels */
    if (ColorSelect_GetIndex(&gColorSelector) == kTransparentColorIndex)
        {
        Beep();
        ColorSelect_SetIndex(&gColorSelector, 0);   /* so choose the first color in the palette */
        }
    
    if (selectionWasActive)
        Sel_RestoreBGPixelColors(sel, icon);
        
    Clipboard_GetCanvas(&panel, &canvas);
    GetTextDisplaySize (text, gTextTool.metaFont, &height, &width);
    
    height = Min(height, kMaxIconDimension);
    width = Min(width, kMaxIconDimension);
    SetCtrlAttribute(panel, canvas, ATTR_HEIGHT, height);
    SetCtrlAttribute(panel, canvas, ATTR_WIDTH, width);
    SetCtrlAttribute(panel, canvas, ATTR_PICT_BGCOLOR, ColorSelect_GetIndexColor(&gColorSelector, kTransparentColorIndex));
    
    SetCtrlAttribute(panel, canvas, ATTR_PEN_COLOR, ColorSelect_GetIndexColor(&gColorSelector, ColorSelect_GetIndex(&gColorSelector)));
    SetCtrlAttribute(panel, canvas, ATTR_PEN_FILL_COLOR, VAL_TRANSPARENT);
    CanvasDrawTextAtPoint (panel, canvas, text,
                           gTextTool.metaFont, MakePoint(0,0), VAL_UPPER_LEFT);
    Sel_ClearSelection(sel, icon);
    Sel_PasteCanvasContents(sel, icon, panel, canvas);
    Sel_GetPosition(sel, &selPosition);
    selBounds = Sel_Bounds(sel);
    Sel_SetPosition(sel, MakePoint(position.x, position.y - selBounds.height - selBounds.top));
    Sel_Apply(sel, icon);
    Display_Update(icon, kEditingIcon);   
}
