#ifndef ICONUTIL_HEADER
#define ICONUTIL_HEADER

#include "iconedit.h"

typedef enum
    {
    kNoDisplayIcon      = 0,
    kEditingIcon        = (1 << 0),
    kSampleIcon         = (1 << 1),
    kSampleAndEditIcons = (kEditingIcon | kSampleIcon)
    } IconDisplay;


typedef enum
    {
    kSelNone    = 0,
    kSelTop     = 1 << 1,
    kSelLeft    = 1 << 2,
    kSelBottom  = 1 << 3,
    kSelRight   = 1 << 4,
    kSelMiddle  = 1 << 5,
    kSelAll     = (kSelTop | kSelLeft | kSelBottom | kSelRight | kSelMiddle),
    kSelUnknown = 1 << 6,       /* selected, but appearance is no yet calculated */
    kSelNonSelectFill  = 1 << 7,
    kSelInvalled = 1 << 8,
    } SelAppearance;
    
typedef struct 
    {
    int             active;
    Point           position;   /* note: selections are alway created with a position of 0,0.  This is changes when the selection is moved */
    int             width;
    int             height;
    int             colorIndex[kMaxIconDimension][kMaxIconDimension];  /* selection contents */
    int             bgColorIndex[kMaxIconDimension][kMaxIconDimension];  /* pre-selection state (in icon coordinates) */
    SelAppearance   selApp[kMaxIconDimension][kMaxIconDimension];
    SelAppearance   selAppTemp[kMaxIconDimension][kMaxIconDimension];
    }   Selection;
    
    
typedef enum
    {
    kGridInvalled = 0,
    kGridHidden,
    kGridShown,
    } GridDisplay;
    
typedef struct
    {
    GridDisplay     displayedGrid;          /* last displayed grid state */
    SelAppearance   displayedSelection;     /* last displayed selection state */
    int             displayedIndex;         /* color last displayed in editing area */
    int             displayedSampleIndex;   /* color last displayed on sample icon */
    } PixelDisplayState;
    
        
typedef void (*GenericSetPixelCallback)(void *callbackData, Point p, int value);    
typedef int  (*GenericGetPixelCallback)(void *callbackData, Point p);   

typedef struct  /* this struct is handy for Set/GetPixel callback data. Some generic draw functions may require it, other may allow any type of callback data */
    {
    void    *data;
    int     setPixelValue;
    int     setPixelFlags;      /* may be optional */
    int     getPixelFlags;      /* may be optional */
    } GenericSetGetPixelCallbackData;   
    
    
extern Selection gEditSelection;

int     Grid_Visible(void);
void    Grid_SetVisible(int visible);
void    Grid_Invalidate(void);

void    Icon_SetPixel(Icon *icon, Point p, int colorIndex, IconDisplay destination);
void    Icon_SaveState(Icon *icon, SavedState state);
void    Icon_RestoreState(Icon *icon, SavedState whichState, int saveCurrentState);
void    Icon_InvalDisplay(Icon *icon, IconDisplay display, int colorIndexToInval);
void    Display_Update(Icon *icon, IconDisplay display);
void    Display_AnimateSelection(Icon *icon, Selection *sel);

void    Icon_DrawLine(Icon *icon, Point start, Point end, int colorIndex, IconDisplay destination);
void    Icon_DrawRect(Icon *icon, Point start, Point end, int colorIndex, IconDisplay destination, int fill);
void    Icon_DrawOval(Icon *icon, Point start, Point end, int colorIndex, IconDisplay destination, int fill);
void    Icon_FloodFill(Icon *icon, Point p, int colorIndex, int colorIndexToReplace, IconDisplay destination);

void    GenericDrawLine (Point start, Point end, GenericSetPixelCallback setPixel, void *callbackData, int value);
void    GenericDrawRect(Point start, Point end, int fill, GenericSetPixelCallback setPixel, void *callbackData, int value);

void    Sel_ClearSelection(Selection *sel, Icon *icon);
void    Sel_Invalidate(Selection *sel);
void    Sel_Compute(Selection *sel, Icon *icon, int selectTransparentPixels);
void    Sel_SelectPixel(Selection *sel, Icon *icon, Point p);
void    Sel_SetPixelCallback(void *callbackData, Point p, int value);
int     Sel_IsIconPixelSelected(Selection *sel, Point p);
int     Sel_CalcOutlineAppearance(Selection *sel);
void    Sel_SetPosition(Selection *sel, Point p);
void    Sel_GetPosition(Selection *sel, Point *p);
void    Sel_Apply(Selection *sel, Icon *icon);
int     Sel_IsActive(Selection *sel);
void    Sel_PasteFromClipboard(Selection *sel, Icon *icon);
void    Sel_CopyToClipboard(Selection *sel);
void    Sel_LiftUp(Selection *sel, Icon *icon);
void    Sel_CutToClipboard(Selection *sel, Icon *icon);
void    Sel_Delete(Selection *sel, Icon *icon);
void    Sel_SelectAll(Selection *sel, Icon *icon);
Rect    Sel_Bounds(Selection *sel);
int     Sel_ShiftSelection(Selection *sel, Icon *icon, int dx, int dy);


int     TextTool_HandleKey(Selection *sel, Icon *icon, int key);
void    TextTool_SetMetaFont(Icon *icon, char *metaFont);
void    TextTool_GetMetaFont(char *metaFont);
void    TextTool_HandleToolChange(Selection *sel, Icon *icon, int newTool,  int oldTool);
void    TextTool_SetPosition(Point p);
Point   TextTool_GetPosition(void);
void    TextTool_ClearText(Icon *icon);
void    TextColor_HandleColorChange(void);
void    TextTool_CommitText(Selection *sel, Icon *icon);

void    StartIconBatchDraw(IconDisplay which);
void    EndIconBatchDraw(IconDisplay which);

#endif /* ICONUTIL_HEADER */
