#include "lockout.h"

/************************************************************************/
/*                                                                      */
/*  This program is similar to chart.prj in that it shows you how to    */
/*  perform a task continuously while waiting for user interface events */
/*  to occur on your front panels.                                      */
/*                                                                      */
/*  Normally, it is possible to access other Windows applications by    */
/*  pressing <Alt-Tab>, <Alt_Esc>, <Ctrl-Esc>, or by clicking on        */
/*  another application.  Also, moving or sizing any top-level window   */
/*  causes the idle events to cease.                                    */
/*                                                                      */
/*  This example also shows you how to lock out Windows so that the     */
/*  end-user cannot access other Window applications or stop idle       */
/*  events.                                                             */
/*                                                                      */
/************************************************************************/

#include <cvirte.h>    /* Needed if linking in external compiler; harmless otherwise */
#include <userint.h>
#include <utility.h>
#include <ansi_c.h>
#include "lockout.h"

#define TRUE  1
#define FALSE 0

int handle, plotdata = FALSE;
double datapoints[3];


int main (int argc, char *argv[])
{
    if (InitCVIRTE (0, argv, 0) == 0)    /* Needed if linking in external compiler; harmless otherwise */
        return -1;    /* out of memory */
    
    handle = LoadPanel (0, "lockout.uir", PANEL);
    SetCtrlAttribute (handle, PANEL_LOCKOUT_TEXT, ATTR_VISIBLE, FALSE);
    DisplayPanel (handle);
    RunUserInterface ();
    EnableTaskSwitching();

    return 0;
}

/************************************************************************/
/*                                                                      */
/*  ProcessLoop checks the plotdata flag.  The plotdata flag is set     */
/*  when the user clicks on the Plot button.  If the flag is set,       */
/*  ProcessLoop will generate and plot data on the strip chart.  The    */
/*  rate at which ProcessLoop is called is set by the SetIdleEventRate  */
/*  function.  Passing zero (0) to SetIdleEventRate will call Process-  */
/*  Loop as fast as possible.                                           */
/*                                                                      */
/************************************************************************/

int CVICALLBACK UpdateStripChart(int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event)
        {
        case EVENT_TIMER_TICK:
            if (plotdata == TRUE) 
                {
                datapoints[0] = 100.0 * rand() / RAND_MAX;
                datapoints[1] = 100.0 * rand() / RAND_MAX;
                datapoints[2] = 100.0 * rand() / RAND_MAX;

                PlotStripChart (handle, PANEL_CHART, datapoints, 3, 0, 0, VAL_DOUBLE);
                }
            break;
        }
        
    return 0;
}


int CVICALLBACK PlotData(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    if (event == EVENT_COMMIT)
        plotdata = TRUE;
        
    return 0;
}

int CVICALLBACK LockoutCallback(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    int locked;
    if (event == EVENT_COMMIT)
        {
        GetCtrlVal (handle, PANEL_LOCKOUT, &locked);
        if (locked)
            {
            DisableTaskSwitching();
            SetPanelAttribute (handle, ATTR_CAN_MINIMIZE, FALSE);
            SetPanelAttribute (handle, ATTR_CAN_MAXIMIZE, FALSE);
            SetPanelAttribute (handle, ATTR_CLOSE_ITEM_VISIBLE, FALSE);
            SetPanelAttribute (handle, ATTR_MOVABLE, FALSE);
            SetPanelAttribute (handle, ATTR_SIZABLE, FALSE);
            SetPanelAttribute (handle, ATTR_SYSTEM_MENU_VISIBLE, FALSE);
            SetCtrlAttribute (handle, PANEL_NORMAL_TEXT, ATTR_VISIBLE, FALSE);
            SetCtrlAttribute (handle, PANEL_LOCKOUT_TEXT, ATTR_VISIBLE, TRUE);
            SetPanelAttribute (handle, ATTR_WINDOW_ZOOM, VAL_MAXIMIZE);
            }
        else
            {
            SetPanelAttribute (handle, ATTR_WINDOW_ZOOM, VAL_NO_ZOOM);
            SetPanelAttribute (handle, ATTR_CAN_MINIMIZE, TRUE);
            SetPanelAttribute (handle, ATTR_CLOSE_ITEM_VISIBLE, TRUE);
            SetPanelAttribute (handle, ATTR_MOVABLE, TRUE);
            SetPanelAttribute (handle, ATTR_SYSTEM_MENU_VISIBLE, TRUE);
            SetCtrlAttribute (handle, PANEL_NORMAL_TEXT, ATTR_VISIBLE, TRUE);
            SetCtrlAttribute (handle, PANEL_LOCKOUT_TEXT, ATTR_VISIBLE, FALSE);
            EnableTaskSwitching();
            }
        }
    return 0;

}


int CVICALLBACK Shutdown(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    if (event == EVENT_COMMIT)
        QuitUserInterface(0);
        
    return 0;
}
