#include <cvirte.h>    /* Needed if linking in external compiler; harmless otherwise */
#include <userint.h>
#include <utility.h>
#include <ansi_c.h>
#include "scopedem.h"
#include "scope.h"

void InitializeGUI (void);

int handle;
double vert = 2.0, horz = 0.002;
int channel = 1;
double waveform[100], period, xzero;
char pathname[MAX_PATHNAME_LEN];

int main (int argc, char *argv[])
{
    if (InitCVIRTE (0, argv, 0) == 0)    /* Needed if linking in external compiler; harmless otherwise */
        return -1;    /* out of memory */
    
    scope_init (1);

    handle = LoadPanel (0, "scopedem.uir", PANEL);
    DisplayPanel (handle);
    InitializeGUI();
    RunUserInterface ();

    return 0;
}

int CVICALLBACK UpdateCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_TIMER_TICK:
            scope_config (channel, vert, 1, horz);
            scope_read_waveform (channel, waveform, &period, &xzero);
            DeleteGraphPlot (handle, PANEL_DISPLAY, -1, 0);

            PlotY (handle, PANEL_DISPLAY, waveform, 100, VAL_DOUBLE, VAL_FAT_LINE,
               VAL_EMPTY_SQUARE, VAL_SOLID, 1, VAL_YELLOW);

            break;
    }
    
    return 0;
}

int CVICALLBACK SetVertical(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    if (event == EVENT_VAL_CHANGED) {
        GetCtrlVal (handle, PANEL_VERTICAL, &vert);
        SetCtrlVal (handle, PANEL_VREADOUT, vert);
    }
    return (0);
}

int CVICALLBACK SetHorizontal(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    if (event == EVENT_VAL_CHANGED) {
        GetCtrlVal (handle, PANEL_HORIZONTAL, &horz);
        SetCtrlVal (handle, PANEL_HREADOUT, horz);
    }
    return(0);
}

int CVICALLBACK SetChannel(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    if (event == EVENT_COMMIT) {
        GetCtrlVal (handle, PANEL_CHANNEL, &channel);
    }
    return(0);

}


int CVICALLBACK QuitButton(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    if (event == EVENT_COMMIT) {
        QuitUserInterface (0);
        scope_close ();
        }
    return(0);
}

int CVICALLBACK SaveToDisk(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    char directory[MAX_PATHNAME_LEN];
    FILE *filehandle;
    int i;

    if (event == EVENT_COMMIT) {
        GetProjectDir (directory);
        if (FileSelectPopup (directory, "*.wfm", "WFM Files",
                             "Save Waveform to Disk", VAL_SAVE_BUTTON, 1,
                             1, 1, 0, pathname) > 0) {
        filehandle = fopen (pathname, "w+");
        fprintf (filehandle, "OSCILLOSCOPE DEMO DATA FILE \n\n");
        fprintf(filehandle, "Channel Acquired: CH%d\n", channel);
        fprintf(filehandle, "Date %s   Time: %s \n", DateStr(), TimeStr());
        fprintf (filehandle, "----------------------------------\n");
        for (i=0;i<100;i++)
            fprintf (filehandle, "%f\n", waveform[i]);
        fclose (filehandle);
        SetCtrlAttribute (handle, PANEL_VIEWDATA, ATTR_DIMMED, 0);
        }
    }
    return (0);
}


int CVICALLBACK ViewDataPoints(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    char filestring[100];

    if (event == EVENT_COMMIT) {
        sprintf (filestring, "notepad.exe %s", pathname);
        LaunchExecutable (filestring);
        }
    return (0);
}

void InitializeGUI ()
{
    SetCtrlVal (handle, PANEL_VREADOUT, 2.0);
    SetCtrlVal (handle, PANEL_HREADOUT, 0.002);
    SetCtrlVal (handle, PANEL_VERTICAL, 2.0);
    SetCtrlVal (handle, PANEL_HORIZONTAL, 0.002);

}
