#include <cvirte.h>    /* Needed if linking in external compiler; harmless otherwise */
#include "analysis.h"
#include "userint.h"
#include "sigproc.h"
#include "utility.h"

void run_analysis(void);
int panelhandle;

int main (int argc, char *argv[])
{
    if (InitCVIRTE (0, argv, 0) == 0)    /* Needed if linking in external compiler; harmless otherwise */
        return -1;    /* out of memory */
    
    panelhandle = LoadPanel (0, "sigproc.uir" , PANEL);
    DisplayPanel (panelhandle);
    RunUserInterface ();

    return 0;
}


int CVICALLBACK update(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
   if (event == EVENT_COMMIT) {
       switch (control) {
            case PANEL_GO :
                run_analysis();
                break;
            case PANEL_QUIT:
                QuitUserInterface (0);
                break;
        }
    }
    return 0;
}


void run_analysis(void)
{
    int source1,source2,i,n,window,filter;
    double amp1,amp2,cyc1,cyc2,mod;
    double delta_t,delta_f,frequency,ffreq,frequency_array[1025];
    double static wfm[1024],wfm1[1024],wfm2[1024],wfm3[1024];
    int static ploth1,ploth2,ploth3;

    GetCtrlVal (panelhandle, PANEL_SOURCE1, &source1);
    GetCtrlVal (panelhandle, PANEL_SOURCE2, &source2);
    GetCtrlVal (panelhandle, PANEL_AMP1, &amp1);
    GetCtrlVal (panelhandle, PANEL_AMP2, &amp2);
    GetCtrlVal (panelhandle, PANEL_FREQ1, &cyc1);
    GetCtrlVal (panelhandle, PANEL_FREQ2, &cyc2);
    GetCtrlVal (panelhandle, PANEL_SAMPLE_FREQ, &frequency);
    GetCtrlVal (panelhandle, PANEL_WINDOW, &window);
    GetCtrlVal (panelhandle, PANEL_FILTER_FREQ, &ffreq);
    GetCtrlVal (panelhandle, PANEL_FILTER, &filter);

    cyc1=cyc1/(frequency/1024);
    cyc2=cyc2/(frequency/1024);

    switch (source1)
    {
        case 0:
            Clear1D (wfm1, 1024);
            break;
        case 1:
            SinePattern (1024, amp1, 0.0, cyc1, wfm1);
            break;
        case 2:
            n=1024/cyc1;
            i=0;
            while (i<1024)
            {
                mod=(double)i/(double)n;
                if ((mod-(int)mod)>0.5)
                    wfm1[i]=amp1;
                else
                    wfm1[i]=-amp1;
                i++;
            }
            break;
        case 3:
            WhiteNoise (1024, amp1, Timer(), wfm1);
            break;
        case 4:
            GaussNoise (1024, amp1, Timer(), wfm1);
            break;


    }

    switch (source2)
    {
        case 0:
            Clear1D (wfm2, 1024);
            break;
        case 1:
            SinePattern (1024, amp2, 0.0, cyc2, wfm2);
            break;
        case 2:
            n=1024/cyc2;
            i=0;
            while (i<1024)
            {
                mod=(double)i/(double)n;
                if ((mod-(int)mod)>0.5)
                    wfm2[i]=amp2;
                else
                    wfm2[i]=-amp2;
                i++;
            }
            break;
        case 3:
            WhiteNoise (1024, amp2, Timer(), wfm2);
            break;
        case 4:
            GaussNoise (1024, amp2, Timer(), wfm2);
            break;
    }

    Add1D (wfm1, wfm2, 1024, wfm);

    if (ploth1>0)
    {
        DeleteGraphPlot (panelhandle, PANEL_GRAPH1, ploth1, 1);
        ploth1=0;
    }

    ploth1=PlotY (panelhandle, PANEL_GRAPH1, wfm, 1024, VAL_DOUBLE,
           VAL_THIN_LINE, VAL_EMPTY_SQUARE, VAL_SOLID, 1, VAL_YELLOW);

    switch (filter)
    {
        case 0:
            Copy1D (wfm, 1024, wfm3);
            break;
        case 1:
            Bw_LPF (wfm, 1024, frequency, ffreq,  5, wfm3);
            break;
        case 2:
            Bw_HPF (wfm, 1024, frequency, ffreq,  5, wfm3);
            break;
    }


    switch (window)
    {
        case 1:
            TriWin (wfm3, 1024);
            break;
        case 2:
            HanWin (wfm3, 1024);
            break;
        case 3:
            HamWin (wfm3, 1024);
            break;
        case 4:
            BkmanWin (wfm3, 1024);
            break;
        case 5:
            KsrWin (wfm3, 1024, 10);
            break;
    }

    if (ploth3>0)
    {
        DeleteGraphPlot (panelhandle, PANEL_GRAPH2, ploth3, 1);
        ploth3=0;
    }

    ploth3=PlotY (panelhandle, PANEL_GRAPH2, wfm3, 1024, VAL_DOUBLE,
           VAL_THIN_LINE, VAL_EMPTY_SQUARE, VAL_SOLID, 1, VAL_GREEN);

    Spectrum (wfm3, 1024);

    delta_t=1/frequency;
    delta_f=1/(1024*delta_t);
    frequency_array[0]=0.0;
    frequency_array[512]=1/(2*delta_t);
    for (i=0;i<512;i++)
    {
        frequency_array[i]=i*delta_f;
        frequency_array[1024-i]=-1*delta_f;
    }

    if (ploth2>0)
    {
        DeleteGraphPlot (panelhandle, PANEL_GRAPH3, ploth2, 1);
        ploth2=0;
    }

        ploth2=PlotXY (panelhandle, PANEL_GRAPH3, frequency_array, wfm3, 512,
                VAL_DOUBLE, VAL_DOUBLE, VAL_THIN_LINE, VAL_EMPTY_SQUARE,
                VAL_SOLID,1, VAL_RED);

}
