/*
 * This module contians the code to print, save, and copy panels
 */
 
// CAUTION : NO ERRROR HANDLING IN THIS VERSION !

#include <utility.h>
#include <userint.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>

#include "BmpFile.h"

// CAUTION : Since the size of BITMAPFILEHEADER structure
// is not a multiple of 4, some attention should be paid here
#pragma pack(2)
typedef struct tagBITMAPFILEHEADER { 
        short                   bfType;             //Specifies the file type. It must be BM. 
        unsigned int  bfSize;           //Specifies the size, in bytes, of the bitmap file. 
        short               bfReserved1;    //Reserved; must be zero. 
        short               bfReserved2;    //Reserved; must be zero. 
        unsigned int    bfOffBits;      //Specifies the offset, in bytes, from the BITMAPFILEHEADER structure to the bitmap bits. 
} BITMAPFILEHEADER; 
#pragma endpack()

typedef struct tagBITMAPINFOHEADER{ 
   int      biSize;                     //Specifies the number of bytes required by the structure.
   int      biWidth;                //Specifies the width of the bitmap, in pixels. 
   int      biHeight;               //Specifies the height of the bitmap, in pixels. If <0 see Win32     
   short  biPlanes;                 //Must be set to 1
   short    biBitCount;             //Specifies the number of bits per pixel. This value must be 1, 4, 8, 16, 24, or 32. 
   int      biCompression;      //Specifies the type of compression for a compressed bottom-up bitmap
   int      biSizeImage;        //Specifies the size, in bytes, of the image
   int      biXPelsPerMeter;    //Specifies the horizontal resolution, in pixels per meter, of the target device for the bitmap
   int      biYPelsPerMeter;    //Specifies the vertical resolution, in pixels per meter, of the target device for the bitmap. 
   int      biClrUsed;              //Specifies the number of color indices in the color table that are actually used by the bitmap
   int      biClrImportant;     //Specifies the number of color indices that are considered important for displaying the bitmap
} BITMAPINFOHEADER; 

            
  
typedef struct tagRRGGBB{ 
   unsigned char RR; 
   unsigned char GG; 
   unsigned char BB; 
} RRGGBB; 



#define BI_RGB  0L                  //No compression   

int SetBitmapToFile(char PathName[], int BmpId){

    int                     ImageHeight, ImageWidth, RowBytes, PixelDepth, BytesPerLine;
    int                     *ColorTable, NbBytesColorTable, NbColors;
    int                     NbBytesImage;
    unsigned char   *BitsImage, *TrueBitsImage;
    int                     Err;
    
    int i, j;     
    RRGGBB *Ptr;
    unsigned char Dummy;
    unsigned char *PtrSource, *PtrTarget, *PtrDummy;
    
    FILE                            *Stream;
    BITMAPFILEHEADER    BmpFileHeader;  
    BITMAPINFOHEADER    BmpInfoHeader; 

    
    Err = GetBitmapInfo (BmpId, &NbBytesColorTable, &NbBytesImage, NULL);
    BitsImage = (unsigned char*)malloc(NbBytesImage);   
    if(NbBytesColorTable){ // Pixel Depth <=8    
        ColorTable = (int*)malloc(NbBytesColorTable);
        Err = GetBitmapData (BmpId, &RowBytes, &PixelDepth, &ImageWidth, &ImageHeight, ColorTable, BitsImage, NULL);
    }else{ // Pixel Depth >8 
        Err = GetBitmapData (BmpId, &RowBytes, &PixelDepth, &ImageWidth, &ImageHeight, NULL, BitsImage, NULL);
    }   
    NbColors = pow(2,PixelDepth);
    // Z! Modulo 4
    BytesPerLine = 4*((RowBytes+3)/4);
    
  // Initialize the fields in the BITMAPINFO structure
  BmpInfoHeader.biSize              = sizeof(BITMAPINFOHEADER);
  BmpInfoHeader.biWidth             = ImageWidth;
  BmpInfoHeader.biHeight            = ImageHeight; // I can't use negative number, why ????
  BmpInfoHeader.biPlanes            = 1;
  BmpInfoHeader.biBitCount          = PixelDepth;
  BmpInfoHeader.biCompression   = BI_RGB;
  BmpInfoHeader.biSizeImage         = BytesPerLine*ImageHeight; 
  BmpInfoHeader.biXPelsPerMeter = 0;
  BmpInfoHeader.biYPelsPerMeter = 0;
  if (PixelDepth<24)
    BmpInfoHeader.biClrUsed = NbColors;
  else
    BmpInfoHeader.biClrUsed = 0;
    BmpInfoHeader.biClrImportant = 0; // All of the device colors are important.

  BmpFileHeader.bfType = 0x4d42;  // Signature 0x42="B" 0x4d="M"
  // Compute the size of the entire file
  BmpFileHeader.bfSize = sizeof(BITMAPFILEHEADER) + sizeof(BITMAPINFOHEADER) + NbBytesColorTable + BmpInfoHeader.biSizeImage;
  BmpFileHeader.bfReserved1 = 0;
  BmpFileHeader.bfReserved2 = 0;
  // Compute the offset to the array of color indices
  BmpFileHeader.bfOffBits = sizeof(BITMAPFILEHEADER) + sizeof(BITMAPINFOHEADER) + NbBytesColorTable;
    
    // Open the file
    Stream = fopen (PathName, "wb+");
    
    // Copy the BITMAPFILEHEADER
    Err = fwrite ((void*)&BmpFileHeader, sizeof(BITMAPFILEHEADER), 1, Stream); 
    // Copy the BITMAPINFOHEADER
    Err = fwrite ((void*)&BmpInfoHeader, sizeof(BITMAPINFOHEADER), 1, Stream);
    // Copy ColorTable into the file
  if (PixelDepth<24)
        Err = fwrite ((void*)ColorTable, sizeof(int), NbColors , Stream);
  
    // Create (in memory) the image to be copied on file
    // Working that way allow us tro save the array with fwrite function
    TrueBitsImage = (unsigned char*)calloc(BmpInfoHeader.biSizeImage,sizeof(unsigned char)); 
    
    
    // We copy each scan line already filled with 0 
    PtrSource = BitsImage;
    PtrTarget = TrueBitsImage;
    for (i=0; i<ImageHeight; i++){
        memcpy (PtrTarget, PtrSource, RowBytes);
        PtrTarget+=BytesPerLine;
        PtrSource+=RowBytes; 
    }
    
    //Since colors must be saved in BBGGRR format instead of RRGGBB ...
    if(PixelDepth>8){
        for (j=0; j<ImageHeight; j++){
            Ptr = (RRGGBB*)(TrueBitsImage+j*BytesPerLine);                
            for (i=0; i<=ImageWidth; i++){          
                Dummy = Ptr->RR;      
                Ptr->RR = Ptr->BB;
                Ptr->BB = Dummy; 
                Ptr++;
            }   
        }   
    }   
        
    // I should not do the next few line but I'm not able to use negative value
    // for biHeight property so I need to upside down the array by hand ...
    // This code should be remove as soon as I can understand what happen
    // with biHeight
    
    // PtrSource points to the beginning of the last scan line
    PtrSource = TrueBitsImage + ((ImageHeight-1)*BytesPerLine);
    // PtrTarget points to the beginning of the first scan line  
    PtrTarget = TrueBitsImage;
    PtrDummy = (unsigned char*)malloc(BytesPerLine);   
    for (i=0; i<ImageHeight/2; i++){
        memcpy (PtrDummy, PtrTarget, BytesPerLine);
        memcpy (PtrTarget, PtrSource, BytesPerLine);
        memcpy (PtrSource, PtrDummy, BytesPerLine);  
        PtrTarget+=BytesPerLine;
        PtrSource-=BytesPerLine;
    }
    free(PtrDummy);
    
    /* Copy the array of color indices into the BMP file. */
    Err = fwrite ((void*)TrueBitsImage, sizeof(unsigned char), BmpInfoHeader.biSizeImage, Stream);  
  // Close the BMP file
    Err = fclose (Stream);
  //Free memory. 
  free(TrueBitsImage);
  if (PixelDepth<24)      
        free(ColorTable);   
    free(BitsImage);
    return (UIENoError);
}   
