/*
 * This file contains the functions to maipulate linked lists of 
 * structures in the followin format:
 *
 *      typedef struct s_Data {
 *          ...                     // Any Data  
 *          struct s_Data *pPrev;   // Previous Pointer  
 *          ...                     // Any Data  
 *          struct s_Data *pNext;   // Next Pointer  
 *          ...                     // Any Data  
 *      } DATA;
 *
 * The list must be initialized using a sample structure template
 * using the FnListInit() function:
 *
 *      LISTDATA DataLinkedList;
 *      DATA DataSample;
 *  
 *      FnListInit(&DataLinkedList, &Data, 
 *              &DataSample.pNext, &DataSample.pPrev);
 * 
 */

/*
 * #includes
 */
#include "List.h"
#include "ansi_c.h"

/*
 * Declaration of non-user functions
 */
void   FnListChgHead(LISTDATA *pLD, void *pItem);
void   FnListChgTail(LISTDATA *pLD, void *pItem);
void   FnListCngNextPtr(LISTDATA *pLD, void *pItem, void *pNext); 
void **FnListGetNextPtr(LISTDATA *pLD, void *pItem);   
void   FnListCngPrevPtr(LISTDATA *pLD, void *pItem, void *pNext); 
void **FnListGetPrevPtr(LISTDATA *pLD, void *pItem);   

/*************************************************************************
 * User Functions
 ************************************************************************/
/*
 * Initilize the list using a sample structure template
 */
void FnListInit(LISTDATA *pLD, void *pSmplStart, 
        void *pSmplNext, void *pSmplPrev)
{
    pLD->iNextOffset = (int) pSmplNext - (int) pSmplStart;
    pLD->iPrevOffset = (int) pSmplPrev - (int) pSmplStart;
    FnListChgHead(pLD, NULL);
    FnListChgTail(pLD, NULL);
}

/*
 * Return the head of the list
 */
void *FnListHead(LISTDATA *pLD)
{
    return (pLD->pHead);
}

/* 
 * Return the tail of the list
 */
void *FnListTail(LISTDATA *pLD)
{
    return (pLD->pTail);
}

/*
 * Get the next pointer of pItem
 */
void *FnListNext(LISTDATA *pLD, void *pItem)
{
    return (*FnListGetNextPtr(pLD, pItem));
}

/*
 * Get the previous pointer of pItem
 */
void *FnListPrev(LISTDATA *pLD, void *pItem)
{
    return (*FnListGetPrevPtr(pLD, pItem));
}

/*
 * Add pItem to the end of the list
 */
void  FnListAddItem(LISTDATA *pLD, void *pItem)
{
    if (!FnListHead(pLD)) 
        FnListChgHead(pLD, pItem);
    else {
        FnListCngPrevPtr(pLD, pItem, FnListTail(pLD));
        FnListCngNextPtr(pLD, FnListTail(pLD), pItem);
    }
    
    FnListChgTail(pLD, pItem);  
}

/*
 * Remove pItem from list
 */
void *FnListRemItem(LISTDATA *pLD, void *pItem)
{
    void *pWalk = NULL;
    
    for (pWalk = FnListHead(pLD); pWalk; pWalk = FnListNext(pLD, pWalk)) 
        if (pWalk == pItem) {
            if (FnListHead(pLD) == FnListTail(pLD)) {
                FnListChgHead(pLD, NULL);
                FnListChgTail(pLD, NULL);   
            }
            else if (pWalk == FnListHead(pLD)) 
                FnListChgHead(pLD, FnListNext(pLD, pWalk));
            else if (pWalk == FnListTail(pLD)) 
                FnListChgTail(pLD, FnListPrev(pLD, pWalk));
            else {
                void *pPrev;
                void *pNext;
                
                pPrev = FnListPrev(pLD, pWalk);
                pNext = FnListNext(pLD, pWalk); 
                
                FnListCngPrevPtr(pLD, pNext, pPrev);
                FnListCngNextPtr(pLD, pPrev, pNext);
            }
            break;
        }
    
    return(pWalk);
}

/*************************************************************************
 * Non-user functions
 ************************************************************************/
/*
 * Changeg the head of the list to pItem
 */
void  FnListChgHead(LISTDATA *pLD, void *pItem) 
{
    pLD->pHead = pItem;
    
    if (pItem) {
        FnListCngPrevPtr(pLD, pItem, NULL);
//      FnListCngNextPtr(pLD, pItem, NULL);
    }
}

/*
 * Changeg the tail of the list to pItem
 */
void  FnListChgTail(LISTDATA *pLD, void *pItem)
{
    pLD->pTail = pItem; 
    
    if (pItem)
        FnListCngNextPtr(pLD, pItem, NULL);
}

/*
 * Changeg the value of the next pointer of pItem to pNext 
 */
void  FnListCngNextPtr(LISTDATA *pLD, void *pItem, void *pNext)
{
    void **pItemNext;
    
    pItemNext = FnListGetNextPtr(pLD, pItem);
    *pItemNext = pNext;
}

/*
 * Changeg the value of the previous pointer of pItem to pNext 
 */
void  FnListCngPrevPtr(LISTDATA *pLD, void *pItem, void *pNext)
{
    void **pItemPrev;
    
    pItemPrev = FnListGetPrevPtr(pLD, pItem);
    *pItemPrev = pNext;
}

/*
 * Get the pItem next pointer
 */
void **FnListGetNextPtr(LISTDATA *pLD, void *pItem)
{
    int iNext;
    
    if (!pItem)
        return(NULL);
    
    iNext = (int) pItem + pLD->iNextOffset;
    return ((void **) iNext);
}

/*
 * Get the pItem previous pointer
 */
void **FnListGetPrevPtr(LISTDATA *pLD, void *pItem)
{
    int iPrev;
    
    if (!pItem)
        return(NULL);
    
    iPrev = (int) pItem + pLD->iPrevOffset;
    return ((void **) iPrev);
}





