#include <userint.h>
#include "UIRView.h"

/* 
 * This module contins the main panel menu callback functions
 */

#include <ansi_c.h>
#include <utility.h>
#include "Main.h"

/* 
 * Update menus
 */
void FnUpdateMenus(void)
{
    int iIndex;
    HANDLE hPanel;
    HANDLE hMenu;
    HANDLE hParent;
    BOOLEAN bCanMax;
    BOOLEAN bCanMin;
    BOOLEAN bCloseCtrl;
    BOOLEAN bMoveable;
    BOOLEAN bParentCheck;
    BOOLEAN bSizable;
    BOOLEAN bTitle;
    BOOLEAN bColors;

    hMenu = FnGetMainMenuHandle();
    if (FnListHead(&UIRList)) 
        SetMenuBarAttribute (hMenu, MAIN_FILE_CLOSE, ATTR_DIMMED, 0);
    else 
        SetMenuBarAttribute (hMenu, MAIN_FILE_CLOSE, ATTR_DIMMED, 1);
    
    hPanel = FnGetCurrentWindowHandle();
    
    if (!hPanel) {
        SetMenuBarAttribute (hMenu, MAIN_PANEL, ATTR_DIMMED, 1);
        return;
    }
    SetMenuBarAttribute (hMenu, MAIN_PANEL, ATTR_DIMMED, 0);

    // can maximize
    GetPanelAttribute (hPanel, ATTR_CAN_MAXIMIZE, &bCanMax);
    SetMenuBarAttribute (hMenu, MAIN_PANEL_ATTR_CAN_MAX, 
            ATTR_CHECKED, bCanMax);
    // can minimize
    GetPanelAttribute (hPanel, ATTR_CAN_MINIMIZE, &bCanMin);
    SetMenuBarAttribute (hMenu, MAIN_PANEL_ATTR_CAN_MIN, 
            ATTR_CHECKED, bCanMin);
    // close control
    GetPanelAttribute (hPanel, ATTR_CLOSE_ITEM_VISIBLE, &bCloseCtrl);
    SetMenuBarAttribute (hMenu,MAIN_PANEL_ATTR_CLOSE_CTRL, 
            ATTR_CHECKED, bCloseCtrl);
    // movable
    GetPanelAttribute (hPanel, ATTR_MOVABLE, &bMoveable);
    SetMenuBarAttribute (hMenu, MAIN_PANEL_ATTR_MOVABLE, 
            ATTR_CHECKED, bMoveable);
    // parent
    GetPanelAttribute (hPanel, ATTR_PANEL_PARENT, &hParent);
    if (hParent > 0)
        bParentCheck = FALSE;
    else
        bParentCheck = TRUE;
    SetMenuBarAttribute (hMenu, MAIN_PANEL_ATTR_PARENT, 
            ATTR_CHECKED, bParentCheck);
    // sizable
    GetPanelAttribute (hPanel, ATTR_SIZABLE, &bSizable);
    SetMenuBarAttribute (hMenu, MAIN_PANEL_ATTR_SIZABLE, 
            ATTR_CHECKED, bSizable);
    // Title visible
    GetPanelAttribute (hPanel, ATTR_TITLEBAR_VISIBLE, &bTitle);
    SetMenuBarAttribute (hMenu, MAIN_PANEL_ATTR_TITLE_BAR, 
            ATTR_CHECKED, bTitle);
    // System colors
    GetPanelAttribute (hPanel, ATTR_CONFORM_TO_SYSTEM, &bColors);
    SetMenuBarAttribute (hMenu, MAIN_PANEL_ATTR_SYSTEM_COLORS,
            ATTR_CHECKED, bColors);
}

/*
 * Funciton to open a UIR file
 */
void CVICALLBACK FnFileOpen (int menuBar, int menuItem, void *callbackData,
        int panel)
{
    char szUIRName[MAX_PATHNAME_LEN]; 
    int iSelectStatus;
    HANDLE hMain;
    
    /* check what window is calling this menu callback */
    if (bMainPnlFocus == FALSE)
        return;

    hMain = FnGetMainPanelHandle(TRUE);
                                      
    iSelectStatus = FileSelectPopup ("", "*.uir", "*.uir", "File Open",
                               VAL_LOAD_BUTTON, 0, 1, 1, 0, szUIRName);
    
    switch (iSelectStatus) {
        UIR *pUIRItem;
        
        case VAL_EXISTING_FILE_SELECTED:
            /* check to see if selected file is already open */
            pUIRItem = FnGetUIRItemFromName(szUIRName); 
            
            /* uir file is not already open, open it */
            if (!pUIRItem) {
                HANDLE hTempPanel;
                PNL PNLSample;
                PNL *pPNLItem;
                char szPNLName[MAX_PATHNAME_LEN];   
                int iID;

                /* if first item in UIR list, remove none and enable ring */
                if (!FnListHead(&UIRList)) { 
                    DeleteListItem (hMain, MAIN_PNL_LB_UIR_SELECT, 0, 1);
                    SetCtrlAttribute (hMain, MAIN_PNL_LB_UIR_SELECT, 
                            ATTR_DIMMED, 0);
                }
                
                /* add new uir item to uir list */
                pUIRItem = (UIR *) malloc(sizeof(UIR));
                pUIRItem->pszName = (char *) malloc(sizeof(szUIRName));
                strcpy(pUIRItem->pszName, szUIRName);
                FnListAddItem(&UIRList, pUIRItem);
                
                /* add uir to uir list */
                InsertListItem (hMain, MAIN_PNL_LB_UIR_SELECT, 0,
                        pUIRItem->pszName, pUIRItem->pszName);
                SetCtrlIndex (hMain, MAIN_PNL_LB_UIR_SELECT, 0);
                
                /* create panel list */
                FnListInit(&pUIRItem->PNLList, &PNLSample,
                        &PNLSample.pNext, &PNLSample.pPrev);
                
                /* add all panels to panel list */
                iID = hTempPanel = 1;
                while (hTempPanel > 0) {
                    int bOLE;
                    int bOPE;
                    int iCtrlID;
                    int iStatus;
                    
                    /* load the panel from ID */
                    bOPE = SetBreakOnProtectionErrors (0);
                    bOLE = SetBreakOnLibraryErrors (0);
                    hTempPanel = LoadPanel (0, szUIRName, iID);
                    
                    SetPanelAttribute (hTempPanel, ATTR_CALLBACK_FUNCTION_POINTER, *FnWindow);
                    iCtrlID = 2;
                    do {
                        iStatus = SetCtrlAttribute (hTempPanel, iCtrlID++,
                                ATTR_CALLBACK_FUNCTION_POINTER, NULL);
                    } while (iStatus >= 0); 

                    SetBreakOnProtectionErrors (bOPE);
                    SetBreakOnLibraryErrors (bOLE);
                    
                    /* if ID is valid, panel handle will also be valid */
                    if (hTempPanel > 0) {
                        GetPanelAttribute (hTempPanel, ATTR_TITLE, szPNLName);
                        pPNLItem = (PNL *) malloc (sizeof(PNL));
                        pPNLItem->pszName = (char *) malloc(sizeof(szPNLName));
                        strcpy(pPNLItem->pszName, szPNLName);
                        pPNLItem->iID = iID;
                        pPNLItem->hPanel = 0;
                        pPNLItem->pUIR = (void *) pUIRItem;
                        FnListAddItem(&pUIRItem->PNLList, pPNLItem);
                        DiscardPanel (hTempPanel);
                        iID++;
                    }   
                }
                FnUIRListBoxUpdate();
                FnPNLListBoxInit(pUIRItem);
                FnUpdateMenus();
            }
            /* uir is already open, set ring control values */
            else {
                int iCurrentIndex;
                int iItemIndex;

                /* get current index and index from value form uir ring */
                GetCtrlIndex (hMain, MAIN_PNL_LB_UIR_SELECT, 
                        &iCurrentIndex);
                GetIndexFromValue (hMain, MAIN_PNL_LB_UIR_SELECT, 
                        &iItemIndex, pUIRItem->pszName);
                
                /* if current index and index from value are not equal.
                 * change uir ring to opened uir */ 
                if (iCurrentIndex != iItemIndex) {
                    SetCtrlIndex (hMain, MAIN_PNL_LB_UIR_SELECT, 
                            iItemIndex);
                            
                    FnUIRListBoxUpdate();
                    FnPNLListBoxInit(pUIRItem);
                    FnUpdateMenus();
                }
            }
            break;
        case VAL_NEW_FILE_SELECTED:
            /* will not happen */
            break;
        case VAL_NO_FILE_SELECTED:
            /* cancel selected */
            break;
        default:
            /* error in FileSelectPopup */
            MessagePopup ("Error", "Error in FileSelectPopup().");
            break;
    }
    return;
}

/* 
 * Funciton to close UIR files
 */
void CVICALLBACK FnFileClose (int menuBar, int menuItem, void *callbackData,
        int panel)
{
    int iUIRIndex;
    UIR *pUIRClose;
    UIR *pUIROpen;
    HANDLE hMain;
    
    // check what window is calling this menu callback  
    if (bMainPnlFocus == FALSE)
        return;

    hMain = FnGetMainPanelHandle(TRUE);
    
    // get UIR item to remove  
    pUIRClose = FnGetCurrentUIRItem();
    
    // get the next the next or previous uir  
    pUIROpen = FnListPrev(&UIRList, pUIRClose);
    if (!pUIROpen)
        pUIROpen = FnListNext(&UIRList, pUIRClose);
    
    // close all panels associated with UIR to close  
    FnCloseAllIUIRPanels(pUIRClose);
            
    // delete item from uir ring  
    GetIndexFromValue (hMain, MAIN_PNL_LB_UIR_SELECT, &iUIRIndex, pUIRClose->pszName);
    DeleteListItem (hMain, MAIN_PNL_LB_UIR_SELECT, iUIRIndex, 1);

    // delete item from uir list  
    FnFreeUIRItem(FnListRemItem(&UIRList, pUIRClose));
 
    // set the current UIR to view and its panels  
    if (!pUIROpen) {
        SetCtrlAttribute (hMain, MAIN_PNL_LB_UIR_SELECT, ATTR_DIMMED, 1);
        InsertListItem (hMain, MAIN_PNL_LB_UIR_SELECT, -1, "None", 0);
    }
    FnPNLListBoxInit(pUIROpen);
}

/* 
 * Funciton to exit program
 */
void CVICALLBACK FnFileExit (int menuBar, int menuItem, void *callbackData,
        int panel)
{
    UIR *pUIRItem;  
    UIR *pWINItem;
    
    /* check what window is calling this menu callback */
    if (bMainPnlFocus == FALSE)
        return;

    /* remove all uir data from ring and free */
    while (pUIRItem = FnListHead(&UIRList)) {
        FnCloseAllIUIRPanels(pUIRItem);
        FnFreeUIRItem(FnListRemItem(&UIRList, pUIRItem));
    }
    
    /* quit user interface */   
    QuitUserInterface (0);
}

/*
 * Place BMP on clippboard
 */
void CVICALLBACK FnPanelCopy (int menuBar, int menuItem, void *callbackData,
        int panel)
{
    HANDLE hBMP;
    HANDLE hPanel;
    HANDLE hMain;
    HANDLE hActive;
    BOOLEAN bFocus;
    
    /* check what window is calling this menu callback */
    if (bMainPnlFocus == FALSE)
        return;

    hPanel = FnGetCurrentWindowHandle();
    hMain = FnGetMainPanelHandle(FALSE);
    
    GetMenuBarAttribute (FnGetMainMenuHandle(),
                         MAIN_OPTIONS_FOCUS, ATTR_CHECKED,
                         &bFocus);
    
    /* set or remove focus */
    hActive = GetActivePanel ();
    if (bFocus) {
        if (hActive != hPanel)  
            SetActivePanel (hPanel);
    }
    else if (hActive != hMain)
        SetActivePanel (hMain);
    
    GetPanelDisplayBitmap (hPanel, VAL_VISIBLE_AREA, VAL_ENTIRE_OBJECT,
                           &hBMP);
    ClipboardPutBitmap (hBMP);
    
    SetActivePanel (hActive);
}

/*
 * Print BMP
 */
void CVICALLBACK FnPanelPrint (int menuBar, int menuItem, void *callbackData,
        int panel)
{
    HANDLE hPanel;
    HANDLE hMain;
    HANDLE hActive;
    BOOLEAN bFocus;
    
    /* check what window is calling this menu callback */
    if (bMainPnlFocus == FALSE)
        return;

    hPanel = FnGetCurrentWindowHandle(); 
    hMain = FnGetMainPanelHandle(FALSE);

    GetMenuBarAttribute (FnGetMainMenuHandle(),
                         MAIN_OPTIONS_FOCUS, ATTR_CHECKED,
                         &bFocus);
    
    /* set or remove focus */
    hActive = GetActivePanel ();
    if (bFocus) {
        if (hActive != hPanel)  
            SetActivePanel (hPanel);
    }
    else if (hActive != hMain)
        SetActivePanel (hMain);
    
    PrintPanel (hPanel, "", 0, VAL_VISIBLE_AREA, 1);
    
    SetWaitCursor (1);
    SetActivePanel (hActive);
    ProcessSystemEvents();
    
    SetWaitCursor (0);
}

/*
 * Save BMP to file
 */
void CVICALLBACK FnPanelSave (int menuBar, int menuItem, void *callbackData,
        int panel)
{
    char szFileName[MAX_PATHNAME_LEN];
    int iFileStatus;
    HANDLE hBMP;
    HANDLE hPanel;
    HANDLE hMain;
    HANDLE hActive;
    BOOLEAN bFocus;
    
    /* check what window is calling this menu callback */
    if (bMainPnlFocus == FALSE)
        return;

    hActive = GetActivePanel();

    iFileStatus = FileSelectPopup ("", "*.bmp", "*.bmp", "Save Bitmap",
                                   VAL_SAVE_BUTTON, 0, 1, 1, 1, szFileName);
    
    switch (iFileStatus) {
        case VAL_EXISTING_FILE_SELECTED:
        case VAL_NEW_FILE_SELECTED:
            hPanel = FnGetCurrentWindowHandle(); 
            hMain = FnGetMainPanelHandle(FALSE);
            
            GetMenuBarAttribute (FnGetMainMenuHandle(),
                         MAIN_OPTIONS_FOCUS, ATTR_CHECKED,
                         &bFocus);
            
            /* set or remove focus */
            if (bFocus) {
                if (hActive != hPanel)  
                    SetActivePanel (hPanel);
            }
            else if (hActive != hMain)
                SetActivePanel (hMain);
 
            GetPanelDisplayBitmap (hPanel, VAL_VISIBLE_AREA, VAL_ENTIRE_OBJECT,
                                   &hBMP);
            SetBitmapToFile(szFileName, hBMP);  

            SetActivePanel (hActive);
            break;
        case VAL_NO_FILE_SELECTED:
            /* cancel selected */
            break;
        default:
            /* error in FileSelectPopup */
            MessagePopup ("Error", "Error in FileSelectPopup().");
            break;
    }
}

/* 
 * Can maximize attribute
 */
void CVICALLBACK FnPanelAttrCanMaximize (int menuBar, int menuItem, void *callbackData,
        int panel)
{
    BOOLEAN bCanMax;
    
    /* check what window is calling this menu callback */
    if (bMainPnlFocus == FALSE)
        return;

    GetMenuBarAttribute (FnGetMainMenuHandle(),
                         MAIN_PANEL_ATTR_CAN_MAX, ATTR_CHECKED,
                         &bCanMax);
    
    SetPanelAttribute (FnGetCurrentWindowHandle(),
                        ATTR_CAN_MAXIMIZE, !bCanMax);
    
    SetMenuBarAttribute (FnGetMainMenuHandle(),
                         MAIN_PANEL_ATTR_CAN_MAX, ATTR_CHECKED,
                         !bCanMax);
}

/*
 * Minimize attribute
 */
void CVICALLBACK FnPanelAttrCanMinimize (int menuBar, int menuItem, void *callbackData,
        int panel)
{
    BOOLEAN bCanMin;
    
    /* check what window is calling this menu callback */
    if (bMainPnlFocus == FALSE)
        return;

    GetMenuBarAttribute (FnGetMainMenuHandle(),
                         MAIN_PANEL_ATTR_CAN_MIN, ATTR_CHECKED,
                         &bCanMin);
    
    SetPanelAttribute (FnGetCurrentWindowHandle(), 
                        ATTR_CAN_MINIMIZE, !bCanMin);
    
    SetMenuBarAttribute (FnGetMainMenuHandle(),
                         MAIN_PANEL_ATTR_CAN_MIN, ATTR_CHECKED,
                         !bCanMin);
}

/*
 * Close control attribute
 */
void CVICALLBACK FnPanelAttrCloseControl (int menuBar, int menuItem, void *callbackData,
        int panel)
{
    BOOLEAN bCloseCtrl;
    
    /* check what window is calling this menu callback */
    if (bMainPnlFocus == FALSE)
        return;

    GetMenuBarAttribute (FnGetMainMenuHandle(),
                         MAIN_PANEL_ATTR_CLOSE_CTRL, ATTR_CHECKED,
                         &bCloseCtrl);
    
    SetPanelAttribute (FnGetCurrentWindowHandle(), 
                        ATTR_CLOSE_ITEM_VISIBLE, !bCloseCtrl);
    
    SetMenuBarAttribute (FnGetMainMenuHandle(),
                         MAIN_PANEL_ATTR_CLOSE_CTRL, ATTR_CHECKED,
                         !bCloseCtrl);
}

/*
 * Movable attribute
 */
void CVICALLBACK FnPanelAttrMovable (int menuBar, int menuItem, void *callbackData,
        int panel)
{
    BOOLEAN bMoveable;
    
    /* check what window is calling this menu callback */
    if (bMainPnlFocus == FALSE)
        return;

    GetMenuBarAttribute (FnGetMainMenuHandle(),
                         MAIN_PANEL_ATTR_MOVABLE, ATTR_CHECKED,
                         &bMoveable);
    
    SetPanelAttribute (FnGetCurrentWindowHandle(), ATTR_MOVABLE,
                       !bMoveable);
    
    SetMenuBarAttribute (FnGetMainMenuHandle(),
                         MAIN_PANEL_ATTR_MOVABLE, ATTR_CHECKED,
                         !bMoveable);
}

/* 
 * Parent attribute
 */
void CVICALLBACK FnPanelAttrParent (int menuBar, int menuItem, void *callbackData,
        int panel)
{
    BOOLEAN bParent;
    WIN *pWINItem;
    PNL *pPNLItem;
    UIR *pUIRItem;
    HANDLE hPanel;
    HANDLE hChild;
    
    /* check what window is calling this menu callback */
    if (bMainPnlFocus == FALSE)
        return;

    pWINItem = FnGetCurrentWINItem();
    pPNLItem = pWINItem->pPNL;
    
    GetMenuBarAttribute (FnGetMainMenuHandle(),
                         MAIN_PANEL_ATTR_PARENT, ATTR_CHECKED,
                         &bParent);
    SetMenuBarAttribute (FnGetMainMenuHandle(),
                         MAIN_PANEL_ATTR_PARENT, ATTR_CHECKED,
                         !bParent);
    FnWindow (pPNLItem->hPanel, EVENT_CLOSE, NULL, 0, 0);
    
    pUIRItem = pPNLItem->pUIR;
    if (!bParent) { 
        hPanel = LoadPanel (0, pUIRItem->pszName, pPNLItem->iID);
        SetPanelAttribute (hPanel, ATTR_TOP, VAL_AUTO_CENTER);
        SetPanelAttribute (hPanel, ATTR_LEFT, VAL_AUTO_CENTER);
        FnTurnOffMenuBarShortCutKeys(hPanel);
    }
    else {
        hChild = FnGetChildPanelHandle(TRUE);
        hPanel = LoadPanel (hChild, pUIRItem->pszName, pPNLItem->iID);
        SetPanelAttribute (hPanel, ATTR_TOP, 30);
        SetPanelAttribute (hPanel, ATTR_LEFT, 10);
        FnTurnOffMenuBarShortCutKeys(hPanel);
    }  
    pPNLItem->hPanel = hPanel;
    SetPanelAttribute (hPanel, ATTR_FLOATING, 0);
    SetPanelAttribute (hPanel, ATTR_CALLBACK_FUNCTION_POINTER, *FnWindow);
    DisplayPanel(hPanel);
    
    pWINItem = (WIN *) malloc (sizeof(WIN));
    pWINItem->pPNL = pPNLItem;
    FnListAddItem(&WINList, pWINItem);
    FnPNLListBoxUpdate(); 
}

/*
 * Sizable attribute
 */
void CVICALLBACK FnPanelAttrSizable (int menuBar, int menuItem, void *callbackData,
        int panel)
{
    BOOLEAN bSizable;
    
    /* check what window is calling this menu callback */
    if (bMainPnlFocus == FALSE)
        return;

    GetMenuBarAttribute (FnGetMainMenuHandle(),
                         MAIN_PANEL_ATTR_SIZABLE, ATTR_CHECKED,
                         &bSizable);
    
    SetPanelAttribute (FnGetCurrentWindowHandle(), ATTR_SIZABLE,
                       !bSizable);
    
    SetMenuBarAttribute (FnGetMainMenuHandle(),
                         MAIN_PANEL_ATTR_SIZABLE, ATTR_CHECKED,
                         !bSizable);

}

/*
 * Panel title attribute
 */
void CVICALLBACK FnPanelAttrTitle (int menuBar, int menuItem, void *callbackData,
        int panel)
{
    char szPNLTitle[256];
    char szBuffer[512];
    WIN *pWINItem;
    PNL *pPNLItem;
    
    /* check what window is calling this menu callback */
    if (bMainPnlFocus == FALSE)
        return;

    pWINItem = FnGetCurrentWINItem();
    pPNLItem = pWINItem->pPNL;

    sprintf(szBuffer, "%s\n'%s'\n%s", "Change current Panel Title from:", pPNLItem->pszName,"to:");
    
    PromptPopup ("Change Panel Title", szBuffer, szPNLTitle, 255);
    SetPanelAttribute (FnGetCurrentWindowHandle(), ATTR_TITLE, szPNLTitle);
    
    free (pPNLItem->pszName);
    pPNLItem->pszName = (char *) malloc(sizeof(szPNLTitle));
    strcpy(pPNLItem->pszName, szPNLTitle);
    FnPNLListBoxInit(FnGetCurrentUIRItem());
    FnUpdateMenus();
}

/*
 * Title visible attribute
 */
void CVICALLBACK FnPanelAttrTitleBar (int menuBar, int menuItem, void *callbackData,
        int panel)
{
    BOOLEAN bTitle;
    
    /* check what window is calling this menu callback */
    if (bMainPnlFocus == FALSE)
        return;

    GetMenuBarAttribute (FnGetMainMenuHandle(),
                         MAIN_PANEL_ATTR_TITLE_BAR, ATTR_CHECKED,
                         &bTitle);
    
    SetPanelAttribute (FnGetCurrentWindowHandle(), ATTR_TITLEBAR_VISIBLE,
                       !bTitle);
    
    SetMenuBarAttribute (FnGetMainMenuHandle(),
                         MAIN_PANEL_ATTR_TITLE_BAR, ATTR_CHECKED,
                         !bTitle);
}

/*
 * System colors attribute
 */
void CVICALLBACK FnPanelAttrSystemColors (int menuBar, int menuItem, void *callbackData,
        int panel)
{
    BOOLEAN bColor;
    
    /* check what window is calling this menu callback */
    if (bMainPnlFocus == FALSE)
        return;

    GetMenuBarAttribute (FnGetMainMenuHandle(),
                         MAIN_PANEL_ATTR_SYSTEM_COLORS, ATTR_CHECKED,
                         &bColor);
    
    SetPanelAttribute (FnGetCurrentWindowHandle(), ATTR_CONFORM_TO_SYSTEM,
                       !bColor);
    
    SetMenuBarAttribute (FnGetMainMenuHandle(),
                         MAIN_PANEL_ATTR_SYSTEM_COLORS, ATTR_CHECKED,
                         !bColor);
}

/*
 * Close panel
 */
void CVICALLBACK FnPanelClose (int menuBar, int menuItem, void *callbackData,
        int panel)
{
    WIN *pWINItem;
    PNL *pPNLItem;
    
    /* check what window is calling this menu callback */
    if (bMainPnlFocus == FALSE)
        return;

    pWINItem = FnGetCurrentWINItem();
    pPNLItem = pWINItem->pPNL;
    
    FnWindow (pPNLItem->hPanel, EVENT_CLOSE, NULL, 0, 0);
}

/*
 * Change parent option
 */
void CVICALLBACK FnOptionsParent (int menuBar, int menuItem, void *callbackData,
        int panel)
{
    BOOLEAN bParent;
    
    /* check what window is calling this menu callback */
    if (bMainPnlFocus == FALSE)
        return;

    GetMenuBarAttribute (FnGetMainMenuHandle(),
                         MAIN_OPTIONS_PARENT, ATTR_CHECKED,
                         &bParent);
    
    SetMenuBarAttribute (FnGetMainMenuHandle(),
                         MAIN_OPTIONS_PARENT, ATTR_CHECKED,
                         !bParent);
}

/*
 * Change focus option
 */
void CVICALLBACK FnOptionsFocus (int menuBar, int menuItem, void *callbackData,
        int panel)
{
    BOOLEAN bFocus;
    
    /* check what window is calling this menu callback */
    if (bMainPnlFocus == FALSE)
        return;

    GetMenuBarAttribute (FnGetMainMenuHandle(),
                         MAIN_OPTIONS_FOCUS, ATTR_CHECKED,
                         &bFocus);
    
    SetMenuBarAttribute (FnGetMainMenuHandle(),
                         MAIN_OPTIONS_FOCUS, ATTR_CHECKED,
                         !bFocus);
}

/*
 * Help using UIR Viewer
 */
void CVICALLBACK FnHelpUIRViewer (int menuBar, int menuItem, void *callbackData,
        int panel)
{
    HANDLE hHelp;
    
    /* check what window is calling this menu callback */
    if (bMainPnlFocus == FALSE)
        return;

    hHelp = LoadPanel (0, UIRVIEWFILE, HELP);
    if (InstallPopup(hHelp) == 0) {     
        RunUserInterface();
        RemovePopup(0);
    }
}

/* 
 * About UIR Viewer
 */
void CVICALLBACK FnHelpAboutUIRViewer (int menuBar, int menuItem, void *callbackData,
        int panel)
{
    int hMain, hAboutHandle, status, control;
    
    /* check what window is calling this menu callback */
    if (bMainPnlFocus == FALSE)
        return;

    hAboutHandle = LoadPanel (0, UIRVIEWFILE, ABOUT);
    if (InstallPopup (hAboutHandle) == 0) {
        RunUserInterface();
        RemovePopup (0);
    }   
}

