#include <utility.h>
/*
 * This module contains the panel callbacks
 */

#include "Main.h"

/*
 * Main panel callback 
 */
int CVICALLBACK FnMainPnl (int panel, int event, void *callbackData,
        int eventData1, int eventData2)
{
    switch (event) {
        case EVENT_GOT_FOCUS:
            bMainPnlFocus = TRUE;
            break;
        case EVENT_LOST_FOCUS:
            bMainPnlFocus = FALSE;
            break;
        case EVENT_CLOSE:
            FnFileExit (0, 0, NULL, 0);
            break;
    }
    return 0;
}

/*
 * Displaed panels callback
 */
int CVICALLBACK FnWindow (int panel, int event, void *callbackData,
        int eventData1, int eventData2)
{
    int iIndex;
    WIN *pWINItem;
    HANDLE hParent;

    /* get the current top window */
    pWINItem = FnListTail(&WINList);

    switch (event) {
        case EVENT_GOT_FOCUS:
            if (pWINItem->pPNL->hPanel != panel) { 
                /* get the WIN item of the new top panel and place it at 
                the end of the list*/
                pWINItem = FnGetWINItemFromHandle(panel);
                pWINItem = FnListRemItem(&WINList, pWINItem);
                FnListAddItem(&WINList, pWINItem);
            }
            SetActivePanel(panel);
        
            /* set the list box items and menu */
            FnListBoxSetToCurrent();            
            FnUpdateMenus();
            FnUIRListBoxUpdate();
            FnPNLListBoxUpdate();
            break;
        case EVENT_CLOSE:
            /* remove top window from list */
            pWINItem = FnListRemItem(&WINList, pWINItem);
            
            /* see if panel was a child window */
            GetPanelAttribute (pWINItem->pPNL->hPanel, 
                    ATTR_PANEL_PARENT, &hParent);
                    
            /* close the window */
            DiscardPanel(pWINItem->pPNL->hPanel);
            pWINItem->pPNL->hPanel = 0;
            free(pWINItem);

            /* if so, close the parant if no other panels are using it */
            if (hParent > 0) {
                for (pWINItem = FnListHead(&WINList); pWINItem;
                        pWINItem = FnListNext(&WINList, pWINItem)) {
                    GetPanelAttribute (pWINItem->pPNL->hPanel, 
                            ATTR_PANEL_PARENT, &hParent);
                    if (hParent > 0)
                        break;
                }
                /* no windows are using it, so close child */
                if (!pWINItem)
                    FnCloseChildPanel();
            }
                
            /* set focus on next window */
            if ((pWINItem = FnListTail(&WINList)) != NULL) {
                SetActivePanel (pWINItem->pPNL->hPanel);
                
                FnListBoxSetToCurrent();            
            }
                
            /* set the list box items and menu */
            FnUpdateMenus();
            FnUIRListBoxUpdate();
            FnPNLListBoxUpdate();
            break;
        case EVENT_KEYPRESS:
            bMainPnlFocus = TRUE;
            if ((eventData1 & VAL_ASCII_KEY_MASK) != 0 
                        &&  (eventData1 & VAL_MENUKEY_MODIFIER) != 0) {
                switch (eventData1 | VAL_MENUKEY_MODIFIER) {
                    case ('O' | VAL_MENUKEY_MODIFIER):
                        FnFileOpen(0, 0, NULL, 0);      
                        break;
                    case ('W' | VAL_MENUKEY_MODIFIER):
                        FnFileClose(0, 0, NULL, 0);      
                        break;
                    case ('X' | VAL_MENUKEY_MODIFIER):
                        FnFileExit(0, 0, NULL, 0);      
                        break;
                    case ('C' | VAL_MENUKEY_MODIFIER):
                        FnPanelCopy(0, 0, NULL, 0);      
                        break;
                    case ('P' | VAL_MENUKEY_MODIFIER):
                        FnPanelPrint(0, 0, NULL, 0);      
                        break;
                    case ('S' | VAL_MENUKEY_MODIFIER):
                        FnPanelSave(0, 0, NULL, 0);      
                        break;
                    case ('D' | VAL_MENUKEY_MODIFIER):
                        FnPanelClose(0, 0, NULL, 0);      
                        break;
                }
            }
            else if ((eventData1 & VAL_F1_VKEY) != 0) 
                FnHelpUIRViewer (0, 0, NULL, 0);
            
            bMainPnlFocus = FALSE;
            break;  
    }
    return 0;
}


/*
 * Panel callback for about panel
 */
int CVICALLBACK FnAboutPanelCallback (int panel, int event, void *callbackData,
        int eventData1, int eventData2)
{
    if (event!=EVENT_GOT_FOCUS)
        QuitUserInterface (0);
    return 0;
}


/*
 * Function to disable user's panel from preventing panel shortcut keystrokes from occuring
 */
int CVIFUNC FnTurnOffMenuBarShortCutKeys (int panel)
{

    int menuHandle = 0;
    int menuCount = 0;
    int menuID = 0;
    int menuItemsNotFound = 0;
    int shortCutKey = 0;
    int dummy;
    int status;
    int prevState;
    
    menuHandle = GetPanelMenuBar (panel);
    if (menuHandle>0) 
    {
        if (GetMenuBarAttribute (menuHandle, 0, ATTR_NUM_MENUS, &menuCount))
            goto Error;
        if (menuCount>0)
        {
            prevState = SetBreakOnLibraryErrors (0);
            menuID = 1;            
            while (menuItemsNotFound<10)
            {
                status = GetMenuBarAttribute (menuHandle, menuID, ATTR_CONSTANT_NAME_LENGTH, &dummy);
                if (status==0) {
                    status = GetMenuBarAttribute (menuHandle, menuID, ATTR_SHORTCUT_KEY, &shortCutKey);
                    if ((shortCutKey = VAL_MENUKEY_MODIFIER | 'C') ||
                        (shortCutKey = VAL_MENUKEY_MODIFIER | 'P') ||
                        (shortCutKey = VAL_MENUKEY_MODIFIER | 'S') ||
                        (shortCutKey = VAL_MENUKEY_MODIFIER | 'D'))
                        {
                            status = SetMenuBarAttribute (menuHandle, menuID, ATTR_SHORTCUT_KEY, 0);
                        }
                }    
                else {
                    menuItemsNotFound++;
                }    
                menuID++;
            }
            SetBreakOnLibraryErrors (prevState);
        }
    }
        
    Error:
    return 0;    
}
