#include <utility.h>
/*
 * This module contains all the data manipulation functions
 */

#include <userint.h>
#include "Main.h"

/* 
 * Get main panle handle, load if not loaded
 */
HANDLE FnGetMainPanelHandle(BOOLEAN bForceOpen)
{
    static HANDLE hMain = 0;
    
    if (!hMain && bForceOpen) {
        hMain = LoadPanel (0, UIRVIEWFILE, MAIN_PNL);
        DisplayPanel (hMain);
    }
    return(hMain);
}

/* 
 * Get Child panle handle, load if not loaded
 */
HANDLE hChild = 0;

HANDLE FnGetChildPanelHandle(BOOLEAN bForceOpen)
{
    if (!hChild && bForceOpen) {
        hChild = LoadPanel (0, UIRVIEWFILE, CHILD_PNL);
        DisplayPanel (hChild);
    }
    return(hChild);
}

/*
 * Close child panel
 */
void FnCloseChildPanel(void)
{
    DiscardPanel(hChild);
    hChild = 0; 
}

/*
 * Get main menu bar handle
 */
HANDLE FnGetMainMenuHandle(void)
{
    static HANDLE hMenu = 0;
    
    if (!hMenu)
        hMenu = GetPanelMenuBar(FnGetMainPanelHandle(FALSE));
    
    return (hMenu);
}

HANDLE FnGetCurrentWindowHandle(void)
{
    WIN *pWINItem;
    
    pWINItem = FnListTail(&WINList);
    if (!pWINItem)
        return (0);
    else
        return (pWINItem->pPNL->hPanel);
}

/* 
 * Set the list box settings to reflect the currently slected panel
 */
void  FnListBoxSetToCurrent(void)
{
    int iOldUIRIndex;
    int iNewUIRIndex;
    int iPNLIndex;
    WIN *pWINItem;
    HANDLE hMain;
    
    if (!(pWINItem = FnListTail(&WINList))) {
        SetCtrlAttribute(FnGetMainPanelHandle(FALSE), MAIN_PNL_LB_PNL_SELECT,
                ATTR_HILITE_CURRENT_ITEM, 0);
        return;
    }
        
    /* set uir item */
    hMain = FnGetMainPanelHandle(FALSE);

    /* get uir index for current displayed panel */
    GetIndexFromValue (hMain, MAIN_PNL_LB_UIR_SELECT, &iNewUIRIndex,
                       pWINItem->pPNL->pUIR->pszName);
    if (iNewUIRIndex < 0)
        return;

    /* get the index that is currently displayed */
    GetCtrlIndex (hMain, MAIN_PNL_LB_UIR_SELECT, &iOldUIRIndex);
    
    if (iNewUIRIndex != iOldUIRIndex) {
        SetCtrlIndex (hMain, MAIN_PNL_LB_UIR_SELECT, iNewUIRIndex);
        FnPNLListBoxInit(pWINItem->pPNL->pUIR); 
    }
            
    /* set pnl item */
    GetIndexFromValue (hMain, MAIN_PNL_LB_PNL_SELECT, &iPNLIndex,
                       pWINItem->pPNL->iID);
                       
    if (iPNLIndex < 0)
        return;
        
    SetCtrlIndex (hMain, MAIN_PNL_LB_PNL_SELECT,
                        iPNLIndex);
}

/* 
 * Update the checks in the UIR list box
 */
void  FnUIRListBoxUpdate(void)
{
    UIR *pUIRCurrent;
    UIR *pUIRItem;
    int iIndex; 

    pUIRCurrent = FnGetCurrentUIRItem();
    
    for (iIndex = 0, pUIRItem = FnListTail(&UIRList); pUIRItem; 
            pUIRItem = FnListPrev(&UIRList, pUIRItem), iIndex++) {
        if (pUIRItem == pUIRCurrent)
            CheckListItem (FnGetMainPanelHandle(FALSE), MAIN_PNL_LB_UIR_SELECT, iIndex, 1);
        else
            CheckListItem (FnGetMainPanelHandle(FALSE), MAIN_PNL_LB_UIR_SELECT, iIndex, 0);
    }
}

/*
 * Set the PNL ring from the uir item
 */
void  FnPNLListBoxInit(UIR *pUIRItem)
{
    PNL* pPNLItem;
    HANDLE hMain;
    BOOLEAN itemInserted = FALSE;
    
    hMain = FnGetMainPanelHandle(FALSE);
    
    DeleteListItem (hMain, MAIN_PNL_LB_PNL_SELECT, 0, -1);
    SetCtrlAttribute (hMain, MAIN_PNL_LB_PNL_SELECT, ATTR_HILITE_CURRENT_ITEM, 0);
    
    if (!pUIRItem) {
        SetCtrlAttribute (hMain, MAIN_PNL_LB_PNL_SELECT, ATTR_HILITE_CURRENT_ITEM, 0);
        SetCtrlAttribute (hMain, MAIN_PNL_LB_PNL_SELECT, ATTR_DIMMED, 1);
        InsertListItem (hMain, MAIN_PNL_LB_PNL_SELECT, -1, "None", 0);
        return;
    }
    SetCtrlAttribute (hMain, MAIN_PNL_LB_PNL_SELECT, ATTR_DIMMED, 0);
    
    for (pPNLItem = FnListHead(&pUIRItem->PNLList); pPNLItem; 
        pPNLItem = FnListNext(&pUIRItem->PNLList, pPNLItem)) 
    { 
        InsertListItem (hMain, MAIN_PNL_LB_PNL_SELECT, -1, 
            pPNLItem->pszName, pPNLItem->iID);
        itemInserted = TRUE;
    }    
    
    FnPNLListBoxUpdate();
    
    if (itemInserted==TRUE)
        SetCtrlAttribute (hMain, MAIN_PNL_LB_PNL_SELECT, ATTR_HILITE_CURRENT_ITEM, 1);
}

/*
 * Update the checks in the PNL list box
 */
void FnPNLListBoxUpdate(void)
{
    int iIndex;
    BOOLEAN bPnlFound = FALSE;
    HANDLE hMain;
    UIR *pUIRItem;
    PNL *pPNLItem;
    WIN *pWINItem;
    
    /* get the current uir */
    pUIRItem = FnGetCurrentUIRItem();
    
    if (!pUIRItem)
        return;
    
    hMain = FnGetMainPanelHandle(FALSE);
    
    for (iIndex = 0, pPNLItem = FnListHead(&pUIRItem->PNLList); pPNLItem;
            pPNLItem = FnListNext(&pUIRItem->PNLList, pPNLItem), iIndex++)
        if (pPNLItem->hPanel) 
            CheckListItem (hMain, MAIN_PNL_LB_PNL_SELECT, iIndex, 1);
        else 
            CheckListItem (hMain, MAIN_PNL_LB_PNL_SELECT, iIndex, 0);

    pWINItem = FnGetCurrentWINItem();
    if (pWINItem && pWINItem->pPNL->pUIR == pUIRItem) {
        for (iIndex = 0, pPNLItem = FnListHead(&pUIRItem->PNLList); pPNLItem && !bPnlFound;
                pPNLItem = FnListNext(&pUIRItem->PNLList, pPNLItem), iIndex++) {
            if (pPNLItem == pWINItem->pPNL) { 
                bPnlFound = TRUE;
                SetCtrlIndex (hMain, MAIN_PNL_LB_PNL_SELECT, iIndex);
                SetCtrlAttribute (hMain, MAIN_PNL_LB_PNL_SELECT,
                    ATTR_HILITE_CURRENT_ITEM, 1);
            }   
        }
    }
    if (!bPnlFound)
        SetCtrlAttribute (FnGetMainPanelHandle(FALSE), MAIN_PNL_LB_PNL_SELECT,
                ATTR_HILITE_CURRENT_ITEM, 0); 
}

/*
 * Close all the panels associated with this UIR
 */
void FnCloseAllIUIRPanels(UIR *pUIRItem)
{
    WIN *pWINItem;
    HANDLE hParent;
    BOOLEAN bTop = FALSE;
    
    pWINItem = FnListTail(&WINList);
    
    if (pWINItem == NULL) 
        return;
    
    if (pWINItem->pPNL->pUIR == pUIRItem) {
        bTop = TRUE;
        pWINItem = FnListPrev(&WINList, pWINItem);
    }

    while (pWINItem) {
        if (pWINItem->pPNL->pUIR == pUIRItem) {
            WIN *pWINPrev;
            
            pWINPrev = FnListPrev(&WINList, pWINItem);
            
            /* remove top window from list */
            pWINItem = FnListRemItem(&WINList, pWINItem);

            /* see if panel was a child window */
            GetPanelAttribute (pWINItem->pPNL->hPanel, 
                    ATTR_PANEL_PARENT, &hParent);
            
            /* close the window */
            DiscardPanel(pWINItem->pPNL->hPanel);
            pWINItem->pPNL->hPanel = 0;
            free(pWINItem);

            /* if so, close the parant if no other panels are using it */
            if (hParent > 0) {
                for (pWINItem = FnListHead(&WINList); pWINItem;
                        pWINItem = FnListNext(&WINList, pWINItem)) {
                    GetPanelAttribute (pWINItem->pPNL->hPanel, 
                            ATTR_PANEL_PARENT, &hParent);
                    if (hParent > 0)
                        break;
                }
                /* no windows are using it, so close child */
                if (!pWINItem)
                    FnCloseChildPanel();
            }

            pWINItem = pWINPrev;
        }
        else
            pWINItem = FnListPrev(&WINList, pWINItem);
    }

    if (bTop) {
        pWINItem = FnListTail(&WINList);        
        FnWindow (pWINItem->pPNL->hPanel, EVENT_CLOSE, NULL, 0, 0);
    }
}


 /*
 * Close all the panels associated with this UIR
 */
/*void FnCloseAllIUIRPanels(UIR *pUIRItem)
{
    WIN *pWINItem;
    
    pWINItem = FnListTail(&WINList);

    while (pWINItem) {
        if (pWINItem->pPNL->pUIR == pUIRItem) {
            WIN *pWINPrev;
            
            pWINPrev = FnListPrev(&WINList, pWINItem); 
            FnWindow (pWINItem->pPNL->hPanel, EVENT_GOT_FOCUS, NULL, 0, 0);
            FnWindow (pWINItem->pPNL->hPanel, EVENT_CLOSE, NULL, 0, 0);
            pWINItem = pWINPrev;
        }
        else
            pWINItem = FnListPrev(&WINList, pWINItem);
    }
}
*/

/*
 * View panel with this ID in this UIR
 */
void FnDisplayNewPanel(PNL *pPNLItem) 
{
    BOOLEAN bOPE;
    BOOLEAN bOLE;
    int iCtrlID;
    int iCtrlIDStatus;
    HANDLE hMenu;
    int iMenuItem;
    int iMenuItemStatus;
    WIN *pWINItem;
    UIR *pUIRItem;
    HANDLE hPanel;
    HANDLE hChild;
    HANDLE hMain;
    BOOLEAN bParent;
    
    pUIRItem = pPNLItem->pUIR;

    GetMenuBarAttribute (FnGetMainMenuHandle(), MAIN_OPTIONS_PARENT,
                         ATTR_CHECKED, &bParent);
    
    if (bParent) { 
        hPanel = LoadPanel (0, pUIRItem->pszName, pPNLItem->iID);
        SetPanelAttribute (hPanel, ATTR_TOP, VAL_AUTO_CENTER);
        SetPanelAttribute (hPanel, ATTR_LEFT, VAL_AUTO_CENTER);
        FnTurnOffMenuBarShortCutKeys(hPanel);
    }
    else {
        hChild = FnGetChildPanelHandle(TRUE);
        hPanel = LoadPanel (hChild, pUIRItem->pszName, pPNLItem->iID);
        SetPanelAttribute (hPanel, ATTR_TOP, 30);
        SetPanelAttribute (hPanel, ATTR_LEFT, 10);
        FnTurnOffMenuBarShortCutKeys(hPanel);
    }  
    pPNLItem->hPanel = hPanel;
    SetPanelAttribute (hPanel, ATTR_FLOATING, 0);
    SetPanelAttribute (hPanel, ATTR_CALLBACK_FUNCTION_POINTER, *FnWindow);
    
    /* set ctrl and menu calbacks to null */
    hMain = FnGetMainPanelHandle(FALSE);
    bOPE = SetBreakOnProtectionErrors (0);
    bOLE = SetBreakOnLibraryErrors (0);

    iCtrlID = 2;
    do {
        iCtrlIDStatus = SetCtrlAttribute (hPanel, iCtrlID++,
                                    ATTR_CALLBACK_FUNCTION_POINTER, NULL);
    } while (iCtrlIDStatus >= 0);   
    
    SetBreakOnProtectionErrors (bOPE);
    SetBreakOnLibraryErrors (bOLE);

    DisplayPanel(hPanel);
    
    pWINItem = (WIN *) malloc (sizeof(WIN));
    pWINItem->pPNL = pPNLItem;
    FnListAddItem(&WINList, pWINItem);
}


/*
 * Get the current UIR Item
 */
UIR *FnGetCurrentUIRItem(void)
{
    static char szUIRName[MAX_PATHNAME_LEN];

    GetCtrlVal(FnGetMainPanelHandle(FALSE), MAIN_PNL_LB_UIR_SELECT, szUIRName);
    return(FnGetUIRItemFromName(szUIRName));
}

/*
 * Get the current PNL Item
 */
PNL *FnGetCurrentPNLItem(void)
{
    int iID;
    UIR *pUIRItem;

    /* get the current UIR */
    pUIRItem = FnGetCurrentUIRItem();
    /* get the current PNL ID */
    GetCtrlVal(FnGetMainPanelHandle(FALSE), MAIN_PNL_LB_PNL_SELECT, &iID);
    /* get the PNL item from the UIR and ID */
    if (iID) {
        PNL *pPNLItem;
    
        for (pPNLItem = FnListHead(&pUIRItem->PNLList); pPNLItem; 
                pPNLItem = FnListNext(&pUIRItem->PNLList, pPNLItem))
            if (pPNLItem->iID == iID)
                break;
                
        return(pPNLItem);
    }
    else
        return(NULL);
}

/*
 * Get the current WIN Item
 */
WIN *FnGetCurrentWINItem(void)
{
    return(FnListTail(&WINList));
}

/*
 * Return the WIN linked list item from the PNL handle 
 */
WIN *FnGetWINItemFromHandle(HANDLE hPanel)
{
    WIN *pWINItem;
    
    for (pWINItem = FnListHead(&WINList); pWINItem; 
            pWINItem = FnListNext(&WINList, pWINItem))
        if (pWINItem->pPNL->hPanel == hPanel)
                break;

    return(pWINItem);
}

/*
 * Return the UIR linked list item from the UIR name 
 */
UIR *FnGetUIRItemFromName(char *pszName)
{
    UIR *pUIRItem;
    
    for (pUIRItem = FnListHead(&UIRList); pUIRItem; 
            pUIRItem = FnListNext(&UIRList, pUIRItem)) 
        if (!strncmp(pUIRItem->pszName, pszName, 
                MAX_PATHNAME_LEN))
            break;
            
    return(pUIRItem);
}

/* 
 * Free allocated memory in uir item
 */
void  FnFreeUIRItem(UIR *pUIRItem)
{
    // Free allocated data from uir item
    if (pUIRItem) {
        PNL *pPNLItem;
        
        // Free allocated data from pnl linked list
        while (pPNLItem = FnListHead(&pUIRItem->PNLList)) 
            FnFreePNLItem(FnListRemItem(&pUIRItem->PNLList, pPNLItem));
    
        free(pUIRItem->pszName);
        free(pUIRItem);
    }
}

/* 
 * Free allocated memory in pnl item
 */
void  FnFreePNLItem(PNL *pPNLItem)
{
    // Free allocated data from uir item
    if (pPNLItem) {
        free(pPNLItem->pszName);
        free(pPNLItem);
    }
}
